/**
 * Last Modified by: $Author: teskridg $
 * On: $Date: 2007/05/09 20:40:37 $
 */
package coi.cmap;

import coi.cmap.monitor.MonitorMapHelper;
import coi.cmap.monitor.MonitorMapViewer;
import core.base.*;
import core.event.MenuItemChosenEvt;
import core.event.MenuItemEvt;
import core.event.PopupOpeningEvt;
import core.exception.InvalidParameterXcp;
import core.exception.InvalidWndTypeXcp;
import core.exception.NoReplyEvtXcp;
import core.util.CoreOptionPane;
import core.util.CoreProgressDialog;
import core.util.EventSynchronizer;
import nlk.base.*;
import nlk.cmap.CMapEditMod;
import nlk.cmap.CMapWnd;
import nlk.code.Constants;
import nlk.code.TemplateMod;
import nlk.code.owlimport.NameSpaceTable;
import nlk.event.folderBrowser.GetSelectedViewNodeEvt;
import nlk.event.folderBrowser.GetSelectedViewNodeRepEvt;
import nlk.event.folderBrowser.ViewNodeDeselectedEvt;
import nlk.event.folderBrowser.ViewNodeSelectedEvt;
import nlk.event.resourceManager.*;
import nlk.exception.resio.ResioXcp;
import nlk.indexer.CMQuery;
import nlk.indexer.ResourceSearchResult;
import nlk.indexer.ShowResult;
import nlk.resio.PrivServer;
import nlk.util.*;

import javax.swing.*;
import java.io.File;
import java.io.IOException;
import java.util.*;

/**
 * Models a class that handles importing/adding and modifying
 * resources for both the FolderBrowser and concept maps
 */
public class COIMod implements Module
{
    public static final String SPECIFIC_TYPE = "Domain Specific";
    public static final String IMPLEMENTED_TYPE = "Implemented";
    public static final String GENERIC_TYPE = "Abstract";
    
    public static final String COI_ACTOR = "COIActor";
    public static final String COI_ACTION = "COIAction";
    public static final String COI_DATA = "COIData";
    public static final String COI = "COI";
    public static final String IHMC_ONTOLOGY_URL = "http://ontology.ihmc.us/";
    
    public static final String COI_MAP_TYPE = "ontologyType";
    
    public static final String SPECIFIC_COIS = SPECIFIC_TYPE + " COIs";
    public static final String IMPLEMENTED_COIS = IMPLEMENTED_TYPE + " COIs";
    public static final String GENERIC_COIS = GENERIC_TYPE + " COIs";
    public static final String NEW_SPECIFIC_COI = "New " + SPECIFIC_TYPE + " COI";
    public static final String NEW_IMPLEMENTED_COI = "New " + IMPLEMENTED_TYPE + " COI";

    public static final String SPECIFIC_COI = SPECIFIC_TYPE + " COI";
    public static final String IMPLEMENTED_COI = IMPLEMENTED_TYPE + " COI";

    public static final String MONITOR_IMPL_COI = "Monitor " + IMPLEMENTED_TYPE + " COI";

    // these are just used for menuitems and should not change
    private static final String FILE_NEW_SPECIFIC_COI_MI = "File/NewSpecificCOI";
    private static final String FILE_NEW_IMPLEMENTED_COI_MI = "File/NewImplementedCOI";
    private static final String NEW_SPECIFIC_COI_MI = "New/SpecificCOI";
    private static final String NEW_IMPLEMENTED_COI_MI = "New/ImplementedCOI";
    private static final String MONITOR_IMPL_COI_MI = "New/MonitorCOI";

    public static final String GENERIC_COI_REF = "genericCOIRef";
    public static final String SPECIFIC_COI_REF = "specificCOIRef";

    public static final String MONITOR_MAP_URL = "monitorMapURL";
    
    private Vector _mapMonitorHelpers = new Vector();
    private static final String MONITOREDFOLDER = "monitored sessions";

    /**
     * Constructs a the COIMod
     */
    public COIMod()
    {
        // initializing internal state variables
        _thisModule = this;
        _moduleInfo = "Communities Of Interest Module";
        _version = "$Revision: 1.33 $";
        _description = "Handles support for Communities of Interest project";
        _authorEmailAddress = "jlott@ihmc.us";
    }

    public static COIMod getCOIMod()
    {
        return _thisModule;
    }

    /**
     * @see core.base.Module
     */
    public void initialize(Core core) throws Exception
    {
        if (core == null)
            throw new NullPointerException("core");
        
        _core = core;
        _ff = new FolderFetcher(_core, _thisModule);
    }

    /**
     * @see core.base.Module
     */
    public String getName()
    {
        return _moduleInfo;
    }

    /**
     * @see core.base.Module
     */
    public String getVersion()
    {
        return _version;
    }

    /**
     * @see core.base.Module
     */
    public String getDescription()
    {
        return _moduleInfo;
    }

    /**
     * @see core.base.Module
     */
    public void registerDSTypes() throws Exception
    {
    }

    /**
     * @see core.base.Module
     */
    public void registerPopupTypes() throws Exception
    {
    }

    /**
     * @see core.base.Module
     */
    public void addMenus() throws Exception
    {
        MenuParams menuParams;
        MenuItemParams menuItemParams;

        // add 'New/SpecificCOI'
        menuItemParams = new MenuItemParams(NEW_SPECIFIC_COI, 1);
        _core.addWndMenuItem(this, "FolderBrowserWnd", FILE_NEW_SPECIFIC_COI_MI, menuItemParams);

        // add 'New/ImplementedCOI'
        menuItemParams = new MenuItemParams(NEW_IMPLEMENTED_COI, 2);
        _core.addWndMenuItem(this, "FolderBrowserWnd", FILE_NEW_IMPLEMENTED_COI_MI, menuItemParams);

        // add 'New/SpecificCOI' to popup menu
        menuItemParams = new MenuItemParams(SPECIFIC_COI, 1);
        _core.addPopupMenuItem(this, "FolderBrowserWndPopup", NEW_SPECIFIC_COI_MI, menuItemParams);
        // add 'New/SpecificCOI' to popup menu
        menuItemParams = new MenuItemParams(IMPLEMENTED_COI, 2);
        _core.addPopupMenuItem(this, "FolderBrowserWndPopup", NEW_IMPLEMENTED_COI_MI, menuItemParams);

        menuItemParams = new MenuItemParams(MONITOR_IMPL_COI, 3);
        _core.addPopupMenuItem(this, "FolderBrowserWndPopup", MONITOR_IMPL_COI_MI, menuItemParams);

        // configure menu items for initial startup
        _core.registerForWndMenuItemEnableVoting(this, "FolderBrowserWnd", FILE_NEW_SPECIFIC_COI_MI, false);
        _core.registerForPopupMenuItemEnableVoting(this, "FolderBrowserWndPopup", NEW_SPECIFIC_COI_MI, false);
        _core.registerForWndMenuItemEnableVoting(this, "FolderBrowserWnd", FILE_NEW_IMPLEMENTED_COI_MI, false);
        _core.registerForPopupMenuItemEnableVoting(this, "FolderBrowserWndPopup", NEW_IMPLEMENTED_COI_MI, false);
        _core.registerForPopupMenuItemEnableVoting(this, "FolderBrowserWndPopup", MONITOR_IMPL_COI_MI, false);


        menuParams = new MenuParams("KAoS",1500);
        _core.addPopupMenu(this, "CMapWndConceptPopup",           "KAoS", menuParams);
        _core.addPopupMenu(this, "CMapWndLinkingPhrasePopup",           "KAoS", menuParams);
        _core.addPopupMenu(this, "CMapWndMultipleEntitiesPopup", "KAoS", menuParams);

        menuParams = new MenuParams("Bootstrap", 1);
        _core.addPopupMenu(this, "CMapWndConceptPopup",           "KAoS/bs", menuParams);
        _core.addPopupMenu(this, "CMapWndMultipleEntitiesPopup", "KAoS/bs", menuParams);
//        _core.addPopupMenu(this, "CMapWndConceptPopup",           "KAoS/Bootstrap", menuParams);
//        _core.addPopupMenu(this, "CMapWndMultipleEntitiesPopup", "KAoS/Bootstrap", menuParams);

        menuItemParams = new MenuItemParams("Show Bootstrap File", 2);
        _core.addPopupMenuItem(this, "CMapWndConceptPopup",            "KAoS/bs/ShowBootstrapFile", menuItemParams);
        _core.addPopupMenuItem(this, "CMapWndMultipleEntitiesPopup",  "KAoS/bs/ShowBootstrapFile", menuItemParams);

        menuItemParams = new MenuItemParams("Show Policies", 3);
        _core.addPopupMenuItem(this, "CMapWndConceptPopup",            "KAoS/ShowPolicies", menuItemParams);
        _core.addPopupMenuItem(this, "CMapWndMultipleEntitiesPopup",  "KAoS/ShowPolicies", menuItemParams);

//        menuItemParams = new MenuItemParams("Show Information",4);
//        _core.addPopupMenuItem(this, "CMapWndConceptPopup",            "KAoS/ShowInfo", menuItemParams);
//        _core.addPopupMenuItem(this, "CMapWndMultipleEntitiesPopup",  "KAoS/ShowInfo", menuItemParams);

        menuItemParams = new MenuItemParams("Show Relations",5);
        _core.addPopupMenuItem(this, "CMapWndConceptPopup",            "KAoS/ShowRelations", menuItemParams);
        _core.addPopupMenuItem(this, "CMapWndMultipleEntitiesPopup",  "KAoS/ShowRelations", menuItemParams);

        menuItemParams = new MenuItemParams("Show Relation",5);
        _core.addPopupMenuItem(this, "CMapWndLinkingPhrasePopup",     "KAoS/ShowRelation", menuItemParams);


        menuItemParams = new MenuItemParams("Show KAoS Connection",5);
        _core.addPopupMenuItem(this, "CMapWndNoSelectionBackgroundPopup",     "ShowConnection", menuItemParams);

        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndConceptPopup", "KAoS", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndConceptPopup", "KAoS/bs/ShowBootstrapFile", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndConceptPopup", "KAoS/ShowPolicies", false);
//        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndConceptPopup", "KAoS/ShowInfo", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndConceptPopup", "KAoS/ShowRelations", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndLinkingPhrasePopup", "KAoS/ShowRelation", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndMultipleEntitiesPopup", "KAoS", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndMultipleEntitiesPopup", "KAoS/bs/ShowBootstrapFile", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndMultipleEntitiesPopup", "KAoS/ShowPolicies", false);
//        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndMultipleEntitiesPopup", "KAoS/ShowInfo", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndMultipleEntitiesPopup", "KAoS/ShowRelations", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndNoSelectionBackgroundPopup", "ShowConnection", false);



    }

    /**
     * @see core.base.Module                                                                         1
     */
    public void addEvtHandlers() throws Exception
    {
        MenuItemChosenEvt mice = new MenuItemChosenEvt();

        _core.registerForPopupMenuItemEvt(this,
                "CMapWndConceptPopup",
                "KAoS/bs/ShowBootstrapFile",
                mice,
                this,
                "handleKAoSEvt",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);

        _core.registerForPopupMenuItemEvt(this,
                "CMapWndConceptPopup",
                "KAoS/ShowPolicies",
                mice,
                this,
                "handleKAoSEvt",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);

//        _core.registerForPopupMenuItemEvt(this,
//                "CMapWndConceptPopup",
//                "KAoS/ShowInfo",
//                mice,
//                this,
//                "handleKAoSEvt",
//                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        _core.registerForPopupMenuItemEvt(this,
                "CMapWndConceptPopup",
                "KAoS/ShowRelations",
                mice,
                this,
                "handleKAoSEvt",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);

        _core.registerForPopupMenuItemEvt(this,
                "CMapWndLinkingPhrasePopup",
                "KAoS/ShowRelation",
                mice,
                this,
                "handleKAoSEvt",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);

        _core.registerForPopupMenuItemEvt(this,
                "CMapWndNoSelectionBackgroundPopup",
                "ShowConnection",
                mice,
                this,
                "handleShowConnection",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);


        // add event handlers for menu item events
        _core.registerForWndMenuItemEvt(this,
                "FolderBrowserWnd",
                "File/New/SpecificCOI",
                mice,
                this,
                "handleNewCOI",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        
        // add event handlers for menu item events
        _core.registerForWndMenuItemEvt(this,
                "FolderBrowserWnd",
                "File/New/ImplementedCOI",
                mice,
                this,
                "handleNewCOI",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        
        // add event for 
        _core.registerForWndMenuItemEvt(this,
                "CMapWnd",
                "File/SaveMap",
                mice,
                this,
                "handleSaveCmap",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        
        // if this is posted on the swing thread, we will handle it in the swing thread
        _core.registerForModuleEvt(this,
                new ViewNodeSelectedEvt(),
                this,
                "handleViewNodeSelected",
                Core.EXECUTE_SYNC | Core.ANY_THREAD);

        // if this is posted on the swing thread, we will handle it in the swing thread
        _core.registerForModuleEvt(this,
                new ViewNodeDeselectedEvt(),
                this,
                "handleViewNodeDeselected",
                Core.EXECUTE_SYNC | Core.ANY_THREAD);

        _core.registerForPopupMenuItemEvt(this,
                "FolderBrowserWndPopup",
                NEW_SPECIFIC_COI_MI,
                mice,
                this,
                "handleNewCOI",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        
        _core.registerForPopupMenuItemEvt(this,
                "FolderBrowserWndPopup",
                NEW_IMPLEMENTED_COI_MI,
                mice,
                this,
                "handleNewCOI",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        
        _core.registerForPopupMenuItemEvt(this,
                "FolderBrowserWndPopup",
                MONITOR_IMPL_COI_MI,
                mice,
                this,
                "handleMonitorCOI",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        
        _core.registerForPopupEvt(this,
                "FolderBrowserWndPopup",
                new PopupOpeningEvt(),
                this,
                "handlePopupOpenEvts",
                Core.EXECUTE_SYNC | Core.ANY_THREAD);

        _core.registerForPopupEvt(this,
                 "CMapWndConceptPopup",
                 new PopupOpeningEvt(),
                 this,
                 "handleConceptPopupOpenEvts",
                 Core.EXECUTE_SYNC | Core.ANY_THREAD);

        _core.registerForPopupEvt(this,
                 "CMapWndMultipleEntitiesPopup",
                 new PopupOpeningEvt(),
                 this,
                 "handleConceptPopupOpenEvts",
                 Core.EXECUTE_SYNC | Core.ANY_THREAD);

        _core.registerForPopupEvt(this,
                 "CMapWndLinkingPhrasePopup",
                 new PopupOpeningEvt(),
                 this,
                 "handleLinkingPhrasePopupOpenEvts",
                 Core.EXECUTE_SYNC | Core.ANY_THREAD);

        _core.registerForPopupEvt(this,
                 "CMapWndNoSelectionBackgroundPopup",
                 new PopupOpeningEvt(),
                 this,
                 "handleBackgroundPopupOpenEvts",
                 Core.EXECUTE_SYNC | Core.ANY_THREAD);

        Module _CMapEditMod = _core.getModule("nlk.cmap.CMapEditMod");
        if (_CMapEditMod != null)
        {
            _core.unregisterForWndMenuItemEvt(_CMapEditMod, 
                     "CMapWnd", 
                     "Edit/Properties",
                    mice,
                     _CMapEditMod, 
                     "handleMenuItemChosenEvents");
            
            _core.registerForWndMenuItemEvt (this, 
                    "CMapWnd", 
                    "Edit/Properties",
                    mice,
                    this, 
                    "handleTemplateProperties", 
                    Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        }
    }

    /**
     * @see core.base.Module
     */
    public void addDSInstances() throws Exception
    {
    }

    /**
     * @see core.base.Module
     */
    public void getFromCore() throws Exception
    {
    }

    /**
     * @see core.base.Module
     */
    public void initCompleted()
    {
        _cmapSettings = (CMapSettings) _core.getAppSettings();

        PartWriterRegistrationEvt pwre = new PartWriterRegistrationEvt(this,
                "x-cmap/x-storable",
                "ontologyPartWriter",
                new OntologyPartWriter(),
                0);
        try
        {
            _core.postModuleEvt(pwre);
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

        try
        {
            _core.registerWndType("SearchWnd", "coi.cmap.monitor.PolicyViewer");
        }
        catch (InvalidWndTypeXcp invalidWndTypeXcp)
        {
            invalidWndTypeXcp.printStackTrace();
        }
        catch (ClassNotFoundException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * @see core.base.Module
     */
    public boolean okToTerminate()
    {
        return true;
    }

    /**
     * @see core.base.Module
     */
    public void preliminaryCleanup() throws Exception
    {

    }

    /**
     * @see core.base.Module
     */
    public void finalCleanup()
    {

    }

    /**
     * @see core.base.Module
     */
    public String getAuthorAddress()
    {
        return _authorEmailAddress;
    }

    public void handleKAoSEvt( MenuItemEvt evt ) throws Exception
    {
        CorePopup cpopup = (CorePopup) evt.menuContainer;
        CMapComponent cmc = (CMapComponent) cpopup.getPopupComponent();
        GraphicalConceptMap gcm = cmc.getGraphicalConceptMapRef();
        ConceptMap cm = gcm.getConceptMapRef();
        MonitorMapViewer mmv = (MonitorMapViewer) cm.getTrans("mmv");
        if ( mmv != null )
        {
            mmv.handleKAoSEvt(evt);
        }
    }


    /**
     * Event handler that handles the "Edit/Properties..." menuitem
     *
     * @param evt the menu item event
     * @throws Exception
     */
    public void handleTemplateProperties(MenuItemEvt evt) throws Exception
    {
        // First try to call the original menue item handler
        CMapEditMod _CMapEditMod = (nlk.cmap.CMapEditMod)_core.getModule("nlk.cmap.CMapEditMod");
        
        if (_CMapEditMod != null)
            _CMapEditMod.handleMenuItemChosenEvents(evt);
        
        // Check if the menue item is property editing
        if (evt.menuSpec.equalsIgnoreCase ("Edit/Properties")) 
        {
            CMapComponent cmc = ((CMapComponentWndInterface) evt.menuContainer).getCMapComponentInFocus();
            CoreWnd cwnd = (CoreWnd) evt.menuContainer;
            
            // try to obtain resource info
            ResourceInfo ri = null;
            try 
            {
                ri = cmc.getGraphicalConceptMapRef().getConceptMapRef().getResourceInfoRef();
            }
            catch (Exception xcp) 
            {
                xcp.printStackTrace();
            }
            
            if (ri == null)
            {
                // TODO: Complain that there is no resource Info.
                return;
            }
            
            // Check if the resource is a template
            if ( ri.getResourceMIMETypeRef().equals(TemplateMod.MIMETYPE_TEMPLATE))
            {
                Enumeration folderBrowserInstances = _core.getAllWndInstances("FolderBrowserWnd");
                CoreWnd fbWnd = (CoreWnd) folderBrowserInstances.nextElement();

                ResourceReference _sourceMapRef = (ResourceReference)ri.get(TemplateMod.SOURCE_ONTOLOGY_KEY);
                String sourceMapID ="";
                if (_sourceMapRef != null)
                    sourceMapID =_sourceMapRef.getResourceId().toString();
                
                String templateOntologyType = (String)ri.get(COI_MAP_TYPE);
                if (templateOntologyType == null)
                    templateOntologyType = COIMod.SPECIFIC_TYPE;
                
                TemplatePropertyLDlg _TemplatePropertyLDlg = new TemplatePropertyLDlg(fbWnd.getDialogParentFrame(), this, sourceMapID, templateOntologyType);

                _TemplatePropertyLDlg.pack();
                _TemplatePropertyLDlg.setVisible(true);

                if (_TemplatePropertyLDlg.wasOkayed())
                {
                    sourceMapID = _TemplatePropertyLDlg.getSourceMapID();
                    templateOntologyType = _TemplatePropertyLDlg.getTemplateOntologyType();
                    
                    if (sourceMapID != null)
                    {
                        ResourceReference newSourceMapRef;
                        if (_sourceMapRef != null)
                            newSourceMapRef = new ResourceReference(_sourceMapRef.getServerId(), _sourceMapRef.getProjectId(), new ObjectId(sourceMapID));
                        else
                            newSourceMapRef = new ResourceReference(new ObjectId(""), new ObjectId(""), new ObjectId(sourceMapID));
                        
                        ri.put(TemplateMod.SOURCE_ONTOLOGY_KEY, newSourceMapRef);
                    }
                    
                    if (templateOntologyType != null)
                        if (templateOntologyType.equals(COIMod.SPECIFIC_TYPE) || templateOntologyType.equals(COIMod.IMPLEMENTED_TYPE))
                            ri.put(COI_MAP_TYPE, templateOntologyType);
                }
            }
        }
    }
    
    /**
     * Event handler that handles Save menu items selections
     *
     * @param evt the event
     * @throws Exception
     */
    public void handleSaveCmap(BaseEvt evt) throws Exception
    {
        GetSelectedViewNodeRepEvt repEvt = getSelectedViewNode();

        ResourceInfo selectedRI = repEvt._resourceInfo;
        ResourceReference selectedRIRef = new ResourceReference(selectedRI.getCachedServerId(), selectedRI.getCachedProjectId(), selectedRI.getResourceId());
        
        if (selectedRI != null)
        {
          
            if (selectedRI.getResourceName().equals(COI_ACTOR) || selectedRI.getResourceName().equals(COI_ACTION)
                    || selectedRI.getResourceName().equals(COI_DATA) || selectedRI.getResourceName().equals(COI))
            {
                String parentMapURL = IHMC_ONTOLOGY_URL + "COI/" + selectedRI.getResourceName() + ".owl";
                
                DynamicTemplateGenerator.generateTemplets(true, parentMapURL, selectedRIRef, _core, _thisModule);
                DynamicTemplateGenerator.generateTemplets(false, parentMapURL, selectedRIRef, _core, _thisModule);
            }
            else if (selectedRI.get(COI_MAP_TYPE) != null)
            {
                if (selectedRI.get(COI_MAP_TYPE).equals(IMPLEMENTED_TYPE))
                    return;
                    
                String parentMapURL = FolderFetcher.getURLToResource(selectedRI.getCachedServerId(), selectedRI.getResourceId()) + "&partName=owl";
                
                DynamicTemplateGenerator.generateTemplets(true, parentMapURL, selectedRIRef, _core, _thisModule);
                DynamicTemplateGenerator.generateTemplets(false, parentMapURL, selectedRIRef, _core, _thisModule);
            }
        }
    }

    private ResourceInfo copyAndRecord( final ResourceInfo origRI )
    {
        final ObjectId sid = origRI.getCachedServerId();
        final ObjectId pid = origRI.getCachedProjectId();
        ObjectId foundFolderId = null;

        // get user info for copy and/or folder creation
        UserInfo userInfo = UserInfo.getUserInfo((CMapSettings) _core.getAppSettings());
        if ( userInfo == null )
        {
            CoreOptionPane.showMessage("Error Monitoring Map","Could not get user information", CoreOptionPane.ERROR);
            return null;
        }
        String creator = userInfo.getName();
        final String account = userInfo.getAccount();
        final String password = userInfo.getPassword();

        // copy the resourceInfo and setup the new name
        String name = origRI.getResourceName() + new Date().toString().replaceAll(":", ".");
        final ResourceInfo shownRI = new ResourceInfo(name,"",account, creator, origRI.getResourceMIMETypeRef());

        // try to find the monitored map folder
        String monitoredFolderId = "";
        monitoredFolderId = findMonitoredFolder(sid, pid);
        if ( monitoredFolderId != null)
            foundFolderId = new ObjectId(monitoredFolderId);
        else
        {
            // if we can't find the folder, then we should create it
            CreateFolderEvt cfe = new CreateFolderEvt(this, sid, pid, MONITOREDFOLDER,"",new Vector(),"","",null,account,password, null, false);
            CreateFolderRepEvt cfer = new CreateFolderRepEvt();
            try
            {
                EventSynchronizer es = new EventSynchronizer(_core, this);
                es.addEventToPost(cfe);
                es.addExpectedReplyEvent(new CreateFolderRepEvt());
                es.setNrExpectedReplies(1);
                es.setTimeout(-1);
                es.go(0, Core.NON_SWING_THREAD | Core.EXECUTE_SYNC);

                cfer = (CreateFolderRepEvt) es.getReplyEvent();
            }
            catch (InvalidParameterXcp invalidParameterXcp)
            {
                invalidParameterXcp.printStackTrace();
            }
            catch (NoReplyEvtXcp noReplyEvtXcp)
            {
                noReplyEvtXcp.printStackTrace();
            }

            if ( cfer._resourceInfo != null )
            {
                foundFolderId = cfer._resourceInfo.getCachedProjectId();
            }
            else
            {
                CoreOptionPane.showMessage("Error Monitoring Map", "Could not create monitor map folder", CoreOptionPane.ERROR);
                return null;
            }
        }


        // now copy the selected map to monitor to the new location, and with the new name
        CopyResourceEvt cre = new CopyResourceEvt(_thisModule, sid, pid, origRI.getResourceId(), account, password, sid, foundFolderId, shownRI, account, password, null);
        CopyResourceRepEvt crer = new CopyResourceRepEvt();
        try
        {
            EventSynchronizer es = new EventSynchronizer(_core, _thisModule);
            es.addEventToPost(cre);
            es.addExpectedReplyEvent(new CopyResourceRepEvt());
            es.setNrExpectedReplies(1);
            es.setTimeout(-1);
            es.go(0, Core.NON_SWING_THREAD | Core.EXECUTE_SYNC);

            crer = (CopyResourceRepEvt) es.getReplyEvent();
        }
        catch (NoReplyEvtXcp noReplyEvtXcp)
        {
            noReplyEvtXcp.printStackTrace();
            return null;
        }
        catch (InvalidParameterXcp invalidParameterXcp)
        {
            invalidParameterXcp.printStackTrace();
            return null;
        }

        return crer._resourceInfo;

    }

    public void handleShowConnection( MenuItemEvt evt ) throws Exception
    {
        try
        {
        CorePopup cpopup = (CorePopup) evt.menuContainer;
        CMapComponent cmc = (CMapComponent) cpopup.getPopupComponent();
        GraphicalConceptMap gcm = cmc.getGraphicalConceptMapRef();
        ConceptMap cm = gcm.getConceptMapRef();
        ResourceInfo ri = cm.getResourceInfoRef();

        String url = (String) ri.get(MONITOR_MAP_URL);
        if ( url != null)
            url = JOptionPane.showInputDialog("Please enter the URL of the KAoS Directory Service controlling the COI:", url);
        else
            JOptionPane.showMessageDialog(cmc,"There is no KAoS Directory Service registered with this map");
        }
        catch (Exception e )
        {
            System.out.println("Got an exception trying to show KAoS Directory Service Address");
            e.printStackTrace();
        }
    }


    /**
     * Event handler that handles MonitorCOI menu items selections      
     *
     * @param evt the event
     * @throws Exception
     */
    public void handleMonitorCOI(BaseEvt evt) throws Exception
    {
        GetSelectedViewNodeRepEvt repEvt = getSelectedViewNode();

        ResourceInfo ri = repEvt._resourceInfo;
        MonitorMapHelper mmh = null;


        if (ri != null)
        {
            String url = (String) ri.get(MONITOR_MAP_URL);
            boolean badConnection = false;
            while (true)
            {
                if ((url == null) || (badConnection))
                {
                    if (badConnection)
                        url = JOptionPane.showInputDialog("Please enter the URL of the KAoS Directory Service controlling the COI:", url);
                    else
                        url = JOptionPane.showInputDialog("Please enter the URL of the KAoS Directory Service controlling the COI:");
                    if (url == null)
                    {
                        return;
                    }
                }


                try
                {
                    mmh = new MonitorMapHelper(url);
                    break;
                }
                catch (Exception xcp)
                {
                    CoreOptionPane.showMessage("Communication Error", "Problem communicating with the KAoS Directory Service controlling the COI", CoreOptionPane.WARNING);
                    xcp.printStackTrace();
                    badConnection=true;
                }
            }
            if (!url.equals(ri.get(MONITOR_MAP_URL)))
            {
                ri.put(MONITOR_MAP_URL, url);
                try
                {
                    _ff.setResourceInfo(ri);
                }
                catch (Exception xcp)
                {
                    xcp.printStackTrace();
                }
            }
        }


        // now that we have a connection, copy it and start the monitor
        ResourceInfo selectedRI = copyAndRecord(ri);

        if ( selectedRI == null )
        {
            CoreOptionPane.showMessage("Monitoring Map","Error configuring monitored map", CoreOptionPane.ERROR);
            return;
        }
        else
        {
            selectedRI.put(MONITOR_MAP_URL, ri.get(MONITOR_MAP_URL));
            CoreProgressDialog cpd = new CoreProgressDialog("Starting Map Monitor", "Opening " + selectedRI.getResourceName(), "", CoreProgressDialog.PROGRESS, null, true);
            ShowResult sr = new ShowResult(_core, this);
            CMapWnd cmw = sr.openCMap(selectedRI,cpd, "");

            // TODO: calling init should cause the tunnelclient to register observers, not before...
            mmh.initialize(cmw.getCMapComponentInFocus(), this, _core);
            _mapMonitorHelpers.add(mmh);

//            MapHistoryMod.getHistoryMod().startHistory(cmw);
            CMapComponent cmc = cmw.getCMapComponentInFocus();
            ConceptMap cm = cmc.getGraphicalConceptMapRef().getConceptMapRef();
            cm.putTrans("mmv", mmh.getMmv());
        }
    }

    private String findMonitoredFolder(ObjectId sid, ObjectId pid)
    {
        String monitoredFolderId = null;
        Vector searchResults = new Vector();
        CMQuery.submitAll(MONITOREDFOLDER + " AND +sid:" + sid+ " AND +pid:"+pid, searchResults);
        if ( searchResults.size() == 0 )
        {
            System.out.println("We did not find the " + MONITOREDFOLDER + " folder");
        }
        else
        {
            System.out.println("There are " + searchResults.size() + " results");
            for (int i = 0; i < searchResults.size(); i++)
            {
                ResourceSearchResult rsr = (ResourceSearchResult) searchResults.elementAt(i);
                if (! rsr.getRid().equals("0"))
                    monitoredFolderId = rsr.getRid();
            }
        }
        return monitoredFolderId;
    }

    /**
     * Event handler that handles ViewNodeSelectedEvt in order
     * to enable the "File/Add..." menuitem
     *
     * @param evt the event
     * @throws Exception
     */
    public void handleViewNodeSelected(BaseEvt evt) throws Exception
    {
        ViewNodeSelectedEvt vnse = (ViewNodeSelectedEvt) evt;
        CoreWnd folderBrowserWnd = vnse._folderBrowserWnd;
        boolean enableSpecific = false;
        boolean enableImplemented = false;
        boolean enableMonitor = false;
        if (vnse._isFolder && vnse._resourceInfo != null)
        {
            ResourceInfo ri = (ResourceInfo) vnse._resourceInfo;
            String menuType = (String) ri.get(COI_MAP_TYPE);
            if (menuType != null)
            {
                if (menuType.equals(SPECIFIC_TYPE))
                {
                    enableSpecific = true;
                    enableImplemented = true;
                }
                else if (menuType.equals(IMPLEMENTED_TYPE))
                {
                    enableMonitor = true;
                }
            }
            else if (vnse._isFolder && vnse._resourceInfo != null)
            {
                String folderName = vnse._resourceInfo.getResourceName();
                enableSpecific = folderName.equals(GENERIC_COIS);
            }
        }
        folderBrowserWnd.setMenuItemEnableVote(this, FILE_NEW_SPECIFIC_COI_MI, enableSpecific);
        folderBrowserWnd.setMenuItemEnableVote(this, FILE_NEW_IMPLEMENTED_COI_MI, enableImplemented);
    }

    /**
     * Event handler that handles ViewNodeDeselectedEvt in order to enable the "File/Add..." menuitem
     *
     * @param evt the event
     * @throws Exception
     */
    public void handleViewNodeDeselected(BaseEvt evt) throws Exception
    {
        ViewNodeDeselectedEvt vnde = (ViewNodeDeselectedEvt) evt;
        CoreWnd folderBrowserWnd = vnde._folderBrowserWnd;
        folderBrowserWnd.setMenuItemEnableVote(this, FILE_NEW_SPECIFIC_COI_MI, false);
        folderBrowserWnd.setMenuItemEnableVote(this, FILE_NEW_IMPLEMENTED_COI_MI, false);
    }

    /**
     * Event handler invoked when the popup menu is shown on the screen
     * Enables/disables "Add.." based on the enabled status of "File/Add..."
     *
     * @param poe
     */
    public void handlePopupOpenEvts(PopupOpeningEvt poe)
    {
        CoreWnd folderBrowserWnd = poe._corePopup.getComponentsWindow();
        try
        {
            poe._corePopup.setMenuItemEnableVote(_thisModule, NEW_SPECIFIC_COI_MI, folderBrowserWnd.getMenuItemEnableVote(_thisModule, FILE_NEW_SPECIFIC_COI_MI));
            poe._corePopup.setMenuItemEnableVote(_thisModule, NEW_IMPLEMENTED_COI_MI, folderBrowserWnd.getMenuItemEnableVote(_thisModule, FILE_NEW_IMPLEMENTED_COI_MI));

            GetSelectedViewNodeRepEvt repEvt = getSelectedViewNode();
            ResourceInfo ri = repEvt._resourceInfo;

            String menuType = (String) ri.get(COI_MAP_TYPE);
            boolean enableMonitor = ((menuType != null) && (menuType.equals(IMPLEMENTED_TYPE)));//IMPLEMENTED_TYPE)));

            poe._corePopup.setMenuItemEnableVote(_thisModule, MONITOR_IMPL_COI_MI, enableMonitor);
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

    }

    public void handleConceptPopupOpenEvts(PopupOpeningEvt poe)
    {
        CMapComponent cmc = (CMapComponent) poe._corePopup.getPopupComponent();
        try
        {
            ConceptMap cm = cmc.getGraphicalConceptMapRef().getConceptMapRef();
            boolean isMonitored = cm.get(Constants.IS_MONITORED_MAP) != null;
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS", isMonitored);
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/bs/ShowBootstrapFile", isMonitored);
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowPolicies", isMonitored);
//            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowInfo", isMonitored);
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowRelations", isMonitored);
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowRelation", isMonitored);            
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS", isMonitored);
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/bs/ShowBootstrapFile", isMonitored);
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowPolicies", isMonitored);
            //poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowInfo", isMonitored);
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowRelations", isMonitored);
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

    }
    public void handleLinkingPhrasePopupOpenEvts(PopupOpeningEvt poe)
    {
        CMapComponent cmc = (CMapComponent) poe._corePopup.getPopupComponent();
        try
        {
            ConceptMap cm = cmc.getGraphicalConceptMapRef().getConceptMapRef();
            boolean isMonitored = cm.get(Constants.IS_MONITORED_MAP) != null;
            poe._corePopup.setMenuItemEnableVote(_thisModule, "KAoS/ShowRelation", isMonitored);
         }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

    }

    public void handleBackgroundPopupOpenEvts(PopupOpeningEvt poe)
    {
        CMapComponent cmc = (CMapComponent) poe._corePopup.getPopupComponent();
        try
        {
            ConceptMap cm = cmc.getGraphicalConceptMapRef().getConceptMapRef();
            boolean isMonitored = cm.get(Constants.IS_MONITORED_MAP) != null;
            poe._corePopup.setMenuItemEnableVote(_thisModule, "ShowConnection", isMonitored);
         }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

    }
    private GetSelectedViewNodeRepEvt getSelectedViewNode() throws Exception
    {
        EventSynchronizer eventSync = new EventSynchronizer(_core, _thisModule);
        GetSelectedViewNodeEvt gsvne = new GetSelectedViewNodeEvt(_thisModule);
        eventSync.addEventToPost(gsvne);
        eventSync.addExpectedReplyEvent(new GetSelectedViewNodeRepEvt());
        eventSync.setNrExpectedReplies(1);
        eventSync.setTimeout(-1);
        eventSync.go(0, Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        GetSelectedViewNodeRepEvt repEvt = (GetSelectedViewNodeRepEvt) eventSync.getReplyEvent();

        return repEvt;
    }

    /**
     * Event handler that handles menu item selected events on "File/New/*COI"
     * <p/>
     * If new is called from GENERIC_COIS
     * 1. copy and create a new SPECIFIC_COI
     * <p/>
     * if new is called from folder in SPECIFIC_COI
     * 1. if create specific, then add a subfolder to the currently selected subfolder
     * 2. if create instance, then create a new subfolder, but put it in IMPLEMENTED_COIS
     *
     * @param evt the event that triggered the call of the event handler
     * @throws Exception
     */
    public void handleNewCOI(MenuItemChosenEvt evt) throws Exception
    {
        // determine if this is a new specific or implemented COI
        boolean specific = (evt.menuSpec.equals(NEW_SPECIFIC_COI_MI) || evt.menuSpec.equals(FILE_NEW_SPECIFIC_COI_MI));

        CoreWnd cw;
        if (evt.menuContainer instanceof CoreWnd)
        {
            cw = (CoreWnd) evt.menuContainer;
        }
        else if (evt.menuContainer instanceof CorePopup)
        {
            cw = ((CorePopup) evt.menuContainer).getComponentsWindow();
        }
        else
        {
            throw new NullPointerException("Could not get a CoreWnd");
        }

        // get the selected node and some info about it
        GetSelectedViewNodeRepEvt repEvt = getSelectedViewNode();
        ObjectId selectedServerId = repEvt._serverId;

        ObjectId selectedFolderId = repEvt._folderId;
        ObjectId selectedResourceId = repEvt._resourceId;
        ResourceInfo selectedRI = repEvt._resourceInfo;

        // this finds the generic,specific, and implemented folders in the path from the selected resourceinfo
        Hashtable repos = getRepositoryInfo( selectedRI );

        ResourceInfo genericRI = (ResourceInfo) repos.get("genericRI");
        ResourceInfo specificRI = (ResourceInfo) repos.get("specificRI");
        ResourceInfo implementedRI = (ResourceInfo) repos.get("implementedRI");

        // we have to quit if we didn't find the required folders
        if (( genericRI == null ) ||
            ( specificRI == null ) ||
            ( implementedRI == null ))
            return;

        ResourceInfo destRI = null;
        ObjectId srcRootFolderId = null;
        ResourceInfo srcRootFolderRI = null;

        if (selectedFolderId.equals(genericRI.getCachedProjectId()))
        {
            srcRootFolderRI = genericRI;
            srcRootFolderId = genericRI.getResourceId();
            destRI = specificRI;
        }
        else //if (selectedRI.get(COI_ONTOLOGY_TYPE) != null )
        {
            // we have selected a previously created SPECIFIC_COI this means we can either subclass this SPECIFIC_COI or implement it
            srcRootFolderRI = selectedRI;
            srcRootFolderId = selectedRI.getResourceId();
            if (specific)
            {
                destRI = selectedRI;
            }
            else
                destRI = implementedRI;
        }

        String type;
        if (specific)
        {
            type = SPECIFIC_TYPE;
        }
        else
        {
            type = IMPLEMENTED_TYPE;
        }
        String destFolderName = JOptionPane.showInputDialog(cw.getDialogParentFrame(), "Name of " + type + " COI:", "New " + type + " COI", JOptionPane.PLAIN_MESSAGE);

        if (destFolderName == null)
        {
            // user pressed cancel
            return;
        }

        String creator = getCreator();
        String account = getAccount();
        String passwd = getPassword();
        String email = getEmail();
        String org = getOrganization();

        // create the subfolder for the specific coi
        CreateFolderEvt cfe = new CreateFolderEvt(_thisModule,
                selectedServerId,
                destRI.getResourceId(),
                destFolderName,
                "",
                creator, // creator
                creator, // owner
                null,
                account,
                passwd,
                null);

        EventSynchronizer eventSync = new EventSynchronizer(_core, _thisModule);
        eventSync.addEventToPost(cfe);
        eventSync.addExpectedReplyEvent(new CreateFolderRepEvt());
        eventSync.setNrExpectedReplies(1);
        eventSync.setTimeout(-1);
        eventSync.go(0, Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        CreateFolderRepEvt folderRepEvt = (CreateFolderRepEvt) eventSync.getReplyEvent();

        if (folderRepEvt._exception != null)
        {
            folderRepEvt._exception.printStackTrace();
            return;
        }

        ResourceInfo folderRI = folderRepEvt._resourceInfo;
        if (specific)
            folderRI.put(COI_MAP_TYPE, SPECIFIC_TYPE);     // means generic -> specific or specific -> specific
        else
            folderRI.put(COI_MAP_TYPE, IMPLEMENTED_TYPE);  // means specific -> implemented

        SetResourceInfoEvt srie = new SetResourceInfoEvt(_thisModule, folderRI.getCachedServerId(), folderRI.getCachedProjectId(), 
            folderRI.getResourceId(), folderRI, account, passwd, null);
        
        eventSync.reset();
        eventSync.addEventToPost(srie);
        eventSync.addExpectedReplyEvent(new SetResourceInfoRepEvt());
        eventSync.setNrExpectedReplies(1);
        eventSync.setTimeout(-1);
        eventSync.go(0, Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
        SetResourceInfoRepEvt saveInfoRepEvt = (SetResourceInfoRepEvt) eventSync.getReplyEvent();
        if (saveInfoRepEvt._exception != null)
        {
            saveInfoRepEvt._exception.printStackTrace();
        }

        // get the contents of the generic COIs, and create corresponding empty maps in the
        // specific coi subfolder
        Vector folderTOC = _ff.getFolderTOC(selectedServerId, srcRootFolderId, account, passwd, null);

        for (Iterator it = folderTOC.iterator(); it.hasNext();)
        {
            ResourceInfo ri = (ResourceInfo) it.next();

            if (ri.getResourceMIMETypeRef().toString().startsWith("x-nlk-project"))
                continue;
          
            ConceptMap cmap = new ConceptMap();

            ResourceInfo newRI = new ResourceInfo(cmap,
                    destFolderName + ri.getResourceName(),
                    "Ontology Cmap",
                    creator,
                    creator,
                    ri.getResourceMIMETypeRef());
            
            cmap.put(Constants.ONTOLOGY_CMAP_KEY, "TRUE");

            String currentKey = TemplateMod.SOURCE_ONTOLOGY_KEY;
            for (int i = 1; true; i++)
            { 
                if (ri.get(currentKey) != null)
                    newRI.put(currentKey, ri.get(currentKey));
                else
                    break;
                
                currentKey = TemplateMod.SOURCE_ONTOLOGY_KEY + i;
            }
            
            ResourceReference resRef = new ResourceReference(ri.getCachedServerId(), ri.getCachedProjectId(), ri.getResourceId());
            newRI.put(currentKey, resRef);
            newRI.put(currentKey, resRef);

            newRI.setResourceCreator(creator);
            newRI.setResourceOwner(creator);
            newRI.setResourceCreatorInfo(creator,email,org);

            if (specific)
                newRI.put(COI_MAP_TYPE, SPECIFIC_TYPE);     // means generic -> specific or specific -> specific
            else
                newRI.put(COI_MAP_TYPE, IMPLEMENTED_TYPE);  // means specific -> implemented
                
            cmap.setResourceInfoRef(newRI);
            
            // Initilize Namespace table of the newly created map for COI
            NameSpaceTable cmapNameSpaceTable = NameSpaceTable.getMappingsFromCmap(cmap);
            cmapNameSpaceTable.addMapping("", getURLToResource(selectedServerId, newRI.getResourceId()) + "#");
            NameSpaceTable.addMappingsToCmap(cmapNameSpaceTable, cmap);

            GraphicalConceptMap gcm = new GraphicalConceptMap(cmap);  // even though this variable is not used,
                                                                      // it sets back-links to the cmap that are required. 

            SaveResourceEvt sre = new SaveResourceEvt(_thisModule,
                    selectedServerId,
                    folderRepEvt._resourceInfo.getResourceId(),
                    newRI,
                    account,
                    passwd,
                    cmap,
                    null,
                    false);
            eventSync.reset();
            eventSync.addEventToPost(sre);
            eventSync.addExpectedReplyEvent(new SaveResourceRepEvt());
            eventSync.setNrExpectedReplies(1);
            eventSync.setTimeout(-1);
            eventSync.go(0, Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
            SaveResourceRepEvt saveRepEvt = (SaveResourceRepEvt) eventSync.getReplyEvent();
            if (saveRepEvt._exception != null)
            {
                saveRepEvt._exception.printStackTrace();
            }
            
            // Generate template from the parent ontology
            ResourceReference parentMapRef = resRef;
            String parentMapURL = getURLToResource(selectedServerId, ri.getResourceId());
            DynamicTemplateGenerator.generateTemplets(specific, parentMapURL, parentMapRef, _core, _thisModule);
        }
    }

    private String getURLToResource(ObjectId serverId, ObjectId resourceId )
    {
        try
        {
            PrivServer server = PrivServer.getPrivServer();
            ObjectId localServerId = server.getServerId();
            ObjectId rootFolderId = server.getRootFolderId();
            boolean local = (localServerId.equals( serverId ));

            if ( local )
            {
                ResourceInfo ri = server.getResourceInfo(resourceId);
                String tempFileName = CMapSettings.getCMapSettings().getWorkingDirectory() + File.separator + ri.getResourceName() + ".owl";
                Hashtable zipEntries = new Hashtable();
                ResourceReader rr = server.getResourceReader(ri.getCachedProjectId(), resourceId, "owl");
                rr.readResource(tempFileName);
                return "file:/" + tempFileName;
            }
            else
            {
                return FolderFetcher.getURLToResource(serverId, resourceId) + "&partName=owl";
            }
        }
        catch (ResioXcp resioXcp)
        {
            resioXcp.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        return "";
    }

    private Hashtable getRepositoryInfo(ResourceInfo sri)
    {
        Hashtable ht = new Hashtable();
        try
        {
            boolean fg = false;  // found generic
            boolean fs = false;  // found specific
            boolean fi = false;  // found implemented
            Vector parents = _ff.getParentResourceInfos(sri.getCachedServerId(), sri.getCachedProjectId(), sri.getResourceId());

            if (parents.size() == 1)
                parents.add(_ff.getLocalInfo());
            
            for (int i = 1; i < parents.size(); i++)   // start from 1 to skip individual resources
            {
                Object resourceOrPlaceInfo = parents.elementAt(i);
                ResourceInfo thisRI = null;
                if (resourceOrPlaceInfo instanceof ResourceInfo)
                {
                    thisRI = (ResourceInfo) resourceOrPlaceInfo;
                }
                else if (resourceOrPlaceInfo instanceof PlaceInfo)
                {
                    thisRI = ((PlaceInfo) resourceOrPlaceInfo).getRootFolderInfo();
                }

                if (thisRI == null)
                    break;

                System.out.println("Checking parent " + thisRI.getResourceName());
                Vector folderTOC = _ff.getFolderTOC(thisRI.getCachedServerId(), thisRI.getResourceId(), "guest", "guest", null);
                Iterator it = folderTOC.iterator();
                while (it.hasNext())
                {
                    ResourceInfo ri = (ResourceInfo) it.next();
                    System.out.println("Checking " + ri.getResourceName());
                    if (ri.getResourceName().equals(GENERIC_COIS))
                    {
                        ht.put("genericRI", ri);
                        fg = true;
                    }
                    if (ri.getResourceName().equals(SPECIFIC_COIS))
                    {
                        ht.put("specificRI", ri);
                        fs = true;
                    }
                    if (ri.getResourceName().equals(IMPLEMENTED_COIS))
                    {
                        ht.put("implementedRI", ri);
                        fi = true;
                    }
                    if (fg && fs && fi)
                        return ht;
                }
            }
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

        return ht;
    }

    private String getCreator()
    {
        UserInfo userInfo = UserInfo.getUserInfo(_cmapSettings);
        if (userInfo != null)
        {
            if (userInfo.getName() != null)
            {
                return userInfo.getName();
            }
        }
        return "guest";
    }


    private String getAccount()
    {
        UserInfo userInfo = UserInfo.getUserInfo(_cmapSettings);
        if (userInfo != null)
        {
            if (userInfo.getAccount() != null)
            {
                return userInfo.getAccount();
            }
        }
        return "guest";
    }

    private String getPassword()
    {
        UserInfo userInfo = UserInfo.getUserInfo(_cmapSettings);
        if (userInfo != null)
        {
            if (userInfo.getPassword() != null)
            {
                return userInfo.getPassword();
            }
        }
        return "guest";
    }

    private String getEmail()
    {
        UserInfo userInfo = UserInfo.getUserInfo(_cmapSettings);
        if (userInfo != null)
        {
            if (userInfo.getEmail() != null)
            {
                return userInfo.getEmail();
            }
        }
        return "name@site.com";
    }
    private String getOrganization()
    {
        UserInfo userInfo = UserInfo.getUserInfo(_cmapSettings);
        if (userInfo != null)
        {
            if (userInfo.getOrganization() != null)
            {
                return userInfo.getOrganization();
            }
        }
        return "My Place";
    }
    /*
    * Internal state variables
    */
    private Core _core;
    private static COIMod _thisModule;
    private String _moduleName;
    private String _moduleInfo;
    private String _version;
    private String _description;
    private String _authorEmailAddress;
    private FolderFetcher _ff;
    private CMapSettings _cmapSettings;
    /*
    * Private methods
    */

}

