package coi.cmap;

import core.base.Module;
import nlk.base.ObjectId;
import nlk.base.ResourceInfo;
import nlk.base.ResourceReference;
import nlk.code.TemplateDNDList;
import nlk.code.TemplateMod;
import nlk.indexer.CMQuery;
import nlk.indexer.SearchResult;
import nlk.resio.PrivServer;
import nlk.util.CMapListener;

import javax.swing.*;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
 * This class designs a JList whose items can be dragged and dropped on a CMapComponent
 *
 * @author sbrenes
 * @created July 23, 2003
 * <p/>
 * Last modification: $Date: 2006/11/07 21:11:42 $
 * Revision: $Revision: 1.9 $
 * By: $Author: auszok $
 */
public class COITemplateDNDList extends TemplateDNDList
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -4369898369422382093L;

    /**
     * Constructs a new DNDList with the necessary information
     *
     * @param parent The SuggestionsPanelComponent that owns this DNDList
     */
    public COITemplateDNDList(Module parent, ResourceInfo ri)
    {
        _module = parent;
        setModel(new DefaultListModel());
        Vector templateList = new Vector();
        
        String myCOI_ONTOLOGY_TYPE = (String) ri.get(COIMod.COI_MAP_TYPE);
        
        Hashtable currentTemplateHt = null;
        String currentKey = TemplateMod.SOURCE_ONTOLOGY_KEY;
        for (int i = 1; true; i++)
        { 
            if (ri.get(currentKey) != null)
            {
                ObjectId serverId = ri.getCachedServerId();
                ResourceReference ref = (ResourceReference)ri.get(currentKey);
                if (ref != null)
                {
                    PrivServer server = PrivServer.getPrivServer();
                    ObjectId localServerId = server.getServerId();
                    boolean local = (localServerId.equals( serverId ));

                    String usedTemplateQuery = createQueryForCOITemplates(ref, myCOI_ONTOLOGY_TYPE);
                    if ( local )
                    {
                        currentTemplateHt = TemplateMod.getTemplateMod().getTemplates(usedTemplateQuery);
                    }
                    else
                    {
                        currentTemplateHt = getRemoteTemplates(usedTemplateQuery, serverId.toString());
                    }
                }
                
                if (currentTemplateHt != null)
                {
                    Vector currentTemplateList = TemplateMod.getTemplateMod().getKeyVector(currentTemplateHt);
                    templateList.addAll(currentTemplateList);
                    
                    _templateHt.putAll(currentTemplateHt);
                }
            }
            else
                break;
            
            currentKey = TemplateMod.SOURCE_ONTOLOGY_KEY + i;
        }
        
        setListData(templateList);
        
        _dragSource = new DragSource();
        _dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE | DnDConstants.ACTION_COPY, this);
        CMapListener.addDragOverListener(this);
    }
    
    public static String createQueryForCOITemplates(ResourceReference myRef, String _myCOI_ONTOLOGY_TYPE)
    {
        String myResult = TemplateMod.SPECIALIZED_TEMPLATE + ":x AND " + TemplateMod.SOURCE_ONTOLOGY_KEY + ":" + myRef.getResourceId().toString() 
            + " AND " + COIMod.COI_MAP_TYPE + ":" + _myCOI_ONTOLOGY_TYPE;
        
        return myResult;
    }
    public Hashtable getRemoteTemplates(String query, String remoteServerId)
    {
        Hashtable myResult = new Hashtable();
        Vector templates = new Vector();
        
        CMQuery.init();
        CMQuery.searchServer(remoteServerId, query, templates);
        
        if (templates == null)
            return null;
        
        System.out.println("Found " + templates.size() + " templates");

        // clear the elements from the ht incase there are duplicates
        myResult.clear();  

        for (Enumeration templatesEn = templates.elements(); templatesEn.hasMoreElements();)
        {
            SearchResult currentTemplate = (SearchResult) templatesEn.nextElement();
            String currentTemplateName = currentTemplate.getName();
            
            myResult.put(currentTemplateName, currentTemplate);
        }
        
        return myResult;
    }
}