/**
 * Last Modified by: $Author: teskridg $
 * On: $Date: 2007/01/17 19:51:54 $
 */
package coi.cmap;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import core.base.Core;
import core.util.EventSynchronizer;
import nlk.base.*;
import nlk.code.CodeStyles;
import nlk.code.Constants;
import nlk.code.TemplateMod;
import nlk.code.owlimport.NameSpaceTable;
import nlk.code.owlimport.RDFTransformedGraphWrapper;
import nlk.event.resourceManager.CreateFolderEvt;
import nlk.event.resourceManager.CreateFolderRepEvt;
import nlk.refactor.RefactorMod;
import nlk.util.CMapComponent;
import nlk.util.FolderFetcher;
import nlk.util.UserInfo;

import java.awt.*;
import java.net.URI;
import java.util.*;

/**
 * Class containing functionality dynamically generating templates for the specific map 
 */
public class DynamicTemplateGenerator
{
   public static final String TEMPLATES_FOLDER_NAME = "templates";
   
   public static void generateTemplets(boolean buildClasses, String sourceOntologyURL, ResourceReference sourceMapID, Core myCoreRef, COIMod myModule) throws Exception
   {
       Vector folderTOC;
       FolderFetcher _ff = new FolderFetcher(myCoreRef, myModule);
       
       String creator = "guest";
       String account = "guest";
       String passwd = "guest";

       UserInfo userInfo = UserInfo.getUserInfo(myCoreRef.getAppSettings());
       if (userInfo != null)
       {
           if (userInfo.getName() != null)
               creator = userInfo.getName();

           if (userInfo.getAccount() != null)
               account = userInfo.getAccount();

           if (userInfo.getPassword() != null)
               passwd = userInfo.getPassword();
       }

       // Get the contents of source directory and check if the template folder exists already
       folderTOC = _ff.getFolderTOC(sourceMapID.getServerId(), sourceMapID.getProjectId(), account, passwd, null);

       ObjectId templatesFolder = null;
       for (Iterator it = folderTOC.iterator(); it.hasNext(); ) 
       {
           ResourceInfo ri  = (ResourceInfo)it.next();
           
           if (ri.getResourceName().equals(TEMPLATES_FOLDER_NAME)) 
           {
               templatesFolder = ri.getResourceId();
               
               break;
           }  
       }
       
       if ( templatesFolder == null)
       {
           EventSynchronizer eventSync = new EventSynchronizer (myCoreRef, myModule);
           
           // Create the subfolder for the templates
           CreateFolderEvt cfe = new CreateFolderEvt(myModule, 
                                                     sourceMapID.getServerId(), 
                                                     sourceMapID.getProjectId(),
                                                     TEMPLATES_FOLDER_NAME,
                                                     "",
                                                     creator, // creator
                                                     creator, // owner
                                                     null,
                                                     account,
                                                     passwd,
                                                     null);
           
           eventSync.reset();
           eventSync.addEventToPost (cfe);
           eventSync.addExpectedReplyEvent(new CreateFolderRepEvt());
           eventSync.setNrExpectedReplies (1);
           eventSync.setTimeout (-1);
           eventSync.go(0, Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);
           CreateFolderRepEvt folderRepEvt = (CreateFolderRepEvt)eventSync.getReplyEvent();        
           
           if (folderRepEvt._exception != null) 
           {
               folderRepEvt._exception.printStackTrace();
               
               return;
           }
           
           templatesFolder = folderRepEvt._resourceInfo.getResourceId();
       }
       
       folderTOC = _ff.getFolderTOC(sourceMapID.getServerId(), templatesFolder, account, passwd, null);
       
       Set existingTemplates = new HashSet();
       for (Iterator it = folderTOC.iterator(); it.hasNext(); ) 
       {
           ResourceInfo ri  = (ResourceInfo)it.next();
           String currentTemplateName = ri.getResourceName();
               
           existingTemplates.add(currentTemplateName);
       }
       
       RefactorMod myRefactorMod = RefactorMod.getRefactorMod();
       TemplateMod myTemplateMod= TemplateMod.getTemplateMod();
       
       // Get the ontology into Jena 
       OntDocumentManager odm = new OntDocumentManager();
       odm.setProcessImports(false);
       odm.setUseDeclaredPrefixes(true);
       OntModelSpec oms = new OntModelSpec(OntModelSpec.OWL_MEM);
       oms.setDocumentManager(odm);

       OntModel myOntologyModel = ModelFactory.createOntologyModel(oms, null);

       myOntologyModel.read(sourceOntologyURL, Constants.OWL_FORMAT); // --- reading ontology

       String myOntologyModelDefaultNamespace = myOntologyModel.getNsPrefixURI("");
       
       // Find the classes 
       ExtendedIterator _classIT = myOntologyModel.listClasses ();
       
       // For each class create a new template
       String newTemplateTitle;
       String patterName;
       String linkingPropertyName;
       StyleSheet patterStyleSheet;
       StyleSheet patternConnectionStyleSheet;
       StyleSheet mainClassConnectionStyleSheet;
       for (Enumeration _classEnum = RDFTransformedGraphWrapper.sortIterator(_classIT).elements(); _classEnum.hasMoreElements(); ) 
       {
           try 
           {
               OntClass currentOntClass = (OntClass) _classEnum.nextElement();
               String currentOntClassName = currentOntClass.getURI();
               URI currentOntClassURI = new URI(currentOntClassName);
               String currentOntClassURIFragment = currentOntClassURI.getRawFragment();

               String currentOntClassNamespace = currentOntClassName.substring(0, currentOntClassName.lastIndexOf(currentOntClassURIFragment));
               if ( ! currentOntClassNamespace.equals(myOntologyModelDefaultNamespace))
                   continue;
               
               // Create a new template
               if (buildClasses)
               {
                   newTemplateTitle = currentOntClassURIFragment.replace("%20", "") + "Subclass";
                   patterName = "<subclass>";
                   patterStyleSheet = CodeStyles.getClassNodeStyles();
                   linkingPropertyName = Constants.SUBCLASS_OF_PROPERTY;
                   patternConnectionStyleSheet = CodeStyles.getClassConnectionsStyles();
                   mainClassConnectionStyleSheet = CodeStyles.getClassHierarchyConnectionsStyles();
               }  
               else
               {
                   newTemplateTitle = currentOntClassURIFragment.replace("%20", "") + "Instance";
                   patterName = "<instance>";
                   patterStyleSheet = CodeStyles.getIndividualNodeStyles();
                   linkingPropertyName = Constants.INDIV_OF_CLASS_PROPERTY;
                   patternConnectionStyleSheet = CodeStyles.getOtherConnectionsStyles();
                   mainClassConnectionStyleSheet = CodeStyles.getOtherConnectionsStyles();
               }
               
               if (existingTemplates.contains(newTemplateTitle))
                   continue;
               
               CMapComponent newTemplate =
                   myRefactorMod.newMap(newTemplateTitle, sourceMapID.getServerId(), templatesFolder, TemplateMod.MIMETYPE_TEMPLATE, false);
               
               Point mainClassLocation = new Point(100, 100);
               GraphicalConcept mainClassNode = 
                   myTemplateMod.createGraphicalConcept(newTemplate, useNameSpace(currentOntClass.getURI(), newTemplate), mainClassLocation, new ObjectId());
               mainClassNode.setStyleSheet(CodeStyles.getClassNodeStyles());
               Concept _mainClassNode = mainClassNode.getConceptRef();
               _mainClassNode.put("COMMENT_DISPLAY", currentOntClass.getURI());
               
               Point linkingLocation = new Point(100, 150);
               GraphicalLinkingPhrase linkingNode = 
                   myTemplateMod.createGraphicalLinkingPhrase(newTemplate, linkingPropertyName, linkingLocation, new ObjectId());
               
               Point patternLocation = new Point(100, 200);
               GraphicalConcept patternNode = myTemplateMod.createGraphicalConcept(newTemplate, patterName, patternLocation, new ObjectId());
               patternNode.setStyleSheet(patterStyleSheet);
               
               GraphicalConnection patternConnection = myTemplateMod.createGraphicalConnection(newTemplate, patternNode, linkingNode, new ObjectId());
               patternConnection.setStyleSheet(patternConnectionStyleSheet);
               GraphicalConnection mainClassConnection = myTemplateMod.createGraphicalConnection(newTemplate,  linkingNode, mainClassNode, new ObjectId());
               mainClassConnection.setStyleSheet(mainClassConnectionStyleSheet);
               
               // Save the template in the current directory
               if (newTemplate != null)
               {
                   GraphicalConceptMap newTemplateMapGCM = newTemplate.getGraphicalConceptMapRef();
                   ConceptMap newTemplateMap = newTemplateMapGCM.getConceptMapRef();
                   ResourceInfo newTemplateRI = newTemplateMap.getResourceInfoRef();
                   
                   newTemplateRI.put(TemplateMod.SOURCE_ONTOLOGY_KEY, sourceMapID);
                   
                   if (buildClasses)
                       newTemplateRI.put(COIMod.COI_MAP_TYPE, COIMod.SPECIFIC_TYPE);
                   else
                       newTemplateRI.put(COIMod.COI_MAP_TYPE, COIMod.IMPLEMENTED_TYPE);
                   
                   boolean res = myRefactorMod.save(newTemplate, newTemplate.getName(), 2);
                   
                   if (res)
                       System.out.println("Save succeeded for - " + newTemplateTitle);
                   else
                       System.out.println("Save failed for - " + newTemplateTitle);
               }
               else
                   System.out.println("No template to save!");    
           }
           catch (Exception xcp) 
           {
               System.out.println("Error when saving template!");  
           }
       }
       
       _classIT.close ();
   }
   
   private static String useNameSpace (String myURL, CMapComponent myCMapComponent)
   {
       String myURLPrefixed = null;
       String prefix  = "n1";
       String namespace = null;
       NameSpaceTable myNameSpaceTable = new NameSpaceTable();
       
       try
       {
           URI vuri = new URI(myURL);
           String fragment = vuri.getRawFragment();

           namespace = myURL.substring(0, myURL.lastIndexOf(fragment));
              
           myURLPrefixed = prefix + ":" + fragment;

           if (myURLPrefixed.charAt(0) == ':')
               return myURLPrefixed.substring(1);
           
           myNameSpaceTable.addMapping(prefix, namespace);
           
           GraphicalConceptMap gcm = myCMapComponent.getGraphicalConceptMapRef();
           ConceptMap cm = gcm.getConceptMapRef();
           cm.put(Constants.ONTOLOGY_CMAP_KEY, "TRUE");
           NameSpaceTable.addMappingsToCmap(myNameSpaceTable, cm);
           
           return myURLPrefixed;
       }
       catch (Exception xcp)
       {
           return myURL;
       }
   }
}

