package coi.cmap.bootstrap;

import core.base.*;
import core.event.MenuItemChosenEvt;
import core.event.PopupOpeningEvt;
import core.util.CoreProgressDialog;
import core.util.ProgressReporter;
import kaos.kpat.applet.JenaListCellRenderer;
import nlk.base.*;
import nlk.code.Constants;
import nlk.code.ShowOntologyTerm;
import nlk.code.nssp.NameSpaceSidePanel;
import nlk.util.CMapComponent;
import nlk.util.CMapComponentWndInterface;
import nlk.util.FolderFetcher;
import nlk.code.lint.LintPellet;

import javax.swing.*;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.io.*;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: Tom
 * Date: Nov 7, 2006
 * Time: 11:56:15 PM
 * To change this template use File | Settings | File Templates.
 */
public class BootstrapMod implements Module
{
    private Set generatedXMLSchemas = new HashSet();
    private String myXMLSchemaNamespace = "coi.jbi";
    
    Core _core;
    static BootstrapMod _this;

    public void initialize(Core c) throws Exception
    {
        _core = c;
        _this = this;
    }

    public void registerDSTypes() throws Exception
    {

    }

    public void registerPopupTypes() throws Exception
    {
    }

    public void addMenus() throws Exception
    {
        MenuItemParams mip;

        mip = new MenuItemParams("Generate Bootstrap Files...", 1);
        _core.addWndMenuItem(this, "CMapWnd", "Tools/genbsfile", mip);

        mip = new MenuItemParams( "Generate Bootstrap file for Actor", 1, KeyEvent.VK_B );
        _core.addPopupMenuItem( this, "CMapWndConceptPopup",           "KAoS/bs/genbsfile" , mip );
        _core.addPopupMenuItem( this, "CMapWndMultipleEntitiesPopup", "KAoS/bs/genbsfile" , mip );

        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndConceptPopup", "KAoS", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndConceptPopup", "KAoS/bs/genbsfile", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndMultipleEntitiesPopup", "KAoS", false);
        _core.registerForPopupMenuItemEnableVoting(this, "CMapWndMultipleEntitiesPopup", "KAoS/bs/genbsfile", false);
    }

    public void addEvtHandlers() throws Exception
    {
        MenuItemChosenEvt mice = new MenuItemChosenEvt();
        _core.registerForWndMenuItemEvt(this,
                "CMapWnd",
                "Tools/genbsfile",
                mice,
                this,
                "handleGenBootstrap",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD);

        _core.registerForPopupMenuItemEvt( this,
                "CMapWndConceptPopup",
                "KAoS/bs/genbsfile",
                mice,
                this,
                "handleGenBootstrapConcept",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD );

        _core.registerForPopupMenuItemEvt( this,
                "CMapWndMultipleEntitiesPopup",
                "KAoS/bs/genbsfile",
                mice,
                this,
                "handleGenBootstrapConcept",
                Core.EXECUTE_SYNC | Core.NON_SWING_THREAD );

        _core.registerForPopupEvt(this,
                 "CMapWndConceptPopup",
                 new PopupOpeningEvt(),
                 this,
                 "handleConceptPopupOpenEvts",
                 Core.EXECUTE_SYNC | Core.ANY_THREAD);

        _core.registerForPopupEvt(this,
                 "CMapWndMultipleEntitiesPopup",
                 new PopupOpeningEvt(),
                 this,
                 "handleConceptPopupOpenEvts",
                 Core.EXECUTE_SYNC | Core.ANY_THREAD);
    }

    public void addDSInstances() throws Exception
    {
    }

    public void getFromCore() throws Exception
    {

    }

    public void initCompleted()
    {
        
    }

    public String getName()
    {
        return "Bootstrap Module";
    }

    public String getVersion()
    {
        return "0.1";
    }

    public String getDescription()
    {
        return "Writes KaOS bootstrap files for COI actors";
    }

    public boolean okToTerminate()
    {
        return true;
    }

    public void preliminaryCleanup() throws Exception
    {

    }

    public void finalCleanup()
    {

    }

    public String getAuthorAddress()
    {
        return "teskridge@ihmc.us";
    }

    // done with overridden methods

    public void handleGenBootstrap(MenuItemChosenEvt evt)
    {
        System.out.println("Generate bootstrap files for the cmap");
        if (( evt != null ) && ( evt.menuContainer != null ) && ( evt.menuContainer instanceof CMapComponentWndInterface))
        {
            CMapComponent cmc = ((CMapComponentWndInterface) evt.menuContainer).getCMapComponentInFocus();
            GraphicalConceptMap gcm = cmc.getGraphicalConceptMapRef();
            ConceptMap cm = gcm.getConceptMapRef();
            genBootstrapFiles(cm.getConceptRefs(), cmc);
        }
    }

    public void handleGenBootstrapConcept(MenuItemChosenEvt evt)
    {
        System.out.println("Generating bootstrap files for the selection");
        CorePopup cpopup = (CorePopup) evt.menuContainer;
        CMapComponent cmc = (CMapComponent) cpopup.getPopupComponent();

        genBootstrapFiles(cmc.getSelectedGraphicalEntitys(), cmc);
    }

    private void genBootstrapFiles (Enumeration concepts, CMapComponent cmc)
    {
        String path = chooseOutputPath(cmc);
        if (path == null)
            // user pressed cancel
            return;

        GraphicalConceptMap gcm = cmc.getGraphicalConceptMapRef();
        ConceptMap cm = gcm.getConceptMapRef();
        
        CoreProgressDialog dlg = new CoreProgressDialog("Generating Bootstrap Files", "", null, CoreProgressDialog.WAIT, cmc.getContainingCoreWnd(), true);
        dlg.showDialog(CoreProgressDialog.CENTER_ON_PARENT, true);

        Vector domains = getDomainsForActorMap(cm.getResourceInfoRef());

        while(concepts.hasMoreElements())
        {
            Concept currentConcept = (Concept) concepts.nextElement();
            generateBootstrapForIndividual(currentConcept, domains, path, dlg, cm);
        }
        
        dlg.dispose();
    }
            
    private boolean isIndividual (Node node) 
    {
        Object nodeType = node.get(Constants.NODE_TYPE);
        boolean isIndividual = false;
        
        if (nodeType instanceof ExtVector) 
        {
            ExtVector v = (ExtVector) nodeType;
            isIndividual = v.contains(Constants.INDIV);
        }
        else if (nodeType instanceof String) 
        {
            String s = (String) nodeType;
            isIndividual = s.equals(Constants.INDIV);
        } 
        
        return isIndividual;
    }
    
    private void generateBootstrapForIndividual(Concept c, Vector domains, String path, ProgressReporter prg, ConceptMap cm)
    {        
        if (isIndividual(c)) 
        {
            System.out.println("Generate Bootstrap for "+ c.getLabel());
            
            if (prg != null)
                prg.updateUpperMessage("Generating file for "+ c.getLabel());
            
            Connection conn = (Connection) c.getOutgoingConnectionRefs().nextElement();
            Node isA = conn.getSecondNodeRef();
            Enumeration ontTypeNodes = isA.getOutgoingConnectionRefs();
            HashSet ontTypes = new HashSet();
            HashMap ontOperations = new HashMap();
            
            while (ontTypeNodes.hasMoreElements()) 
            {
                Connection connect = (Connection) ontTypeNodes.nextElement();
                Node ontTypeNode = connect.getSecondNodeRef();
                
                String ontTypeNodeURI = getURIForNode(ontTypeNode);
                ontTypes.add(ontTypeNodeURI);
                
                // Now we need to follow the ont type node and find the ontological operations
                if (ontTypeNodeURI != null) 
                {
                    ShowOntologyTerm sot = new ShowOntologyTerm(_core, this);
                    ResourceReference ref = sot.getCmapForURI(ontTypeNodeURI, false);
                    
                    // Now we need to open this map, find the node in question, and follow the appropriate links get the resource reader
                    try 
                    {
                        FolderFetcher ff = new FolderFetcher(_core, this);
                        ResourceReader rr = ff.getResourceReader (ref.getServerId(), ref.getProjectId(), ref.getResourceId());
                        
                        //Get the cmap
                        ConceptMap cmap = (ConceptMap) rr.readResource();
                        
                        for (Enumeration concepts = cmap.getConceptRefs(); concepts.hasMoreElements(); ) 
                        {
                            Concept concept = (Concept) concepts.nextElement();
                            
                            if (ontTypeNodeURI.equals(getURIForNode(concept))) 
                            {
                                // Might be a duplicate node, check to see if it has links other than "are"
                                boolean foundOntOps = false;
                                for (Enumeration outRefs = concept.getOutgoingConnectionRefs(); outRefs.hasMoreElements(); ) 
                                {
                                    Connection conn2 = (Connection) outRefs.nextElement();
                                    String linkName = conn2.getSecondNodeRef().getLabel();
                                    
                                    if ( ! linkName.equalsIgnoreCase("are")) 
                                    {
                                        // Found an ontological operation
                                        String ontOp = getURIForNode(conn2.getSecondNodeRef());
                                        for (Enumeration e = conn2.getSecondNodeRef().getOutgoingConnectionRefs(); e.hasMoreElements(); ) 
                                        {
                                            Node ontOpClass = ((Connection) e.nextElement()).getSecondNodeRef();
                                            Enumeration e2 = ontOpClass.getOutgoingConnectionRefs();
                                            
                                            // Check to see if the ontOpClass has instances defined
                                            if (e2.hasMoreElements()) 
                                            {
                                                // Values are instances
                                                Connection isDefAsConn = (Connection) ontOpClass.getOutgoingConnectionRefs().nextElement();
                                                
                                                for (Enumeration instanceConns = isDefAsConn.getSecondNodeRef().getOutgoingConnectionRefs(); 
                                                    instanceConns.hasMoreElements(); ) 
                                                {
                                                    Node instance = ((Connection) instanceConns.nextElement()).getSecondNodeRef();
                                                    putWithDuplicateKeys(ontOperations, ontOp, getURIForNode(instance));
                                                }
                                            }
                                            else
                                                // Value is the class itself
                                                putWithDuplicateKeys(ontOperations, ontOp, getURIForNode(ontOpClass));
                                        }
                                        
                                        foundOntOps = true;
                                    }
                                }
                                
                                if (foundOntOps)
                                    break;
                            }
                        }
                    }
                    catch (Exception xcp) 
                    {
                        System.err.println("Error getting ontological operations for " + ontTypeNode.getLabel());
                        xcp.printStackTrace();
                    }
                }
            }
                        
            // Now we've got the ontological types and ontological operations
            
            // Write the boot file
            try 
            {
                writeBootFile(getURIForNode(c), ontTypes, ontOperations, domains, path);
                writeJavaClientFile(getURIForNode(c), ontTypes, ontOperations, domains, path);
                generatedXMLSchemas = new HashSet();
                writeXMLSchemas(ontOperations, path, cm);
            }
            catch (IOException ioXcp) 
            {
                ioXcp.printStackTrace();
            }
        }
    }

    private String chooseOutputPath(Component parent)
    {
        JFileChooser chooser = new JFileChooser(_outputPath);
        chooser.setDialogTitle("Please select a folder for the bootstrap files");
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        
        if (chooser.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) 
        {
            _outputPath = chooser.getSelectedFile().getPath();
            return _outputPath;
        }
        
        return null;
    }
    
    private void writeXMLSchemas (HashMap ontOperations, String path, ConceptMap cm) throws IOException
    {
        String filename;        
        FileOutputStream fos;
        BufferedWriter writer;
        NameSpaceSidePanel nssp = (NameSpaceSidePanel)cm.getTrans("nssp");
        LintPellet myLintPellet = null;
        
        if (nssp != null)
            myLintPellet = nssp.getOntologyContentsManager().getLintPellet();
        
        for (Iterator ontTypesIT = ontOperations.values().iterator(); ontTypesIT.hasNext(); ) 
        {
            String currentOntType = (String)ontTypesIT.next();
            String _currentOntType = JenaListCellRenderer.getShortName(currentOntType).replaceAll("%20", "_").replaceAll("\\(", "").replaceAll("\\)", "");
            
            if (generatedXMLSchemas.contains(currentOntType))
                continue;
            
            // Open the XMLSchma file
            filename = path + File.separator + myXMLSchemaNamespace + "." + _currentOntType + ".xsd"; 
            fos = new FileOutputStream(filename);
            writer = new BufferedWriter(new OutputStreamWriter(fos));
            
            // Generate the Schema
            writer.write("<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" elementFormDefault=\"qualified\">"); writer.newLine();
            writer.write("    <xs:annotation>"); writer.newLine();
            writer.write("        <xs:documentation xml:lang=\"en\">"); writer.newLine();
            writer.write("            Originated from ontology class: "); writer.newLine();
            writer.write("                " + currentOntType.replace("&", "&amp;")); writer.newLine();
            writer.write("        </xs:documentation>"); writer.newLine();
            writer.write("    </xs:annotation>"); writer.newLine();
            writer.write("    <xsd:element name=\"" + _currentOntType + "metadata\">"); writer.newLine();
            writer.write("        <xsd:complexType>"); writer.newLine();
            writer.write("            <xsd:sequence>"); writer.newLine();
            writer.write("                <xsd:element name=\"" + _currentOntType + "Object\" type=\"" + _currentOntType + "Object\"/>"); writer.newLine();
            writer.write("            </xsd:sequence>"); writer.newLine();
            writer.write("        </xsd:complexType>"); writer.newLine();
            writer.write("    </xsd:element>"); writer.newLine();
            writer.write("    <xsd:complexType name=\"" + _currentOntType + "Object\">"); writer.newLine();
            writer.write("        <xsd:sequence>"); writer.newLine();
            
            // -- Generate contend of metaobject
            if (myLintPellet != null)
            {
                Set currentOntTypeProperties = myLintPellet.getProperties(currentOntType);
                
                for (Iterator currentOntTypePropertiesIT = currentOntTypeProperties.iterator() ; currentOntTypePropertiesIT.hasNext(); )
                {
                    Object currentProperty = currentOntTypePropertiesIT.next();
                }
            }
            
            writer.write("        </xsd:sequence>"); writer.newLine();
            writer.write("    </xsd:complexType>"); writer.newLine();
            writer.write("</xsd:schema>"); writer.newLine();
            
            // Close the XMLSchema file
            writer.close();
            fos.close();
            
            // Add the type to the set of processed types
            generatedXMLSchemas.add(currentOntType);
        }
    }
    
    private void writeBootFile (String uri, HashSet ontTypes, HashMap ontOperations, List domains, String path) throws IOException
    {
        String filename = path + File.separator + JenaListCellRenderer.getShortName(uri) + ".cfg";        
        FileOutputStream fos = new FileOutputStream(filename);
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(fos));
        
        // Header
        writer.write("(java-agent-services");
        writer.newLine();
        writer.write("    (key-prefix javax.agent.service)");
        writer.newLine();
        writer.write("    (agent-configuration");
        writer.newLine();
        
        // id
        writer.write("        (id " + uri + ")");
        writer.newLine();
        
        // Domains
        writer.write("        (domains");
        writer.newLine();
        Iterator it = domains.iterator();
        for (int i=1; it.hasNext(); i++) 
        {
            writer.write("            (" + it.next() + " " + i + ")");
            writer.newLine();
        }
        writer.write("        )");
        writer.newLine();
        
        // Ontological types
        writer.write("        (ontological-types");
        writer.newLine();
        it = ontTypes.iterator();
        for (int i=1; it.hasNext(); i++) 
        {
            writer.write("            (" + it.next() + " " + i + ")");
            writer.newLine();
        }
        writer.write("        )");
        writer.newLine();

        // Ontological operations
        writer.write("        (OntologicalOperations");
        writer.newLine();
        for (it = ontOperations.keySet().iterator(); it.hasNext(); ) 
        {
            String key = (String) it.next();
            writer.write("            (" + key + " " + ontOperations.get(key) + ")");
            writer.newLine();
        }
        writer.write("        )");
        writer.newLine();
        writer.write("    )");
        writer.newLine();
        writer.write(")");
        writer.newLine();
        
        writer.close();
        fos.close();
    }
    
    private void writeJavaClientFile (String uri, HashSet ontTypes, HashMap ontOperations, List domains, String path) throws IOException
    {
        String myNamespace = "coi.jbi";
        String myGeneratedClassName = JenaListCellRenderer.getShortName(uri) + "Client";
        myGeneratedClassName = myGeneratedClassName.replaceAll("%20", "_").replaceAll("\\(", "").replaceAll("\\)", "");
        
        String myLastNamespacePart = null;
        if ( ! domains.isEmpty())
            myLastNamespacePart = JenaListCellRenderer.getShortName((String)domains.toArray()[0]).
                replaceAll("%20", "_").replaceAll("\\(", "").replaceAll("\\)", "").replace("-", "");
        
        if (myLastNamespacePart == null)
            myLastNamespacePart = "demo";
        
        if (myLastNamespacePart.equals(""))
            myLastNamespacePart = "demo";
        
        // Open the file
        String filename = path + File.separator + JenaListCellRenderer.getShortName(uri) + ".java";        
        FileOutputStream fos = new FileOutputStream(filename);
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(fos));
        
        // Package
        writer.write("package " + myNamespace + "." + myLastNamespacePart +";"); writer.newLine();
        writer.newLine();

        // Imports
        writer.write("import java.util.Map;"); writer.newLine();
        writer.write("import java.util.HashMap;"); writer.newLine();
        writer.write("import kaos.core.csi.OntInstanceDescriptionImpl;"); writer.newLine();
        writer.write("import kaos.core.csi.OntPropertyDescription;"); writer.newLine();
        writer.write("import kaos.core.csi.OntPropertyDescriptionImpl;"); writer.newLine();
        writer.write("import kaos.ontology.util.OntModelUtils;"); writer.newLine();
        writer.newLine();
        writer.write("import org.infospherics.jbi.client.InfoObject;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.PublisherSequence;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.SubscriberSequence;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.ConnectionException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.InvalidAttributeXMLException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.InvalidPredicateException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.PermissionDeniedException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.PlatformFailureException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.PredicateLanguageException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.SequenceStateException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.UnknownAttributeException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.UnrecognizedObjectTypeException;"); writer.newLine();
        writer.write("import org.infospherics.jbi.client.exception.UnsupportedVersionException;"); writer.newLine();
        writer.newLine();
        writer.write("import com.hp.hpl.jena.shared.PrefixMapping;"); writer.newLine();
        writer.newLine();
        writer.write("import coi.jbi.interceptors.JBIBaseClient;"); writer.newLine();
        writer.newLine();
        writer.write("import org.infospherics.jbi.client.ObjectAvailableCallback;"); writer.newLine();
        writer.newLine();
        writer.write("public class " + myGeneratedClassName + " extends JBIBaseClient");  writer.newLine();
        writer.write("{");  writer.newLine();
        
        // Sequances variables definitions
        for (Iterator ontOperationsIT = ontOperations.keySet().iterator(); ontOperationsIT.hasNext(); ) 
        {
            String key = (String)ontOperationsIT.next();
            String myTypeName = JenaListCellRenderer.getShortName((String)ontOperations.get(key));
            myTypeName = myTypeName.replaceAll("%20", "_").replaceAll("\\(", "").replaceAll("\\)", "");
            
            if (key.contains("isProducing"))
            {
                writer.write("    protected PublisherSequence " + myTypeName + "Pub = null;");
                writer.newLine(); 
            }
            else
            {
                writer.write("    protected SubscriberSequence " + myTypeName + "Sub = null;");
                writer.newLine(); 
            }
        }
        writer.newLine();
        
        // Deafult constructor
        writer.write("    public " + myGeneratedClassName + "()"); writer.newLine();
        writer.write("    {"); writer.newLine();
        writer.write("        super();"); writer.newLine();
        writer.write("    }"); writer.newLine();
        writer.newLine();
        
        // initCommunications method
        writer.write("    public void initCommunications() throws UnrecognizedObjectTypeException, PermissionDeniedException, PlatformFailureException, SequenceStateException,");
        writer.newLine();
        writer.write("        UnsupportedVersionException, InvalidPredicateException, PredicateLanguageException, ConnectionException"); 
        writer.newLine();
        writer.write("    {");  writer.newLine();
        
        // Opening sequances
        for (Iterator ontOperationsIT = ontOperations.keySet().iterator(); ontOperationsIT.hasNext(); ) 
        {
            String key = (String)ontOperationsIT.next();
            String myTypeName = JenaListCellRenderer.getShortName((String)ontOperations.get(key));
            myTypeName = myTypeName.replaceAll("%20", "_").replaceAll("\\(", "").replaceAll("\\)", "");
            
            if (key.contains("isProducing"))
            {
                String mySequanceName = myTypeName + "Pub";
                
                writer.write("        " + mySequanceName + " = initPublisherChannel(" + myXMLSchemaNamespace + "."+ myTypeName + ", \"1.0\");");
                writer.newLine();
                writer.write("        " + mySequanceName + ".activateSequence();");
                writer.newLine(); writer.newLine(); 
            }
            else
            {
                String mySequanceName = myTypeName + "Sub";
                
                writer.write("        " + mySequanceName + " = initSubscriberChannel(" + myXMLSchemaNamespace + "." + myTypeName + ", \"1.0\");");
                writer.newLine(); 
                writer.write("        ObjectAvailableCallback my" + myTypeName + "Callback" + " = new " + myTypeName + "Callback();");
                writer.newLine(); 
                writer.write("        " + mySequanceName +".setSequenceCallback(my" + myTypeName + "Callback);");
                writer.newLine(); 
                writer.write("        " + mySequanceName + ".activateSequence();");
                writer.newLine(); writer.newLine(); 
            }
        }
        
        // End of initCommunications method
        writer.write("    }");  writer.newLine();
        
        // End of class
        writer.write("}");  writer.newLine();
        
        // End writing to the file
        writer.close();
        fos.close();
    }
    
    private void putWithDuplicateKeys (HashMap map, String key, String value)
    {
        String newKey = key;
        
        for (int i=2; map.containsKey(newKey); i++) 
        {
            if (map.get(newKey).equals(value))
                return;
            
            newKey = key + i;
        }
        
        map.put(newKey, value);
    }

    private String getURIForNode (Node node)
    {
        String uri = null;
        String label = node.getLabel();
        int idx = label.indexOf(':');
        String nsKey = "";
        String name = label;
            
        if (idx > 0) 
        {
            nsKey = label.substring(0,idx);
            name = label.substring(idx+1);
        }
            
        ExtHashtable namespaces = (ExtHashtable) node.getConceptMapRef().get(Constants.NAMESPACE_TAG);
            
        if (namespaces != null) 
        {
            String namespace = (String) namespaces.get(nsKey);
                
            if (namespace != null)
                uri = namespace + name;
         }

        return uri;
    }

    private Vector getDomainsForActorMap (ResourceInfo resourceInfo) 
    {
        Vector domains = new Vector();
        String existingName = resourceInfo.getResourceName();
        
        if (existingName.endsWith("Actor")) 
        {
            String nameToFind = existingName.substring(0, existingName.length() - 5);
            FolderFetcher ff = new FolderFetcher(_core, this);
            
            try 
            {
                Vector toc = ff.getFolderTOC(resourceInfo.getCachedServerId(), resourceInfo.getCachedProjectId(), "guest", "guest", null);
                for (Iterator it = toc.iterator(); it.hasNext();) 
                {
                    ResourceInfo ri = (ResourceInfo) it.next();
                    
                    if (ri.getResourceName().equals(nameToFind) && ri.getResourceMIMETypeRef().getMajorType().equals("x-cmap")) 
                    {
                        // Found the map
                        ResourceReader rr = ff.getResourceReader(ri.getCachedServerId(), ri.getCachedProjectId(), ri.getResourceId());
                        ConceptMap cmap = (ConceptMap) rr.readResource();
                        
                        for (Enumeration concepts = cmap.getConceptRefs(); concepts.hasMoreElements(); ) 
                        {
                            Concept c = (Concept) concepts.nextElement();
                            if (isIndividual(c))
                                domains.add(getURIForNode(c));
                        }
                        
                        break;
                    }
                }
            }
            catch (Exception xcp) 
            {
                xcp.printStackTrace();
            }
        }
        
        return domains;
    }

    public void handleConceptPopupOpenEvts(PopupOpeningEvt poe)
    {
        CMapComponent cmc = (CMapComponent) poe._corePopup.getPopupComponent();
        
        try
        {
            ConceptMap cm = cmc.getGraphicalConceptMapRef().getConceptMapRef();

            boolean isActor = true;

            poe._corePopup.setMenuItemEnableVote(this, "KAoS", isActor);
            poe._corePopup.setMenuItemEnableVote(this, "KAoS/bs/genbsfile", isActor );
        }
        catch (Exception xcp)
        {
            xcp.printStackTrace();
        }

    }

    private String _outputPath = null;
}