package coi.cmap.monitor;

import core.util.graphics.CoreGraphicsUtils;
import nlk.base.*;
import nlk.code.Constants;

import java.awt.*;
import java.util.Enumeration;
import java.util.StringTokenizer;

/**
 * Provides the standard default styles for interactions in monitor maps for COI.
 *
 * @version $Revision: 1.5 $
 * @author  Tom Eskridge, Thomas Reichherzer, Raul Saavedra
 */
public class InteractionStyles
{
    /*
     * Private member variables
     */
    private static StyleSheet _classNodeStyles;
    private static StyleSheet _registeredStyles;
    private static StyleSheet _individualNodeStyles;
    private static StyleSheet _literalNodeStyles;
    private static StyleSheet _errLiteralNodeStyles;
    private static StyleSheet _rootNodeStyles;
    private static StyleSheet _propertyNodeStyles;
    private static StyleSheet _internalNodeLabelStyles;

    private static StyleSheet _classConnectionsStyles;
    private static StyleSheet _classDefConnectionsStyles;
    private static StyleSheet _classHierarchyConnectionsStyles;
    private static StyleSheet _domainRangeConnectionsStyles;
    private static StyleSheet _equivalenceConnectionStyles;
    private static StyleSheet _propertyConnectionsStyles;
    private static StyleSheet _otherConnectionsStyles;
    private static StyleSheet _initialInteractionStyles;
    private static StyleSheet _continuedInteractionStyles;
    private static StyleSheet _latentInteractionStyles;
    private static StyleSheet _registeredIndivStyles;

    private static StyleSheet _registeredNIOStyles;
    private static StyleSheet _registeredIndivNIOStyles;


    // Inner classes ConnStyle and NodeStyle
    public static class ConnStyle
    {
        ExtColor _lineColor;
        Integer  _lineDir;
        Integer  _lineStyle;
        Integer  _lineThick;
        Integer  _lineType;

        public ConnStyle (ExtColor  lineColor,
                          Integer   lineDir,
                          Integer   lineStyle,
                          Integer   lineThick,
                          Integer   lineType)
        {
            _lineColor = lineColor;
            _lineDir   = lineDir;
            _lineStyle = lineStyle;
            _lineThick = lineThick;
            _lineType  = lineType;
        }

        public ConnStyle (String str)
        {
            if (str == null) {
                throw new IllegalArgumentException ("Null str");
            }
            StringTokenizer stok = new StringTokenizer (str);
            int red   = Integer.parseInt (stok.nextToken());
            int green = Integer.parseInt (stok.nextToken());
            int blue  = Integer.parseInt (stok.nextToken());
            boolean transp = stok.nextToken().toLowerCase().equals ("true");
            _lineColor = new ExtColor (red, green, blue);
            _lineColor.setTransparent (transp);
            _lineDir   = new Integer (stok.nextToken());
            _lineStyle = new Integer (stok.nextToken());
            _lineThick = new Integer (stok.nextToken());
            _lineType  = new Integer (stok.nextToken());
        }

        public String toString ()
        {
            return _lineColor +"; "+
                   _lineDir   +"; "+
                   _lineStyle +"; "+
                   _lineThick +"; "+
                   _lineType;
        }
    }

    public static class NodeStyle
    {
        String    _fontName;
        Integer   _fontSize;
        Integer   _fontStyle;
        ExtInsets _insets;
        ExtColor  _bkgColor;
        ExtColor  _bdrColor;
        Integer   _bdrThick;
        Integer   _bdrType;
        Integer   _bdrShape;

        public NodeStyle (String    fname,
                          Integer   fsize,
                          Integer   fstyle,
                          ExtInsets insets,
                          ExtColor  bkgcolor,
                          ExtColor  bdrcolor,
                          Integer   bdrthick,
                          Integer   bdrtype,
                          Integer   bdrshape)
        {
            _fontName  = fname;
            _fontSize  = fsize;
            _fontStyle = fstyle;
            _insets    = insets;
            _bkgColor  = bkgcolor;
            _bdrColor  = bdrcolor;
            _bdrThick  = bdrthick;
            _bdrType   = bdrtype;
            _bdrShape  = bdrshape;
        }

        /**
         * Constructor that Parses all the member fields out of one string, assuming a specific format
         * (see comments in nlk/code/styles/codestyles.txt for details)
         */
        public NodeStyle (String str)
        {
            if (str == null) {
                throw new IllegalArgumentException ("Null str");
            }
			StringTokenizer stok = new StringTokenizer (str);
            _fontName  = stok.nextToken ();
            _fontSize  = new Integer (stok.nextToken ());
            _fontStyle = new Integer (stok.nextToken ());
            int top    = Integer.parseInt (stok.nextToken());
            int left   = Integer.parseInt (stok.nextToken());
            int bottom = Integer.parseInt (stok.nextToken());
            int right  = Integer.parseInt (stok.nextToken());
            _insets = new ExtInsets (top, left, bottom, right);

            // RGB values + transparency
            int red   = Integer.parseInt (stok.nextToken());
            int green = Integer.parseInt (stok.nextToken());
            int blue  = Integer.parseInt (stok.nextToken());
            boolean transp = stok.nextToken().toLowerCase().equals ("true");
            _bkgColor = new ExtColor (red, green, blue);
            _bkgColor.setTransparent (transp);

            red    = Integer.parseInt (stok.nextToken());
            green  = Integer.parseInt (stok.nextToken());
            blue   = Integer.parseInt (stok.nextToken());
            transp = stok.nextToken().toLowerCase().equals ("true");
            _bdrColor = new ExtColor (red, green, blue);
            _bdrColor.setTransparent (transp);

            _bdrThick = new Integer (stok.nextToken());
            _bdrType  = new Integer (stok.nextToken());
            _bdrShape = new Integer (stok.nextToken());
        }

        public String toString ()
        {
            return "('"+_fontName+", "+_fontSize+", "+_fontStyle+"); "+
                   _insets  +"; "+
                   _bkgColor+"; "+
                   _bdrColor+"; "+
                   _bdrThick+"; "+
                   _bdrType +"; "+
                   _bdrShape;
        }
    }

    //==============================================
    // NODE STYLES:
    //==============================================
    public static StyleSheet getNodeStyles (String nodeType) throws Exception
    {
        if (nodeType.equals(Constants.OWL_CLASS)) {
            return getClassNodeStyles();
        }
        else if (nodeType.equals(Constants.INDIV)) {
            return getIndividualNodeStyles();
        }
        else if (nodeType.equals(Constants.LITERAL)) {
            return getLiteralNodeStyles();
        }
        else if (nodeType.equals(Constants.ANY_OF)) {
            return getClassNodeStyles();
        }
        else if (nodeType.equals(Constants.THINGS_WHICH)) {
            return getClassNodeStyles();
        }
        else if (nodeType.equals(Constants.ONE_OF)) {
            return getClassNodeStyles();
        }

        throw new Exception("Invalid node type");
    }

    public static StyleSheet getClassNodeStyles ()
    {
        if (_classNodeStyles == null) {
            InteractionStyles.NodeStyle classNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.PLAIN),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor (233, 243, 245),
                                 new ExtColor (Color.black),
                                 new Integer (1),
                                 new Integer (CoreGraphicsUtils.LT_SOLID),
                                 new Integer (CoreGraphicsUtils.BS_ROUNDED));
            _classNodeStyles = new StyleSheet();
            setConceptStyleValues(_classNodeStyles, classNodeStyle);
        }

        return copyStyleSheet(_classNodeStyles);
    }

    public static StyleSheet getRegisteredStyles ()
    {
        if (_registeredStyles == null) {
            InteractionStyles.NodeStyle classNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.PLAIN),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor (233, 243, 245),
                                 new ExtColor (Color.green),
                                 new Integer (4),
                                 new Integer (CoreGraphicsUtils.LT_SOLID),
                                 new Integer (CoreGraphicsUtils.BS_ROUNDED));
            _registeredStyles = new StyleSheet();
            setConceptStyleValues(_registeredStyles, classNodeStyle);
        }

        return copyStyleSheet(_registeredStyles);
    }
    public static StyleSheet getRegisteredIndivStyles ()
    {
        if (_registeredIndivStyles == null) {
            InteractionStyles.NodeStyle indivNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.PLAIN),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor (233, 243, 245),
                                 new ExtColor (Color.green),
                                 new Integer (4),
                                 new Integer (CoreGraphicsUtils.LT_SOLID),
                                 new Integer (CoreGraphicsUtils.BS_SQUARE));
            _registeredIndivStyles = new StyleSheet ();
            setConceptStyleValues(_registeredIndivStyles, indivNodeStyle);
        }

        return copyStyleSheet(_registeredIndivStyles);

    }

    public static StyleSheet getRegisteredNotInOntologyStyles ()
    {
        if (_registeredNIOStyles == null) {
            InteractionStyles.NodeStyle classNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.ITALIC),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor (233, 243, 245),
                                 new ExtColor (Color.GRAY),
                                 new Integer (4),
                                 new Integer (CoreGraphicsUtils.LT_SOLID),
                                 new Integer (CoreGraphicsUtils.BS_ROUNDED));
            _registeredNIOStyles = new StyleSheet();
            setConceptStyleValues(_registeredNIOStyles, classNodeStyle);
        }

        return copyStyleSheet(_registeredNIOStyles);
    }
    public static StyleSheet getRegisteredIndivNotInOntologyStyles ()
    {
        if (_registeredIndivNIOStyles == null) {
            InteractionStyles.NodeStyle indivNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.ITALIC),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor (233, 243, 245),
                                 new ExtColor (Color.GRAY),
                                 new Integer (4),
                                 new Integer (CoreGraphicsUtils.LT_SOLID),
                                 new Integer (CoreGraphicsUtils.BS_SQUARE));
            _registeredIndivNIOStyles = new StyleSheet ();
            setConceptStyleValues(_registeredIndivNIOStyles, indivNodeStyle);
        }

        return copyStyleSheet(_registeredIndivNIOStyles);

    }


    public static StyleSheet getIndividualNodeStyles ()
    {
        if (_individualNodeStyles == null) {
            InteractionStyles.NodeStyle indivNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.PLAIN),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor (233, 243, 245),
                                 new ExtColor (Color.black),
                                 new Integer (1),
                                 new Integer (CoreGraphicsUtils.LT_SOLID),
                                 new Integer (CoreGraphicsUtils.BS_SQUARE));
            _individualNodeStyles = new StyleSheet ();
            setConceptStyleValues(_individualNodeStyles, indivNodeStyle);
        }

        return copyStyleSheet(_individualNodeStyles);

    }


    public static StyleSheet getLiteralNodeStyles ()
    {
        if (_literalNodeStyles == null) {
            InteractionStyles.NodeStyle literNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.PLAIN),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor ("1255"), // transparent color for background
                                 new ExtColor ("1255"), // transparent color for border
                                 new Integer (1),
                                 new Integer (CoreGraphicsUtils.LT_INVISIBLE),
                                 new Integer (CoreGraphicsUtils.BS_SQUARE));
            _literalNodeStyles = new StyleSheet ();
            setConceptStyleValues(_literalNodeStyles, literNodeStyle);
        }

        return copyStyleSheet(_literalNodeStyles);
    }


    public static StyleSheet getErrLiteralNodeStyles ()
    {
        if (_errLiteralNodeStyles == null) {
            InteractionStyles.NodeStyle literNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.PLAIN),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor ("1255"), // transparent color for background
                                 new ExtColor ("1255"), // transparent color for border
                                 new Integer (1),
                                 new Integer (CoreGraphicsUtils.LT_INVISIBLE),
                                 new Integer (CoreGraphicsUtils.BS_SQUARE));
            _errLiteralNodeStyles = new StyleSheet ();
            setConceptStyleValues(_literalNodeStyles, literNodeStyle);
        }

        return copyStyleSheet(_errLiteralNodeStyles);
    }
    public static StyleSheet getRootNodeStyles ()
    {
        if (_rootNodeStyles == null) {
            InteractionStyles.NodeStyle rootNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.BOLD),
                                 new ExtInsets (4, 4, 4, 4),
                                 new ExtColor (255, 204, 204),
                                 new ExtColor (Color.black),
                                 new Integer (3),
                                 new Integer (CoreGraphicsUtils.LT_SOLID),
                                 new Integer (CoreGraphicsUtils.BS_SQUARE));
            _rootNodeStyles = new StyleSheet ();
            setConceptStyleValues(_rootNodeStyles, rootNodeStyle);
        }

        return copyStyleSheet(_rootNodeStyles);
    }


    public static StyleSheet getPropertyNodeStyles ()
    {
        if (_propertyNodeStyles == null) {
            InteractionStyles.NodeStyle propNodeStyle
                = new InteractionStyles.NodeStyle("Verdana",
                                 new Integer (12),
                                 new Integer (Font.PLAIN),
                                 new ExtInsets (1, 1, 1, 1),
                                 new ExtColor ("1255"),
                                 new ExtColor (Color.red),
                                 new Integer (1),
                                 new Integer (CoreGraphicsUtils.LT_INVISIBLE),
                                 new Integer (CoreGraphicsUtils.BS_SQUARE));
            _propertyNodeStyles = new StyleSheet ();
            setConceptStyleValues(_propertyNodeStyles, propNodeStyle);
        }

        return copyStyleSheet(_propertyNodeStyles);
    }


    public static StyleSheet getInternalNodeLabelStyles ()
    {
        if (_internalNodeLabelStyles == null) {
            _internalNodeLabelStyles = new StyleSheet ();
            ExtColor bkClr = new ExtColor (0, 0, 0);
            bkClr.setTransparent (true);
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_BACKGROUND_COLOR, bkClr); // transparent background color
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_BORDER_HAS_SHADOW, new Boolean (false));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_BORDER_TYPE,
                                          new Integer (CoreGraphicsUtils.LT_INVISIBLE));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_WIDTH, new Integer ( -1));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_HEIGHT, new Integer ( -1));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_INSETS,
                                          new ExtInsets (4, 4, 4, 4));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_TEXT_COLOR,
                                          new ExtColor (Color.black));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_TEXT_FONT_NAME, "Verdana");
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_TEXT_FONT_SIZE, new Integer (12));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_TEXT_FONT_STYLE,
                                          new Integer (Font.PLAIN));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_TEXT_HORIZONTAL_ALIGNMENT,
                                          new Integer (CoreGraphicsUtils.TA_CENTER));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_TEXT_VERTICAL_ALIGNMENT,
                                          new Integer (CoreGraphicsUtils.TA_CENTER));
            _internalNodeLabelStyles.put (GraphicalConcept.CONCEPT_TEXT_DRAWING_MODE,
                                          new Integer (CoreGraphicsUtils.DM_OVERBITMAP));
        }
        return copyStyleSheet(_internalNodeLabelStyles);
    }


    //==============================================
    // CONNECTION STYLES:
    //==============================================
    public static StyleSheet getInitialInteractionStyles ()
    {
        if (_initialInteractionStyles == null) {
            _initialInteractionStyles = new StyleSheet ();
            setConnectionStyleValues(_initialInteractionStyles,
                                      new ExtColor (Color.BLUE),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (2),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_initialInteractionStyles);
    }


    public static StyleSheet getContinuedInteractionStyles ()
    {
        if (_continuedInteractionStyles == null) {
            _continuedInteractionStyles = new StyleSheet ();
            setConnectionStyleValues(_continuedInteractionStyles,
                                      new ExtColor (Color.ORANGE),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (8),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_continuedInteractionStyles);
    }

    public static StyleSheet getLatentInteractionStyles ()
    {
        if (_latentInteractionStyles == null) {
            _latentInteractionStyles = new StyleSheet ();
            setConnectionStyleValues(_latentInteractionStyles,
                                      new ExtColor (Color.BLUE),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (2),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_latentInteractionStyles);
    }


    public static StyleSheet getClassConnectionsStyles ()
    {
        if (_classConnectionsStyles == null) {
            _classConnectionsStyles = new StyleSheet ();
            setConnectionStyleValues(_classConnectionsStyles,
                                      new ExtColor (Color.black),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (1),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_classConnectionsStyles);
    }


    public static StyleSheet getClassDefConnectionStyles ()
    {
        if (_classDefConnectionsStyles == null) {
            _classDefConnectionsStyles = new StyleSheet ();
            setConnectionStyleValues(_classDefConnectionsStyles,
                                      new ExtColor (Color.black),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LT_DOTTED),
                                      new Integer (3),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_classDefConnectionsStyles);
    }


    public static StyleSheet getEquivalenceConnectionStyles ()
    {
        if (_equivalenceConnectionStyles == null) {
            _equivalenceConnectionStyles = new StyleSheet ();
            setConnectionStyleValues(_equivalenceConnectionStyles,
                                      new ExtColor (Color.red),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LT_DOTTED),
                                      new Integer (3),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_equivalenceConnectionStyles);
    }


    public static StyleSheet getPropertyConnectionsStyles ()
    {
        if (_propertyConnectionsStyles == null) {
            _propertyConnectionsStyles = new StyleSheet ();
            setConnectionStyleValues(_propertyConnectionsStyles,
                                      new ExtColor (Color.black),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (1),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_propertyConnectionsStyles);
    }

    public static StyleSheet getOtherConnectionsStyles ()
    {
        if (_otherConnectionsStyles == null) {
            _otherConnectionsStyles = new StyleSheet ();
            setConnectionStyleValues(_otherConnectionsStyles,
                                      new ExtColor (70, 70, 70),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (1),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_otherConnectionsStyles);
    }

    public static StyleSheet getDomainRangeConnectionsStyles ()
    {
        if (_domainRangeConnectionsStyles == null) {
            _domainRangeConnectionsStyles = new StyleSheet ();
            setConnectionStyleValues(_domainRangeConnectionsStyles,
                                      new ExtColor (Color.black),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (2),
                                      new Integer (CoreGraphicsUtils.LT_DASHED));
        }
        return copyStyleSheet(_domainRangeConnectionsStyles);
    }

    public static StyleSheet getClassHierarchyConnectionsStyles ()
    {
        if (_classHierarchyConnectionsStyles == null) {
            _classHierarchyConnectionsStyles = new StyleSheet ();
            setConnectionStyleValues(_classHierarchyConnectionsStyles,
                                      new ExtColor (Color.black),
                                      new Integer (CoreGraphicsUtils.DIT_ALWAYS_IF_TO_CONCEPT),
                                      new Integer (CoreGraphicsUtils.LS_STRAIGHT),
                                      new Integer (1),
                                      new Integer (CoreGraphicsUtils.LT_SOLID));
        }
        return copyStyleSheet(_classHierarchyConnectionsStyles);
    }


    private static void setConceptStyleValues (StyleSheet styleSheet, InteractionStyles.NodeStyle ns)
    {
        if (styleSheet == null) {
            throw new IllegalArgumentException ("Null StyleSheet");
        }
        if (ns == null) {
            throw new IllegalArgumentException ("Null NodeStyle");
        }
        setConceptStyleValues(styleSheet, ns._fontName,
                                           ns._fontSize,
                                           ns._fontStyle,
                                           ns._insets,
                                           ns._bkgColor,
                                           ns._bdrColor,
                                           ns._bdrThick,
                                           ns._bdrType,
                                           ns._bdrShape);
    }

    private static void setConceptStyleValues (StyleSheet styleSheet,
                                               String     fontName,
                                               Integer    fontSize,
                                               Integer    fontStyle,
                                               ExtInsets  insets,
                                               ExtColor   backgroundColor,
                                               ExtColor   borderColor,
                                               Integer    borderThickness,
                                               Integer    borderType,
                                               Integer    borderShape)
    {
        if (styleSheet == null) {
            throw new IllegalArgumentException ("Null stylesheet");
        }
        styleSheet.put (GraphicalConcept.CONCEPT_TEXT_FONT_NAME,   fontName);
        styleSheet.put (GraphicalConcept.CONCEPT_TEXT_FONT_SIZE,   fontSize);
        styleSheet.put (GraphicalConcept.CONCEPT_TEXT_FONT_STYLE,  fontStyle);
        styleSheet.put (GraphicalConcept.CONCEPT_INSETS,           insets);
        styleSheet.put (GraphicalConcept.CONCEPT_BACKGROUND_COLOR, backgroundColor);
        styleSheet.put (GraphicalConcept.CONCEPT_BORDER_COLOR,     borderColor);
        styleSheet.put (GraphicalConcept.CONCEPT_BORDER_THICKNESS, borderThickness);
        styleSheet.put (GraphicalConcept.CONCEPT_BORDER_TYPE,      borderType);
        styleSheet.put (GraphicalConcept.CONCEPT_BORDER_SHAPE,     borderShape);
    }


    private static void setConnectionStyleValues (StyleSheet styleSheet, InteractionStyles.ConnStyle cs)
    {
        if (styleSheet == null) {
            throw new IllegalArgumentException ("Null StyleSheet");
        }
        if (cs == null) {
            throw new IllegalArgumentException ("Null ConnStyle");
        }
        setConnectionStyleValues(styleSheet,
                                  cs._lineColor,
                                  cs._lineDir,
                                  cs._lineStyle,
                                  cs._lineThick,
                                  cs._lineType);
    }

    private static void setConnectionStyleValues (StyleSheet styleSheet,
                                                  ExtColor   lineColor,
                                                  Integer    lineDirection,
                                                  Integer    lineStyle,
                                                  Integer    lineThickness,
                                                  Integer    lineType
                                                  )
    {
        if (styleSheet == null) {
            throw new IllegalArgumentException ("Null stylesheet");
        }
        styleSheet.put (GraphicalConnection.CONNECTION_LINE_COLOR,     lineColor);
        styleSheet.put (GraphicalConnection.CONNECTION_DIRECTION,      lineDirection);
        styleSheet.put (GraphicalConnection.CONNECTION_LINE_STYLE,     lineStyle);
        styleSheet.put (GraphicalConnection.CONNECTION_LINE_THICKNESS, lineThickness);
        styleSheet.put (GraphicalConnection.CONNECTION_LINE_TYPE,      lineType);
    }

    private static StyleSheet copyStyleSheet (StyleSheet styleSheet)
    {
        StyleSheet copy = new StyleSheet();
        for (Enumeration enumKeys = styleSheet.getAllKeys();enumKeys.hasMoreElements();) {
            String key = (String)enumKeys.nextElement();
            copy.put (key, styleSheet.get(key));
        }
        return copy;
    }

    public static void dumpStyleSheet (StyleSheet ss, String txt)
    {
        msgnoh("Style Sheet: "+txt);
        Enumeration keys = ss.getKeys ();
        while (keys.hasMoreElements ()) {
            String key = (String) keys.nextElement();
            Object val = ss.get(key);
            msgnoh("   "+key+":\t"+val.toString());
        }
    }

    private static void msg (String m)
    {
        System.out.println ("CodeStyle: "+m);
    }


    private static void msgnoh (String m)
    {
        System.out.println (m);
    }
}
