/*
 * MonitorMapHelper.java
 *
 * Created on October 6, 2006, 2:54 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package coi.cmap.monitor;

import coi.ontologyAnnotation.OntologyAnnotationConstants;
import coi.relationMonitor.RelationMonitorManager;
import coi.relationMonitor.RelationMonitorService;
import coi.semanticTranslator.SemanticTranslationInfo;
import coi.semanticTranslator.SemanticTranslationService;
import coi.semanticTranslator.SemanticTranslatorManager;
import core.base.Core;
import core.base.Module;
import kaos.core.csi.extension.obligation.ObligationMonitor;
import kaos.core.csi.extension.obligation.ObligationMonitorManager;
import kaos.core.csi.observe.KAoSObserver;
import kaos.core.csi.observe.NotificationException;
import kaos.core.csi.policy.ObligationViolationException;
import kaos.core.service.directory.DefaultKAoSAgentDescription;
import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.service.directory.KAoSEntityDescription;
import kaos.core.util.KAoSConstants;
import kaos.core.util.Msg;
import kaos.core.util.PolicyMsg;
import kaos.core.util.UniqueIdentifier;
import kaos.kpat.tunnel.TunnelClient;
import kaos.policy.information.OntologyPolicyContainer;
import kaos.policy.util.KAoSPolicyBuilderImpl;
import nlk.code.Constants;
import nlk.util.CMapComponent;

import javax.agent.service.directory.AgentDescription;
import java.util.*;

/**
 * @author jlott
 */
public class MonitorMapHelper implements KAoSObserver
{
    /** Creates a new instance of MonitorMapHelper */
    public MonitorMapHelper (String url) throws Exception
    {
        _tunnelClient = new TunnelClient();
        _tunnelClient.connectToDS(url);
    }
    
    public void initialize (CMapComponent cmc, Module module, Core core)
    {
        _module = module;
        _cmc = cmc;
        _mmv = new MonitorMapViewer(cmc, module, core, this);

        cmc.getGraphicalConceptMapRef().getConceptMapRef().put(Constants.IS_MONITORED_MAP, "true");
        // TODO
        
        // Register for updates from KAoS
        Hashtable updateTypes = new Hashtable();
        Hashtable condition = new Hashtable();
        condition.put(KAoSConstants.AGENT_LIST, KAoSConstants.AGENT_LIST);
        updateTypes.put(KAoSConstants.REGISTRATION_UPDATE, condition);
        updateTypes.put(KAoSConstants.DEREGISTRATION_UPDATE, new Hashtable());

        try
        {
            _tunnelClient.registerObserver(updateTypes, this);
        } 
        catch (Exception ex) 
        {
            ex.printStackTrace();
        }
    }
    
    public void entityRegistered (String id, List ontologicalTypes, List domains)
    {
        System.out.println("An entity registered: " + id );
        _mmv.entityRegistered(id, ontologicalTypes, domains);
    }
    
    public void entityDeregistered (String id)
    {
        System.out.println("An entity de-registered: " + id );
        _mmv.entityDeregistered( id );
    }
    
    public void newRelation (String fromId, String fromIdChannel, String toId, String toIdChannel)
    {
        System.out.println("A new relation was formed between: " + fromId + " and " + toId );
        _mmv.newRelation( fromId, fromIdChannel, toId, toIdChannel );
    }
    
    /**
     * Returns a map of entities to which the given node has a relationship
     * fromId = (String) entityId
     * value = RelationInfo
     */
    public Map queryRelations (String fromId, String fromIdChannel) throws Exception
    {        
        Vector args = new Vector(1);
        args.add(fromIdChannel);
        DefaultKAoSAgentDescription desc = (DefaultKAoSAgentDescription) _agentDescriptions.get(fromId);
        if (desc == null )
            return null;
        String guardId = (String) desc.getGuardIDs().get(0);
        
        return (Map) _tunnelClient.invokeKAoSExtensionComponentMethod(RelationMonitorManager.RELATION_MONITOR_MANAGER_TYPE,
                RelationMonitorService.RELATION_MONITOR_SERVICE_URL,
                "queryRelations", args, guardId);
    }

    public List getActions(String id) throws Exception
    {
        Vector args = new Vector(1);
        args.add(id);
        DefaultKAoSAgentDescription desc = (DefaultKAoSAgentDescription) _agentDescriptions.get(id);
        String guardId = (String) desc.getGuardIDs().get(0);
        
        // TODO: fill in repType and ontType appropriately
        return (List) _tunnelClient.invokeKAoSExtensionComponentMethod(OntologyAnnotationConstants.ONTOLOGY_ANNOTATION_MANAGER_TYPE,
                                                                       OntologyAnnotationConstants.ONTOLOGY_ANNOTATION_SERVICE,
                                                                       "getActions", args, guardId);
    }

    public List getInfoObjectTypesForAction(String actionURL, String id) throws Exception
    {
        Vector args = new Vector(2);
        args.add(actionURL);
        args.add(id);
        DefaultKAoSAgentDescription desc = (DefaultKAoSAgentDescription) _agentDescriptions.get(id);
        String guardId = (String) desc.getGuardIDs().get(0);
        
        // TODO: fill in repType and ontType appropriately
        return (List) _tunnelClient.invokeKAoSExtensionComponentMethod(OntologyAnnotationConstants.ONTOLOGY_ANNOTATION_MANAGER_TYPE,
                                                                       OntologyAnnotationConstants.ONTOLOGY_ANNOTATION_SERVICE,
                                                                       "getActions", args, guardId);
    }
    
    /**
     * Add translation info to the given node
     * nodeId - (String) entityId
     * mySemanticTranslationInfo - transaltion information to be added
     */
    public void addTranslationInfo (String nodeId, SemanticTranslationInfo mySemanticTranslationInfo) throws Exception
    {        
        Vector args = new Vector(1);
        args.add(mySemanticTranslationInfo);
        DefaultKAoSAgentDescription desc = (DefaultKAoSAgentDescription) _agentDescriptions.get(nodeId);
        String guardId = (String)desc.getGuardIDs().get(0);
        
        _tunnelClient.invokeKAoSExtensionComponentMethod(SemanticTranslatorManager.SEMANTIC_TRANSLATOR_MANAGER_TYPE,
                                                         SemanticTranslationService.SEMANTIC_TRANSLATION_SERVICE_URL,
                                                         "addTranslationInfo", args, guardId);
    }
    
    public List getActorDomains(String id) throws Exception
    {
        return _tunnelClient.getAgentDescriptionForAgentID(id).getDomainNames();  
    }
    
    public Enumeration getActorOntTypes(String id) throws Exception
    {
        return _tunnelClient.getAgentDescriptionForAgentID(id).getEntityOntologicalTypeNames();  
    }
    
    public Map getActorPoliciesInOWL(String id) throws Exception
    {
        Map owlPolicies = new Hashtable();
        Vector actorPolicies = _tunnelClient.getPoliciesForEntity(id); 
        
        for (Iterator actorPoliciesIT = actorPolicies.iterator(); actorPoliciesIT.hasNext(); )
        {
            //PolicyMsg currentPolicy = (PolicyMsg)actorPoliciesIT.next();
            Msg currentPolicyM = (Msg)actorPoliciesIT.next();
            PolicyMsg currentPolicy = new PolicyMsg(currentPolicyM);
            KAoSPolicyBuilderImpl currentPolicyBuilder = new KAoSPolicyBuilderImpl(currentPolicy);
            OntologyPolicyContainer opc = currentPolicyBuilder.getPolicy();
            opc.currentFormat = OntologyPolicyContainer.OWL_FORMAT;
            String pName = currentPolicy.getName();

            owlPolicies.put(pName, opc.toString());
/*
            if ( owlPolicies.get(currentPolicyBuilder.getPolicyName()) != null)
                owlPolicies.put(currentPolicyBuilder.getPolicyName(), opc.toString());
            else
                owlPolicies.put(currentPolicyBuilder.getPolicyID(), opc.toString());
*/

/*            if ( owlPolicies.get(currentPolicyBuilder.getPolicyName()) != null)
                owlPolicies.put(currentPolicyBuilder.getPolicyName(), currentPolicyBuilder.getPolicyAsOWL());
            else
                owlPolicies.put(currentPolicyBuilder.getPolicyID(), currentPolicyBuilder.getPolicyAsOWL());
                */
        }
        
        return owlPolicies;
    }
    
    public void monitorObligationViolations (String agentId)
    {
        KAoSAgentDescription agentDesc = (KAoSAgentDescription) _agentDescriptions.get(agentId);
        try
        {
            List guardDescs = _tunnelClient.getGuardDescriptionsForEntity(agentDesc.getEntityNameAsString());

            Hashtable updateTypes = new Hashtable();
            Hashtable condition = new Hashtable();
            condition.put(KAoSConstants.AGENT_NAME, agentId);
            updateTypes.put(ObligationMonitor.OBLIGATION_VIOLATION_UPDATE, condition);

            for (Iterator it2 = guardDescs.iterator(); it2.hasNext();) 
            {
                KAoSEntityDescription guardDesc = (KAoSEntityDescription) it2.next();

                _tunnelClient.registerKAoSExtensionComponentObserver(ObligationMonitorManager.OBLIGATION_MONITOR_MANAGER_TYPE,
                                                                     ObligationMonitor.OBLIGATION_MONITOR_URL,
                                                                     updateTypes,
                                                                     guardDesc);
            }
        }
        catch (Exception xcp) 
        {
            xcp.printStackTrace();
        }
        
    }
    
    public void handleObligationViolation (String agentId, ObligationViolationException xcp)
    {
        // todo: notify the monitor map viewer that an obligation has been violated
        List violations = xcp.getViolatedObligations();
        ObligationViolationDlg ovd = new ObligationViolationDlg(agentId, violations);
        ovd.pack();
        ovd.setVisible(true);
    }
    
    // Implements KAoSObserver
    public void update(String observableID, String updateType,  Object updateCondition, Object update) throws NotificationException
    {
        if (updateType.equals(KAoSConstants.REGISTRATION_UPDATE)) 
        {
            Vector descs = new Vector();
            
            if (update instanceof KAoSAgentDescription)
                descs.add(update);
            else if (update instanceof List)
                descs.addAll((List) update);
            
            for (Iterator it = descs.iterator(); it.hasNext(); ) 
            {
                Object desc = it.next();
                
                if (desc instanceof KAoSAgentDescription) 
                {
                    KAoSAgentDescription agentDesc = (KAoSAgentDescription) desc;
                    
                    Vector ontologicalTypes = new Vector();
                    Map ontTypesMap = agentDesc.getEntityOntologicalTypes();
                    
                    if (ontTypesMap != null)
                        ontologicalTypes.addAll(ontTypesMap.keySet());
                    
                    _agentDescriptions.put(agentDesc.getEntityNameAsString(), agentDesc);
                    
                    if (agentDesc.getGuardIDs().isEmpty())
                        continue;
                   

                    try
                    {
                        List guardDescs = _tunnelClient.getGuardDescriptionsForEntity(agentDesc.getEntityNameAsString());
                        
                        Hashtable condition = new Hashtable();
                        
                        condition.put(RelationMonitorService.NEW_RELATION_UPDATE, new Hashtable());
                        condition.put(RelationMonitorService.NEW_PRODUCER_CHANNEL_UPDATE, new Hashtable());
                        condition.put(RelationMonitorService.NEW_CONSUER_CHANNEL_UPDATE, new Hashtable());
                        
                        for (Iterator it2 = guardDescs.iterator(); it2.hasNext();) 
                        {
                            KAoSEntityDescription guardDesc = (KAoSEntityDescription) it2.next();
                            
                            if ( ! _guardIds.contains(guardDesc.getEntityNameAsString())) 
                            {
                                _tunnelClient.registerKAoSExtensionComponentObserver(RelationMonitorManager.RELATION_MONITOR_MANAGER_TYPE,
                                                                                     RelationMonitorService.RELATION_MONITOR_SERVICE_URL,
                                                                                     condition,
                                                                                     guardDesc);
                                
                                _guardIds.add(guardDesc.getEntityNameAsString());
                            }
                        }
                    }
                    catch (Exception xcp) 
                    {
                        xcp.printStackTrace();
                    }
                    
                    entityRegistered(agentDesc.getEntityNameAsString(), ontologicalTypes, agentDesc.getDomainNames());
                }
            }
        }
        else if (updateType.equalsIgnoreCase(KAoSConstants.DEREGISTRATION_UPDATE)) 
        {
            AgentDescription desc = (AgentDescription) update;
            
            if (desc instanceof KAoSAgentDescription) 
            {
                KAoSAgentDescription agentDesc = (KAoSAgentDescription) desc;
                _agentDescriptions.remove(agentDesc.getEntityNameAsString());
                entityDeregistered(agentDesc.getEntityNameAsString());
            }
        }
        else if (updateType.equalsIgnoreCase(RelationMonitorService.NEW_RELATION_UPDATE)) 
        {
            Hashtable table = (Hashtable) update;
            String fromIdChannel = (String) table.keys().nextElement();
            String toIdChannel = (String) table.get(fromIdChannel);
            
            if ((fromIdChannel == null) || (toIdChannel == null))
                return;
            
            String fromId = (String) channelsToProducers.get(fromIdChannel);
            String toId = (String) channelsToConsumers.get(toIdChannel);
            
            if ((fromId == null) || (toId == null))
            {
                // TODO: channels have not been registered!!
                System.out.println("The channels have not been registered... returning");
                return;
            }
            
            newRelation(fromId, fromIdChannel, toId, toIdChannel);
        }
        else if (updateType.equalsIgnoreCase(RelationMonitorService.NEW_PRODUCER_CHANNEL_UPDATE)) 
        {
            handleChannelInfoUpdate(update, RelationMonitorService.PRODUCER_ID, pruducerChannels, channelsToProducers);
        }
        else if (updateType.equalsIgnoreCase(RelationMonitorService.NEW_CONSUER_CHANNEL_UPDATE)) 
        {
            handleChannelInfoUpdate(update, RelationMonitorService.CONSUMER_ID, consumerChannels, channelsToConsumers);
        }
        else if (updateType.equalsIgnoreCase(ObligationMonitor.OBLIGATION_VIOLATION_UPDATE)) {
            Hashtable condition = (Hashtable) updateCondition;
            String agentId = (String) condition.get(KAoSConstants.AGENT_NAME);
            handleObligationViolation(agentId, (ObligationViolationException) update);
        }
        else
        {
            System.out.println("other relation");
        }
    }

    private void handleChannelInfoUpdate(Object update, String actorType, Hashtable actorChannels, Hashtable channelMap) 
    {
        Hashtable table = (Hashtable)update;
        
        String actorId = (String)table.get(actorType);
        
        if (actorId == null)
            return;
        
        String channelId = (String)table.get(actorId);
        
        if (channelId == null)
            return;
        
        if (actorChannels.get(actorId) == null)
            actorChannels.put(actorId, new Hashtable());
        
        Hashtable currentChannels = (Hashtable)actorChannels.get(actorId);
        
        if (currentChannels.get(channelId) != null)
        {
            //TODO: something wrong handle this situation
        }

        String channelType = (String)table.get(channelId);
        
        if (channelType == null)
            channelType = "";
        
        currentChannels.put(channelId, channelType);
        channelMap.put(channelId, actorId);
    }

    public String getID()
    {
        return _id;
    }
    // END KAoSObserver
    
    
    private String _id = UniqueIdentifier.GenerateUID();
    private Module _module;
    private Hashtable _agentDescriptions = new Hashtable();
    private TunnelClient _tunnelClient;
    private CMapComponent _cmc;
    private HashSet _guardIds = new HashSet();
    private Hashtable pruducerChannels = new Hashtable();
    public Hashtable channelsToProducers = new Hashtable();
    private Hashtable consumerChannels = new Hashtable();
    public Hashtable channelsToConsumers = new Hashtable();

    public MonitorMapViewer getMmv()
    {
        return _mmv;
    }

    private MonitorMapViewer _mmv;
}
