package coi.jbi.client;

import org.infospherics.jbi.client.exception.ConnectionException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;

import java.util.Enumeration;
import java.util.Hashtable;

public class ConnectionManager implements  org.infospherics.jbi.client.ConnectionManager
{ 
	private mil.af.rl.im.capi.client.impl.ConnectionManager 	_manager;	
	private Hashtable											_connections;
	
	public ConnectionManager() 
    {
		_manager = new mil.af.rl.im.capi.client.impl.ConnectionManager();
		_connections = new Hashtable();
	}

	/**
	 * 
	 *The client uses this method to select a platform from the list and supply a PlatformDescriptor as the connectionDescriptor in the createConnection method.
	 *<br><br>
	 * @param constraints This method would take a an XML document that conforms to the following structure:
	 * <PlatformDescriptor>
	 * 	...descriptor attributes
	 * </PlatformDescriptor>
	 * @return The returned String[] would contain all PlatformDescriptors (as XML) that match the supplied criteria. If null is supplied then the method returns all registered platforms.
	 *
	 * @throws PlatformFailureException
	 * @throws PermissionDeniedException
	 */
	public String[] getPlatforms(String constraints) throws PlatformFailureException,  PermissionDeniedException
    {
		return _manager.getPlatforms(constraints);
    }

	/**
	 * This method is used to create a Connection to a particular JBI platform implementation The platform descriptor is meant to be an XML document
	 * that describes the particular platform implementation. In the future, this document will allow clients to potentially connect to a number
	 * possibly disparate platform implementations within the same client. Currently an legal string can be used to describe the particular platform.
	 * 
	 * @param connectionName
	 *          a client name provided for the connection
	 * @param connectionDescriptor
	 *          XML document that describes the particular platform implementation.
	 * 
	 * @return <b>Connection </b>- the connection interface created that will be used to interact with the platform
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public org.infospherics.jbi.client.Connection createConnection(String connectionName, String connectionDescriptor) 
		throws 	PlatformFailureException, PermissionDeniedException, ConnectionException
    {
		org.infospherics.jbi.client.Connection connection = _manager.createConnection(connectionName, connectionDescriptor);
		coi.jbi.client.Connection kwConnection = new coi.jbi.client.Connection(connection);
		_connections.put(connectionName,kwConnection);
        
		return kwConnection;
    }
		  
	/**
	 * This method is used to destroy a connection created for a particular JBI platform implementation. If the connection is active and sequences exist
	 * on the connection the sequences will be destroyed before the client disconnects from the specified connection.
	 * 
	 * @param connection
	 *          a previously created connection
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public void destroyConnection(org.infospherics.jbi.client.Connection connection) 
		throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {
		_connections.remove(connection);
		_manager.destroyConnection(connection);
    }

	/**
	 * This method returns all of the current connections that the client has to all JBI platforms.
	 * 
	 * @return <b>Connection[] </b>- an array of connection interfaces that exist within the client application
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public org.infospherics.jbi.client.Connection[] getConnections( ) throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {
		org.infospherics.jbi.client.Connection[] connections = new org.infospherics.jbi.client.Connection[_connections.size()];
		Enumeration connectionsEnum = _connections.elements();
        
		for (int i = 0; i < _connections.size(); i++)		
			connections[i] = (org.infospherics.jbi.client.Connection) connectionsEnum.nextElement();		

		return connections;
    }

	/**
	 * This method returns a connection that corresponds to the provided connection name or a null if a connection with that name has not been created.
	 * 
	 * @param connectionName
	 *          a client provided connection name.
	 * 
	 * @return <b>Connection </b>- the connection that is associated with the specified name
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public org.infospherics.jbi.client.Connection getConnection(String connectionName) 
        throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {
		return (org.infospherics.jbi.client.Connection) _connections.get(connectionName);  
    }
}
