package coi.jbi.client;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.SubscriberSequence;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.StaleSequenceUIDException;
import org.infospherics.jbi.client.exception.UnimplementedException;

import coi.relationMonitor.RelationMonitorService;

import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.KAoSGuard;

/**
 * This class is used for subscription to a JBI service. It receives a callback from JBI each time
 * new data is available to a subscribed client. This COI base class reports each JBI callback to
 * the COI RelationMonitorService.
 *
 * All COI subscribers must use a callback of this type.
 *
 * @author lbunch
 */
public class ObjectAvailableCallback implements org.infospherics.jbi.client.ObjectAvailableCallback
{
    protected RelationMonitorService _relationMonitorService = null;
    protected SubscriberSequence _subscriberSeq = null;
    protected String _subscriptionOntClassURI = null;
    protected String channelId = null;

    public ObjectAvailableCallback(SubscriberSequence mySubscriberSequence, String subscriptionOntClassURI)
    {
        KAoSGuard guard = (KAoSGuard)GuardRetriever.getGuard();
        _subscriberSeq = mySubscriberSequence;
        _subscriptionOntClassURI = subscriptionOntClassURI;
        _relationMonitorService = (RelationMonitorService)guard.
        getKAoSExtensionComponents("RelationMonitor", RelationMonitorService.RELATION_MONITOR_SERVICE_URL);

        try
        {
            channelId = ((mil.af.rl.im.capi.client.impl.SubscriberSequence)mySubscriberSequence).getSequenceUID();
        }
        catch (StaleSequenceUIDException e)
        {
            e.printStackTrace();
        }
        catch (UnimplementedException e)
        {
            e.printStackTrace();
        }
        catch (PlatformFailureException e)
        {
            e.printStackTrace();
        }
        catch (PermissionDeniedException e)
        {
            e.printStackTrace();
        }
    }

    public void objectsAvailable(InfoObject[] objects)
    {
        if (objects != null && objects.length != 0)
        {
            for (int i = 0; i < objects.length; i++)
            {
                InfoObject obj = objects[i];

                try
                {
                    InfoObjectHelper infoObjHelper = new InfoObjectHelper(obj);
                    String infoObjType = infoObjHelper.getTypeName();
                    if (infoObjType.equals(_subscriberSeq.getInfoObjectTypeName()))
                    {
                        String pubId = infoObjHelper.getPublisherID();

                        // report the relation
                        if ((_relationMonitorService != null) && (channelId != null) && !channelId.equals(pubId))
                        {
                            _relationMonitorService.reportRelation(pubId, channelId, _subscriptionOntClassURI/*infoObjHelper.getOntClassURI()*/, true);
                            System.out.println("Reported Relation " + pubId + "   " + channelId + "   " + _subscriptionOntClassURI/*infoObjHelper.getOntClassURI()*/);
                        }
                        else
                           System.out.println("\n\n\tNULL RelationMonitor Service");
                    }
                    else
                    {
                        // received data I wasn't subscribed to - need xpath filter
                        System.out.println("received data I wasn't subscribed to - need xpath filter to exclude " + infoObjType);
                    }
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
        }
    }
}