package coi.jbi.demo.metoc;

import java.util.Map;
import java.util.HashMap;

import kaos.core.csi.OntInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;
import kaos.ontology.util.OntModelUtils;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.PublisherSequence;
import org.infospherics.jbi.client.SubscriberSequence;
import org.infospherics.jbi.client.exception.ConnectionException;
import org.infospherics.jbi.client.exception.InvalidAttributeXMLException;
import org.infospherics.jbi.client.exception.InvalidPredicateException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.PredicateLanguageException;
import org.infospherics.jbi.client.exception.SequenceStateException;
import org.infospherics.jbi.client.exception.UnknownAttributeException;
import org.infospherics.jbi.client.exception.UnrecognizedObjectTypeException;
import org.infospherics.jbi.client.exception.UnsupportedVersionException;

import com.hp.hpl.jena.shared.PrefixMapping;


import coi.jbi.interceptors.JBIBaseClient;

import org.infospherics.jbi.client.ObjectAvailableCallback;

// WEATHER SENSOR REPORT PUBLISHER
// MISSION FORECAST SUBSCRIBER
// WEATHER WARNING SUBSCRIBER
public class GenericMETOCClient extends JBIBaseClient
{
    protected PublisherSequence weatherSensorReportPub = null;
    protected SubscriberSequence missionForecastSub = null;
    protected SubscriberSequence weatherWarningSub = null;

    public static void main (String[] args)
    {
        try
        {
            // Default number of iterations and period.
            int iter = 2000;
            int sleep = 30000;

            // Create the client, the name will be used to find a jbi-clients/credentials/name.xml file
            GenericMETOCClient testClient = new GenericMETOCClient();

            testClient.initCommunications();
            
            // Publish <iter> times, sleeping <sleep> millisecs between each publish
            testClient.startPublishing(iter, sleep);

            // Close the JBI connection
            testClient.disconnect();

        } 
        catch (Exception e1)
        {
            e1.printStackTrace();
        }

        // Really nifty and impressive message.
        System.out.println("The End!");
    }

    public GenericMETOCClient()
    {
        super();
    }

    // Initilize Publish and subscription channels
    public void initCommunications() throws UnrecognizedObjectTypeException, PermissionDeniedException, PlatformFailureException, SequenceStateException, 
        UnsupportedVersionException, InvalidPredicateException, PredicateLanguageException, ConnectionException
    {
        // Get the JBI connection
        connect();

        // --------- WEATHER SENSOR REPORT PUBLISHER ----------
        weatherSensorReportPub = initPublisherChannel(OntologyConcepts.WEATHER_SENSORREPORT_JBI_TYPE, "1.0"); // Publish Sequence refs.
        
        // Activate publishing
        weatherSensorReportPub.activateSequence();

        // ---------- MISSION FORECAST SUBSCRIBER ----------
        missionForecastSub = initSubscriberChannel(OntologyConcepts.MISSION_CONTROL_FORECAST_TYPE, "1.0", null); // Subscriber Sequence refs.
        
        // Create a callback to be used to notify the subscribing client when an object is delivered that matches her subscription.
        ObjectAvailableCallback forecastCallback = new MissionForecastCallback(); //OntologyConcepts.MISSION_CONTROL_FORECAST_ACTION_CLASS)
        
        // Set the callback associated with the sequence.
        missionForecastSub.setSequenceCallback(forecastCallback);
        
        // Activate subscription
        missionForecastSub.activateSequence();

        // ---------- WEATHER WARNING SUBSCRIBER ----------
        weatherWarningSub = initSubscriberChannel(OntologyConcepts.WEATHER_WARNING_JBI_TYPE, "1.0", null); // Subscriber Sequence refs.
        
        // Create a callback to be used to notify the subscribing client when an object is delivered that matches her subscription.
        ObjectAvailableCallback warningCallback = new MissionForecastCallback(); // OntologyConcepts.WEATHER_WARNING_ACTION_CLASS
        
        // Set the callback associated with the sequence.
        weatherWarningSub.setSequenceCallback(warningCallback);
        
        // Activate subscription
        weatherWarningSub.activateSequence();
    }

    public void publishWeatherSensorReport(double airTempFarenheit, double windSpeedMPH, double visibilityMeters)
    {
        try
        {
            OntInstanceDescriptionImpl airTempInstance = 
                new OntInstanceDescriptionImpl("http://localhost/default#SensorReport1AirTemperature", OntologyConcepts.WEATHER_CLASS_AIR_TEMP);
            OntPropertyDescriptionImpl hasValue = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_HAS_VALUE);
            hasValue.addValue(String.valueOf(airTempFarenheit));
            OntPropertyDescriptionImpl hasUnits = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_HAS_UNITS);
            hasUnits.addValue("degreesFarhrenheit");
            airTempInstance.addProperty(hasValue);
            airTempInstance.addProperty(hasUnits);
            OntPropertyDescription containsAirTempProp = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_CONTAINS);
            containsAirTempProp.setValueDescForValue(airTempInstance.getName(), airTempInstance);

            OntInstanceDescriptionImpl windSpeedInstance = 
                new OntInstanceDescriptionImpl("http://localhost/default#SensorReport1WindSpeed", OntologyConcepts.WEATHER_CLASS_WIND_SPEED_AVG);
            hasValue = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_HAS_VALUE);
            hasValue.addValue(String.valueOf(windSpeedMPH));
            hasUnits = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_HAS_UNITS);
            hasUnits.addValue("MilesPerHour");
            windSpeedInstance.addProperty(hasValue);
            windSpeedInstance.addProperty(hasUnits);
            OntPropertyDescription containsWindSpeedProp = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_CONTAINS);
            containsWindSpeedProp.setValueDescForValue(windSpeedInstance.getName(), windSpeedInstance);

            OntInstanceDescriptionImpl visibilityInstance = 
                new OntInstanceDescriptionImpl("http://localhost/default#SensorReport1Visibility", OntologyConcepts.WEATHER_CLASS_VISIBILITY);
            hasValue = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_HAS_VALUE);
            hasValue.addValue(String.valueOf(visibilityMeters));
            hasUnits = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_HAS_UNITS);
            hasUnits.addValue("Meters");
            visibilityInstance.addProperty(hasValue);
            visibilityInstance.addProperty(hasUnits);
            OntPropertyDescription containsVisibProp = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_CONTAINS);
            containsVisibProp.setValueDescForValue(visibilityInstance.getName(), visibilityInstance);

            OntInstanceDescriptionImpl weatherSensorReport = 
                new OntInstanceDescriptionImpl("http://localhost/default#SensorReport1", OntologyConcepts.WEATHER_SENSORREPORT_ACTION_CLASS);

            // Start broken nsprefix: ensure jena uses the expected namespace prefixes
            Map namespaces = new HashMap();
            namespaces.put(OntologyConcepts.WEATHER_PROPERTIES_NSPREFIX, OntologyConcepts.WEATHER_PROPERTIES_NAMESPACE);
            namespaces.put(OntologyConcepts.WEATHER_PUBLICATIONS_NSPREFIX, OntologyConcepts.WEATHER_PUBLICATIONS_NAMESPACE);
            PrefixMapping pm = weatherSensorReport.getInstanceOntModel().setNsPrefixes(namespaces);
            // End broken nsprefix

            weatherSensorReport.addProperty(containsAirTempProp);
            weatherSensorReport.addProperty(containsWindSpeedProp);
            weatherSensorReport.addProperty(containsVisibProp);

            String weatherSensorReportString = OntModelUtils.deconstructOntModel(weatherSensorReport.getInstanceOntModel());
            
            // Start namespace workaround
            weatherSensorReportString = weatherSensorReportString.replaceFirst("xmlns:j.1", "xmlns:n1");
            weatherSensorReportString = weatherSensorReportString.replaceFirst("xmlns:j.2", "xmlns:n2");
            weatherSensorReportString = weatherSensorReportString.replaceAll("j.1:", "n1:");
            weatherSensorReportString = weatherSensorReportString.replaceAll("j.2:", "n2:");
            weatherSensorReportString = weatherSensorReportString.replaceAll("&amp;", "&amp;amp;");
            // End namespace workaround

            // Create an information object using the connection.
            InfoObject io = 
                getConnection().createInfoObject(OntologyConcepts.WEATHER_SENSORREPORT_JBI_TYPE, "1.0", new byte[0]/*no payload*/, weatherSensorReportString);

            // Publish the object
            weatherSensorReportPub.publishInfoObject(io);
        }
        catch (Exception ie)
        {
            System.out.println("Exception on publish! " + ie.getMessage());
            ie.printStackTrace();
        }
    }

    public void generateAndpublishRandomWeatherSensorReport()
    {
        double rndAirTmp = Math.ceil(Math.random() * 100);
        double rndWndSpeed = Math.ceil(Math.random() * 100);
        double rndVisib = Math.ceil(Math.random() * 1000);

        publishWeatherSensorReport(rndAirTmp, rndWndSpeed, rndVisib);
    }
    
    public void startPublishing(int iter, int sleep) throws PermissionDeniedException, PlatformFailureException, UnsupportedVersionException, 
        InvalidAttributeXMLException, UnknownAttributeException, SequenceStateException
    {
        for (int i = 0; i < iter; i++)
        {
            // Actual Information Object creation and publication
            generateAndpublishRandomWeatherSensorReport();

            // Sleep for a time then create and publish again...
            try
            {
                if (Math.random() > 0.75)
                    Thread.sleep(sleep);
                else
                    // Introduced obligation violation 
                    Thread.sleep(2 * sleep);
            }
            catch (Exception ie)
            {
                System.out.println("Exception on publish! " + ie.getMessage());
                ie.printStackTrace();
            }
        }
    }
}