package coi.jbi.demo.metoc;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Iterator;

import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.OntInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;
import kaos.ontology.util.OntModelUtils;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.PublisherSequence;
import org.infospherics.jbi.client.SubscriberSequence;
import org.infospherics.jbi.client.exception.ConnectionException;
import org.infospherics.jbi.client.exception.InvalidPredicateException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.PredicateLanguageException;
import org.infospherics.jbi.client.exception.SequenceStateException;
import org.infospherics.jbi.client.exception.UnrecognizedObjectTypeException;
import org.infospherics.jbi.client.exception.UnsupportedVersionException;

import org.infospherics.jbi.client.ObjectAvailableCallback;

import com.hp.hpl.jena.shared.PrefixMapping;

import mil.af.rl.im.capi.client.util.Utils;

import coi.jbi.interceptors.JBIBaseClient;
import coi.jbi.client.InfoObjectHelper;


// WEATHER SENSOR REPORT SUBSCRIBER
// WEATHER WARNING PUBLISHER
// MISSION FORECAST PUBLISHER
public class JMFUKorea extends JBIBaseClient
{
    SubscriberSequence weatherSensorReportSub = null;
    PublisherSequence weatherWarningPub = null;
    PublisherSequence missionForecastPub = null;

    public static void main (String[] args)
    {
        // Create the client, the name will be used to find a jbi-clients/credentials/name.xml file
        JMFUKorea testClient = new JMFUKorea();

        try
        {
            // Start publishing and subscribing via JBI
            testClient.initCommunications();

            // Superclass BaseClient adds shutdown hooks to deregister from KAoS and disconnect from JBI
        }
        catch (Exception e1)
        {
            e1.printStackTrace();
            testClient.disconnect();
        }
    }

    public JMFUKorea()
    {
        super();
    }

    public void initCommunications() throws UnrecognizedObjectTypeException, PermissionDeniedException, PlatformFailureException, SequenceStateException,
        UnsupportedVersionException, InvalidPredicateException, PredicateLanguageException, ConnectionException
    {
        // Get the JBI connection
        connect();
      
        // ------ WEATHER SENSOR REPORT SUBSCRIBER
        weatherSensorReportSub = initSubscriberChannel(OntologyConcepts.WEATHER_SENSORREPORT_JBI_TYPE, "1.0", null); // Subscriber Sequence refs.
        
        // Create a callback to be used to notify the subscribing client when an object is delivered that matches her subscription.
        WeatherWarningChecker callback = new WeatherWarningChecker(weatherSensorReportSub, OntologyConcepts.WEATHER_SENSORREPORT_ACTION_CLASS);
        
        // Set the callback associated with the sequence.
        weatherSensorReportSub.setSequenceCallback(callback);
        
        // Activate subscription
        weatherSensorReportSub.activateSequence();

        // ------- WEATHER WARNING PUBLISHER
        weatherWarningPub = initPublisherChannel(OntologyConcepts.WEATHER_WARNING_JBI_TYPE, "1.0"); // Publish Sequence refs.
        weatherWarningPub.activateSequence();

        // MISSION FORECAST PUBLISHER
        missionForecastPub = initPublisherChannel(OntologyConcepts.MISSION_CONTROL_FORECAST_TYPE, "1.0"); // Publish Sequence refs.
        missionForecastPub.activateSequence();

    }

    public void publishWeatherWarning(String warningMessage)
    {
        // publish weather warning for severe conditions
        //        String xmlConfigPath = System.getProperty("sequence.xml.path");
        //        String fs = System.getProperty("file.separator");
        //        String weatherWarningMeta = Utils.readFile(xmlConfigPath + fs + "sequenceXML" + fs + "publish" + fs
        //                + OntologyConcepts.WEATHER_WARNING_TYPE + ".owl");
        //        // Create an information object using the connection.
        //        InfoObject io;
        //        try
        //        {
        //            io = getConnection().createInfoObject(OntologyConcepts.WEATHER_WARNING_TYPE, "1.0", new byte[0]/*no payload*/, weatherWarningMeta);
        //            // Publish the object
        //            weatherWarningPub.publishInfoObject(io);
        //        }
        //        catch (Exception e)
        //        {
        //            e.printStackTrace();
        
        //        }
        try
        {
            OntInstanceDescriptionImpl warningAction = 
                new OntInstanceDescriptionImpl("http://localhost/default#WeatherWarning1", OntologyConcepts.WEATHER_WARNING_ACTION_CLASS);

            OntPropertyDescription warningHasText = new OntPropertyDescriptionImpl(OntologyConcepts.WEATHER_PROPERTY_HAS_TEXT);
            warningHasText.addValue(warningMessage);

            // Start broken nsprefix: ensure jena uses the expected namespace prefixes
            HashMap namespaces = new HashMap();
            namespaces.put(OntologyConcepts.WEATHER_PROPERTIES_NSPREFIX, OntologyConcepts.WEATHER_PROPERTIES_NAMESPACE);
            namespaces.put(OntologyConcepts.WEATHER_PUBLICATIONS_NSPREFIX, OntologyConcepts.WEATHER_PUBLICATIONS_NAMESPACE);
            PrefixMapping pm = warningAction.getInstanceOntModel().setNsPrefixes(namespaces);
            // End broken nsprefix

            warningAction.addProperty(warningHasText);

            String warningActionString = OntModelUtils.deconstructOntModel(warningAction.getInstanceOntModel());
            
            // Start namespace workaround
            warningActionString = warningActionString.replaceFirst("xmlns:j.1", "xmlns:n1");
            warningActionString = warningActionString.replaceFirst("xmlns:j.2", "xmlns:n2");
            warningActionString = warningActionString.replaceAll("j.1:", "n1:");
            warningActionString = warningActionString.replaceAll("j.2:", "n2:");
            warningActionString = warningActionString.replaceAll("&amp;", "&amp;amp;");
            // End namespace workaround

            //lkb can't due to jbi server parse error            wsrActionString = wsrActionString.replaceAll("&amp;amp;", "&amp;");
            // Create an information object using the connection.
            InfoObject io = getConnection().createInfoObject(
                    OntologyConcepts.WEATHER_WARNING_JBI_TYPE, "1.0", new byte[0]/*no payload*/, warningActionString);

            weatherWarningPub.publishInfoObject(io);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public void publishMissionControlForecast()
    {
        // Publish mission control forecast
        String xmlConfigPath = System.getProperty("sequence.xml.path");
        String fs = System.getProperty("file.separator");
        String forecastMeta = Utils.readFile(xmlConfigPath + fs + "sequenceXML" + fs + "publish" + fs
                + OntologyConcepts.MISSION_CONTROL_FORECAST_TYPE + ".owl");

        try 
        {
            OntInstanceDescriptionImpl mcfAction = new OntInstanceDescriptionImpl(forecastMeta);
            mcfAction.getProperty("http://norma.ihmc.us:8082/servlet/SBReadResourceServlet?rid=1166317683986_976042720_3066&amp;partName=owl#MCForecast1Element1");
        } 
        catch (URISyntaxException e1) 
        {
            e1.printStackTrace();
        }

        // Create an information object using the connection.
        InfoObject io;
        try
        {
            io = getConnection().createInfoObject(OntologyConcepts.MISSION_CONTROL_FORECAST_TYPE, "1.0", new byte[0]/*no payload*/, forecastMeta);
            
            // Publish the object
            missionForecastPub.publishInfoObject(io);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Checks the WeatherSensorReport subscriber feed for severe conditions and publishes a WeatherWarning.
     *
     * @author lbunch
     */
    public class WeatherWarningChecker implements ObjectAvailableCallback
    {
        public WeatherWarningChecker(SubscriberSequence mySubscriberSequence, String subscriptionOntClassURI)
        {
        }

        public void objectsAvailable(InfoObject[] objects)
        {
            for (int i = 0; i < objects.length; i++)
            {
                try 
                {
                    InfoObjectHelper infoObj = new InfoObjectHelper(objects[i]);
                    String metaOWL = infoObj.getMetadata();

                    OntInstanceDescriptionImpl aid = new OntInstanceDescriptionImpl(metaOWL);

                    // Check for severe conditions in weather sensor report subscription
                    // specifically, if wind speed >= 60mph or visibility less than 500meters
                    OntPropertyDescription containsProp = aid.getProperty(OntologyConcepts.WEATHER_PROPERTY_CONTAINS);
                    
                    if (containsProp == null)
                        continue; // Go on to process the next info object
                    
                    for(Iterator containsValuesIter = containsProp.getValues().iterator(); containsValuesIter.hasNext(); )
                    {
                        String containsInstName = (String)containsValuesIter.next();
                        OntInstanceDescriptionImpl containsInst = (OntInstanceDescriptionImpl)containsProp.getValueDescForValue(containsInstName);
                        
                        if (containsInst != null && containsInst.getTypes().contains(OntologyConcepts.WEATHER_CLASS_WIND_SPEED_AVG))
                        {
                            OntPropertyDescription windSpeedProp = containsInst.getProperty(OntologyConcepts.WEATHER_PROPERTY_HAS_VALUE);
                            String windSpeedStr = (String)windSpeedProp.getValues().iterator().next();
                            double windSpeed = Double.valueOf(windSpeedStr).doubleValue();
                            
                            if (windSpeed >= 60)
                                publishWeatherWarning("Wind speed too high for safe landing!");
                        }
                        else if (containsInst != null && containsInst.getTypes().contains(OntologyConcepts.WEATHER_CLASS_VISIBILITY))
                        {
                            OntPropertyDescription visibProp = containsInst.getProperty(OntologyConcepts.WEATHER_PROPERTY_HAS_VALUE);
                            String visibStr = (String)visibProp.getValues().iterator().next();
                            double visibMeters = Double.valueOf(visibStr).doubleValue();
                            
                            if (visibMeters <= 500)
                                publishWeatherWarning("Visibility too low for safe landing!");
                        }
                    }
                } 
                catch (Exception e) 
                {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Publishes a Mission Control Forecast once every 30 secs until the agent is killed.
     *
     * @author lbunch
     */
    public class MissionControlForecastPubThread extends Thread
    {
        public MissionControlForecastPubThread()
        {
            super ("MissionControlForecastPubThread");
        }

        public void run()
        {
            while(true)
            {
                try 
                {
                    Thread.sleep(30000);
                } 
                catch (InterruptedException e) 
                {
                }
                
                publishMissionControlForecast();
            }
        }
    }
}