
package coi.jbi.interceptors;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import mil.af.rl.im.capi.client.core.exceptions.ReceptorAlreadyRegisteredException;
import mil.af.rl.im.capi.client.core.impl.ReceptorConfigDAO;
import mil.af.rl.im.capi.client.util.Logger;

import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.infospherics.jbi.client.exception.PlatformFailureException;

import mil.af.rl.im.capi.client.impl.util.ClientProperties;
/**
 * This class is abesed on the original JBI class creating and managing parameters provided by clients that connect to the infoshpere.
 * The role of this class is to read parameteres from the original JBI client.properties file. This only singnificant change it this class is 
 * new vale for fileName which is set to  "clientOriginal.properties".
 * 
 * <br>
 * <br>
 * AFRL provides this Software to you on an "AS IS" basis, without warranty of any kind. AFRL HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES OR CONDITIONS,
 * EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. You are solely responsible for determining the appropriateness of using this Software and assume all risks associated with the use of this
 * Software, including but not limited to the risks of program errors, damage to or loss of data, programs or equipment, and unavailability or
 * interruption of operations.
 * 
 * @author Vaughn T. Combs <br>
 * @version JBIConfigParams version 0.5
 */

/*
 * Revisions:
 * 
 * $Log: JBIOriginalConfigParams.java,v $
 * Revision 1.2  2007/06/08 19:19:47  auszok
 * Extracted KAoS fiunctionality into KAoSInterceptor class, added policy checking to many JBI CAPI calls, added comments and cleaned the code.
 *
 * Revision 1.1  2007/06/07 20:46:30  auszok
 * Implemention of dynamic loading of original capi classes in the KAoS interceptor layer for JBI client
 *
 * Revision 1.3  2006/11/09 08:23:08  grantr
 * Added a complete client runtime that dynamically loads the simplified publisher
 *
 * Revision 1.2  2006/10/18 02:35:51  grantr
 * fixed a bug in which an error will now be thrown if you the client.properties cannot be found
 *
 * Revision 1.1  2006/03/16 04:30:09  grantr
 * *** empty log message ***
 *
 * Revision 1.4  2005/05/06 10:30:44  combsv
 * Changes made to accomodate and use the new platform discovery capability and the new connection descriptor. We ACTUALLY use one now!
 *
 * Revision 1.3  2005/04/06 06:30:55  grantr
 * Organized imports
 *
 * Revision 1.2  2005/03/31 06:43:16  grantr
 * implemented more of the new typemgt packages
 * Revision 1.1 2005/03/29 11:05:07 grantr Hail Mary, here comes the motherload of CAPI 1.5 Changes
 * 
 * Revision 1.20 2005/03/21 11:12:56 vincelec Undid the reformatting of the source. :) Rob told me to just hit commit.
 * 
 * Revision 1.18 2005/02/25 07:28:53 grantr added som instrumentation stuff
 * 
 * Revision 1.17 2005/01/11 09:46:27 grantr fixed some admin stuff
 * 
 * Revision 1.16 2005/01/07 11:19:28 combsv Rob... DONT CHECK IN YOUR DIAGNOTIC CODE... If the Admin subelement tag is not present it blows up the app
 * with NullPointerException.
 * 
 * Revision 1.15 2005/01/05 09:37:11 grantr fixed some administration stuff
 * 
 * Revision 1.14 2005/01/04 09:53:14 grantr removed administration exception
 * 
 * Revision 1.13 2005/01/04 08:55:58 grantr added administration stuff
 * 
 * Revision 1.12 2004/06/24 09:59:47 combsv Additions and changes to accomodate multiple endpoints (connections to multiple platforms). For now this
 * can be done but the clients must utilize the same connection technologies for dynamically loaded CAPI (WS, J2EE, etc.).
 * 
 * Revision 1.11 2004/06/21 10:39:38 combsv Changes to accomodate connection to multiple JBIs.
 * 
 * Revision 1.10 2004/05/27 12:28:23 vincelec Better error message provided when the server.keystore file cannot be found.
 * 
 * Revision 1.9 2004/05/27 05:17:01 combsv Addition of code to individually load CAPI constructs.
 * 
 * Revision 1.8 2004/05/19 06:05:47 vincelec removed debug comments.
 * 
 * Revision 1.7 2004/05/17 03:29:12 vincelec Replaced J2EE specific properties with the generic "end.point" moniker. end.point can contain any valid
 * subset of xml and its up to the individual implementation plug-ins to make sense of it.
 * 
 * Revision 1.6 2004/05/13 04:59:03 vincelec Added a ReceptorManager ref. Which stores all the receptor configurations in the client.properties.
 * 
 * Revision 1.5 2004/05/04 07:58:32 combsv New subelement processing for client.properties.
 * 
 * Revision 1.4 2004/02/24 04:01:52 combsv *** empty log message ***
 * 
 * Revision 1.3 2004/02/17 06:20:55 vincelec Optimized imports
 * 
 * Revision 1.2 2004/01/14 13:26:01 vincelec Cleaned up javadoc.
 * 
 * Revision 1.1 2004/01/14 11:59:31 vincelec Initial JBI Platform 1.2 Source tree checkin
 * 
 * Revision 1.1 2004/01/12 07:15:33 vincelec Common api 1.2 Pre Merge Snapshot
 * 
 * Revision 1.2 2003/09/09 06:06:39 vincelec <No Comment Entered>
 * 
 *  
 */

public class JBIOriginalConfigParams
{
  // these variables you will find in the client.properties files as xml elements. The text information
  // associated with each element is the plugable classes of each component.
  private static Properties props = System.getProperties();

  private static final String rootElem = "client_properties";
  private static final String jndiPropElem = "jndi_properties";
  private static final String secPropElem = "security_properties";
  private static final String capiPropElem = "capi_properties";
  private static final String simPropElem = "simplified_properties";
  private static final String endPointElem = "end.point";
  private static final String simEndPointElem = "simplified_endpoint";
  private static final String simIPElem = "sim_ip";
  private static final String administrationPropElem = "admin_properties";
  private static final String xpathSeparator = "/";
  private static final String fileSeparator = props.getProperty("file.separator");
  private static final String fileName = "clientOriginal.properties";
  private static final String configPropName = "jbi.client.properties.file";
  private static final String configPropDir = "jbi.client.properties.dir";

  private static String filePath = "clientOriginal.properties";
  private static ClientProperties client = null;

  // current allowable levels within the receptor, proliferator, and configurator
  // sub elements of client.properties
  private static String connectionLevel = "/connection";

  private static String subscriberLevel = "/subscriber";

  static
  {
    // if they wish to change the name or the client.properties file.
    String name = props.getProperty("jbi.client.properties.file");
    // if they wish to change the directory where the file lives.
    String dir = props.getProperty("jbi.client.properties.dir");

    if (dir == null)
      dir = "";
    else
      dir = dir + JBIOriginalConfigParams.fileSeparator;

    if (name == null)
      filePath = dir + fileName;
    else
      filePath = dir + name;
  }

  public static ClientProperties getClientProperties( )
  {
    if (client == null)
    {
        try 
        {
            constructJBIProperties();
        } 
        catch (Exception e) 
        {
            e.printStackTrace();
            
            return client;
        }
        
        client.setAsProperties();
    }
    
    return client;
  }

  /**
   * reads the client.properties file for the plugable components.
   */
  public static void constructJBIProperties( )throws Exception
  {
    SAXReader reader = new SAXReader();
    File config = new File(filePath);
    
    if(!config.exists())
    {
    	throw new Exception("The " + config.getName() + " at absolute path " + config.getAbsolutePath() + "does not exhist!");
    }
    Document doc = null;
    Node node = null;

    try
    {
      doc = reader.read(config);
    }
    catch (Exception e)
    {
      Logger.log("Problem encountered with the client.properties file that has been provided!\n" + e.getMessage(), true, true, 0);
    }

    // use XPath to walk to the jndi, security, capi properties node
    String jndiPath = JBIOriginalConfigParams.xpathSeparator + rootElem + xpathSeparator + JBIOriginalConfigParams.endPointElem + JBIOriginalConfigParams.xpathSeparator
        + jndiPropElem + JBIOriginalConfigParams.xpathSeparator;

    // create and populate ClientProperties object
    JBIOriginalConfigParams.client = new ClientProperties();

    String capiPath = JBIOriginalConfigParams.xpathSeparator + rootElem + JBIOriginalConfigParams.xpathSeparator + capiPropElem + JBIOriginalConfigParams.xpathSeparator;
    
    String simPath = JBIOriginalConfigParams.xpathSeparator + rootElem + JBIOriginalConfigParams.xpathSeparator + simPropElem + JBIOriginalConfigParams.xpathSeparator;
    
    String simEndpointPath = JBIOriginalConfigParams.xpathSeparator + rootElem + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiEndpoints
    + xpathSeparator + ClientProperties.capiEndpoint + JBIOriginalConfigParams.xpathSeparator + simEndPointElem + JBIOriginalConfigParams.xpathSeparator + simIPElem;

    // create and populate AdminProperties object
    String importexportPath = JBIOriginalConfigParams.xpathSeparator + rootElem + JBIOriginalConfigParams.xpathSeparator + administrationPropElem
        + JBIOriginalConfigParams.xpathSeparator;

    String instrumentationPath = JBIOriginalConfigParams.xpathSeparator + rootElem + JBIOriginalConfigParams.xpathSeparator + administrationPropElem
        + JBIOriginalConfigParams.xpathSeparator;

    String discoPath = JBIOriginalConfigParams.xpathSeparator + rootElem + JBIOriginalConfigParams.xpathSeparator + 
    	ClientProperties.capiEndpoints + xpathSeparator + ClientProperties.capiEndpoint + xpathSeparator 
    	+ ClientProperties.capiDisco+ JBIOriginalConfigParams.xpathSeparator;
   
    
    List nodes;
     
    // discoDomain
    node = doc.selectSingleNode(discoPath + ClientProperties.capiDomain);

    if (node != null)
    {
      String discoDomain = node.getStringValue();
      JBIOriginalConfigParams.client.setDomain(discoDomain);
    }
    
    
    // capiTechnology
    node = doc.selectSingleNode(discoPath + ClientProperties.capiTechnology);

    if (node != null)
    {
      String capiTechnology = node.getStringValue();
      JBIOriginalConfigParams.client.setTechnology(capiTechnology);
    }
    
    
    // discoQueryURI
    node = doc.selectSingleNode(discoPath + ClientProperties.discoInquiryURI);
    
    if (node != null)
    {
      String discoQueryURI = node.getStringValue();
      JBIOriginalConfigParams.client.setInquiryURI(discoQueryURI);
    }
    
    
    // discoPublishURI
    node = doc.selectSingleNode(discoPath + ClientProperties.discoPublishURI);
    
    if (node != null)
    {
      String discoPublishURI = node.getStringValue();
      JBIOriginalConfigParams.client.setPublishURI(discoPublishURI);
    }
    
    
    // discoKeystore
    node = doc.selectSingleNode(discoPath + ClientProperties.discoKeystore);
    
    if (node != null)
    {
      String discoKeystore = node.getStringValue();
      JBIOriginalConfigParams.client.setKeystore(discoKeystore);
    }
    
    
    String securityPath = JBIOriginalConfigParams.xpathSeparator + rootElem + JBIOriginalConfigParams.xpathSeparator + secPropElem + JBIOriginalConfigParams.xpathSeparator;

    // on to the SECURITY PROPERTIES
    // first javax.net.ssl.trustStore
    node = doc.selectSingleNode(securityPath + ClientProperties.trustName);

    if (node != null)
    {
      String certificateFileName = node.getStringValue();
      File certFile = new File(certificateFileName);

      if (certFile.exists() == false)
      {
        throw new PlatformFailureException("Problem finding " + certificateFileName + ". The Certificate file specified: " + certificateFileName
            + " does not exist!");
      }

      JBIOriginalConfigParams.client.setNameTrustStore(node.getStringValue());
    }
    else
      throw (new PlatformFailureException("Problem finding " + securityPath + ClientProperties.trustName + " within client.properties!"));

    // next auth_properties.implementation.class
    node = doc.selectSingleNode(securityPath + ClientProperties.secImplName);
    if (node != null)
      JBIOriginalConfigParams.client.setNameSecImplClass(node.getStringValue());
    else
      throw (new PlatformFailureException("Problem finding " + securityPath + ClientProperties.secImplName + " within client.properties!"));

    // finally capi properties
    // get info on all capi loadable classes

    // first connection oriented
    // connection manager
    node = doc.selectSingleNode(capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiConnectionsTag
        + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiConnectionManagerTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiConnectionManagerImpl(node.getStringValue());
    else
      throw (new PlatformFailureException("Problem finding " + capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator
          + ClientProperties.capiConnectionsTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiConnectionManagerTag
          + " within client.properties!"));

    // connection
    node = doc.selectSingleNode(capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiConnectionsTag
        + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiConnectionTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiConnectionImpl(node.getStringValue());
    else
      throw (new PlatformFailureException("Problem finding " + capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator
          + ClientProperties.capiConnectionsTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiConnectionTag + " within client.properties!"));

    // next sequences
    // subscription
    node = doc.selectSingleNode(capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiSequencesTag
        + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiSubscriptionTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiSubscriberImpl(node.getStringValue());
    else
      throw (new PlatformFailureException("Problem finding " + capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator
          + ClientProperties.capiSequencesTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiSubscriptionTag + " within client.properties!"));

    //publication
    node = doc.selectSingleNode(capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiSequencesTag
        + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiPublicationTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiPublisherImpl(node.getStringValue());
    else
      throw (new PlatformFailureException("Problem finding " + capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator
          + ClientProperties.capiSequencesTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiPublicationTag + " within client.properties!"));

    //query
    node = doc.selectSingleNode(capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiSequencesTag
        + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiQueryTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiQueryImpl(node.getStringValue());
    else
      throw (new PlatformFailureException("Problem finding " + capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator
          + ClientProperties.capiSequencesTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiQueryTag + " within client.properties!"));

    //MDR
    node = doc.selectSingleNode(capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiRepositoriesTag
        + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiMDRTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiMDRImpl(node.getStringValue());
    else
      throw (new PlatformFailureException("Problem finding " + capiPath + ClientProperties.capiImplTag + JBIOriginalConfigParams.xpathSeparator
          + ClientProperties.capiRepositoriesTag + JBIOriginalConfigParams.xpathSeparator + ClientProperties.capiMDRTag + " within client.properties!"));

    // next importexport
    // importexport

    node = doc.selectSingleNode(importexportPath + ClientProperties.adminImplTag + JBIOriginalConfigParams.xpathSeparator
        + ClientProperties.adminImportExportTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameAdminImportExportImpl(node.getStringValue());

    node = doc.selectSingleNode(instrumentationPath + ClientProperties.adminImplTag + JBIOriginalConfigParams.xpathSeparator
        + ClientProperties.adminInstrumentationTag);
    if (node != null)
      JBIOriginalConfigParams.client.setNameAdminInstrumentationImpl(node.getStringValue());

    //--------------------------------------------------------------------------
    //Begin code hurtfully abused by the ANTI-VTC Muhahahahahahahaha
    //--------------------------------------------------------------------------

    //obvious
    nodes = doc.selectNodes(capiPath + ClientProperties.capiReceptors + xpathSeparator + ClientProperties.capiReceptorSeparator);

    Iterator nodeIterator = nodes.iterator();

    // load all of the specified Receptors and store them in Data Access Objects.
    while (nodeIterator.hasNext())
    {
      Node currentNode = (Node) nodeIterator.next();
      ReceptorConfigDAO rcdao = new ReceptorConfigDAO(currentNode.selectSingleNode(ClientProperties.capiReceptorEventType).getStringValue(),
          currentNode.selectSingleNode(ClientProperties.capiReceptorScope).getStringValue(), currentNode.selectSingleNode(
              ClientProperties.capiReceptorImpl).getStringValue(), currentNode.selectSingleNode(ClientProperties.capiReceptorProps).asXML());

      try
      {
        JBIOriginalConfigParams.client.getCapiReceptorManager().registerReceptor(rcdao);
      }
      //We can't allow two receptors at the same type and scope. It's just not proper!
      catch (ReceptorAlreadyRegisteredException e1)
      {
        throw new PlatformFailureException(e1.getMessage(), e1);
      }

    }
    //--------------------------------------------------------------------------
    //End code hurtfully abused by the ANTI-VTC
    //--------------------------------------------------------------------------

    // now all possible proliferator at all levels.
    // start with level connection.
    node = doc.selectSingleNode(capiPath + ClientProperties.capiProlifs + JBIOriginalConfigParams.connectionLevel);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiConnectionProlif(node.getStringValue());

    // next subscriber level.
    node = doc.selectSingleNode(capiPath + ClientProperties.capiProlifs + JBIOriginalConfigParams.subscriberLevel);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiSubscriberProlif(node.getStringValue());

    // now all possible configurators at all levels.
    // start with level connection.
    node = doc.selectSingleNode(capiPath + ClientProperties.capiConfs + JBIOriginalConfigParams.connectionLevel);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiConnectionConf(node.getStringValue());

    // next subscriber level.
    node = doc.selectSingleNode(capiPath + ClientProperties.capiConfs + JBIOriginalConfigParams.subscriberLevel);
    if (node != null)
      JBIOriginalConfigParams.client.setNameCapiSubscriberConf(node.getStringValue());

  }

  public static void main(String[] args)
  {

    JBIOriginalConfigParams jbiConfig = new JBIOriginalConfigParams();

    try
    {
      JBIOriginalConfigParams.constructJBIProperties();
    }
    catch (Exception e)
    {
      System.out.println(e.getMessage());
      e.printStackTrace();
    }

    System.out.println(JBIOriginalConfigParams.client.toString() + "\n\n");
  }
}