package coi.jbi.interceptors;

import org.infospherics.jbi.client.exception.ConnectionException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import mil.af.rl.im.capi.client.impl.pluggable.interfaces.ConnectionManager;

import coi.jbi.ontology.JBIActionConcepts;

public class KAoSConnectionManager extends KAoSInterceptor implements ConnectionManager 
{ 
	private ConnectionManager 	_originalConnectionManager;	
	
	public KAoSConnectionManager() 
    {
        super();
        
        _originalConnectionManager = getOriginalConnectionManager();
	}

    //  Method creating original implementation of the ConnectionManager based on the configuraiton in the original client.properties
    private ConnectionManager getOriginalConnectionManager()
    {
        // Dynamically load and instantiate the pluggable connection class instantiate implementation of capi ConnectionManager class
        Class capiClass = null;

        try
        {
            capiClass = this.getClass().getClassLoader().loadClass(JBIOriginalConfigParams.getClientProperties().getNameCapiConnectionManagerImpl());
        }
        catch (ClassNotFoundException cnfe)
        {
            throw new PlatformFailureException("Specified CAPI implementation class was not found: ", cnfe);
        }

        try
        {
            ConnectionManager impl = (mil.af.rl.im.capi.client.impl.pluggable.interfaces.ConnectionManager)capiClass.newInstance();
            
            return impl;
        }
        catch (Exception e)
        {
            String exceptionMessage = "Problem instantiating CAPI implementation:";

            if (e.getMessage() != null)
                exceptionMessage = exceptionMessage + "\n Details Provided: " + e.getMessage();

          throw new PlatformFailureException(exceptionMessage, e);
        }
    }
    
	/**
	 * The client uses this method to select a platform from the list and supply a PlatformDescriptor as the connectionDescriptor in the createConnection method.
	 * <br><br>
	 * @param constraints This method would take a an XML document that conforms to the following structure:
	 * <PlatformDescriptor>
	 * 	...descriptor attributes
	 * </PlatformDescriptor>
	 * @return The returned String[] would contain all PlatformDescriptors (as XML) that match the supplied criteria. If null is supplied then the method returns all registered platforms.
	 *
	 * @throws PlatformFailureException
	 * @throws PermissionDeniedException
	 */
	public String[] getPlatforms(String constraints) throws PlatformFailureException,  PermissionDeniedException
    {
		return _originalConnectionManager.getPlatforms(constraints);
    }

	/**
	 * This method is used to create a Connection to a particular JBI platform implementation The platform descriptor is meant to be an XML document
	 * that describes the particular platform implementation. In the future, this document will allow clients to potentially connect to a number
	 * possibly disparate platform implementations within the same client. Currently an legal string can be used to describe the particular platform.
	 * 
	 * @param connectionName  - a client name provided for the connection
	 * @param connectionDescriptor - XML document that describes the particular platform implementation.
	 * 
	 * @return <b>Connection </b>- the connection interface created that will be used to interact with the platform
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public org.infospherics.jbi.client.Connection createConnection(String connectionName, String connectionDescriptor) 
		throws 	PlatformFailureException, PermissionDeniedException, ConnectionException
    {
        // ---------- Register in KAoS first --------------
	    registerWithKAoS();
          
        checkPolicyForJBIAction(JBIActionConcepts.Connect);
        
		org.infospherics.jbi.client.Connection connection = _originalConnectionManager.createConnection(connectionName, connectionDescriptor);
        
		return connection;
    }
		  
	/**
	 * This method is used to destroy a connection created for a particular JBI platform implementation. If the connection is active and sequences exist
	 * on the connection the sequences will be destroyed before the client disconnects from the specified connection.
	 * 
	 * @param connection =a previously created connection
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public void destroyConnection(org.infospherics.jbi.client.Connection connection) throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {   
        _originalConnectionManager.destroyConnection(connection);
    }

	/**
	 * This method returns all of the current connections that the client has to all JBI platforms.
	 * 
	 * @return <b>Connection[] </b>- an array of connection interfaces that exist within the client application
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public org.infospherics.jbi.client.Connection[] getConnections( ) throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {	
		return _originalConnectionManager.getConnections();
    }

	/**
	 * This method returns a connection that corresponds to the provided connection name or a null if a connection with that name has not been created.
	 * 
	 * @param connectionName a client provided connection name.
	 * 
	 * @return <b>Connection </b>- the connection that is associated with the specified name
	 * 
	 * @throws PermissionDeniedException
	 * @throws ConnectionException
	 * @throws PlatformFailureException
	 * 
	 * @since CAPI 1.0
	 */
	public org.infospherics.jbi.client.Connection getConnection(String connectionName) 
        throws PlatformFailureException, PermissionDeniedException, ConnectionException
    {
		return _originalConnectionManager.getConnection(connectionName);  
    }

    public void closeAllConnections() 
    {
        _originalConnectionManager.closeAllConnections();
    }

    public void removeConnection(String arg0) 
    {
        _originalConnectionManager.removeConnection(arg0);
    }
}
