package coi.jbi.interceptors;

import java.net.URISyntaxException;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.agent.service.ServiceFailure;
import javax.agent.service.directory.AlreadyRegisteredException;
import javax.agent.service.directory.DirectoryFailure;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.exception.InvalidMetadataException;
import org.infospherics.jbi.client.exception.InvalidPayloadException;
import org.infospherics.jbi.client.exception.ObjectSizeException;
import org.infospherics.jbi.client.exception.ObjectUnavailableException;
import org.infospherics.jbi.client.exception.PausedSequenceException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.SequenceStateException;
import org.infospherics.jbi.client.exception.TimeoutException;
import org.infospherics.jbi.client.exception.UnimplementedException;
import org.infospherics.jbi.client.exception.UnsupportedVersionException;
import org.infospherics.jbi.client.exception.VersionNumberException;

import kaos.core.service.directory.KAoSAgentDescription;
import kaos.core.csi.ActionInstanceDescription;
import kaos.core.csi.ActionInstanceDescriptionImpl;
import kaos.core.csi.CSIFactory;
import kaos.core.csi.OntInstanceDescription;
import kaos.core.csi.OntInstanceDescriptionImpl;
import kaos.core.csi.OntPropertyDescription;
import kaos.core.csi.OntPropertyDescriptionImpl;
import kaos.core.csi.policy.KAoSSecurityException;
import kaos.core.csi.policy.PolicyChecking;
import kaos.core.csi.registration.AgentConfiguration;
import kaos.core.csi.registration.Registration;
import kaos.ontology.vocabulary.ActionConcepts;
import kaos.policy.guard.GuardInstantiationException;
import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.KAoSGuard;

import kaos.core.csi.extension.obligation.ObligationMonitor;
import kaos.core.csi.extension.obligation.ObligationMonitorManager;

import coi.semanticTranslator.SemanticTranslationService;
import coi.semanticTranslator.SemanticTranslatorManager;

public class KAoSInterceptor 
{	
    private static String _clientName ="";
    private static AgentConfiguration myAgentConfig = null;
    private static KAoSAgentDescription   myAgentDesc = null;
    private static SemanticTranslationService mySemanticTranslationService = null;
    private static ObligationMonitor myObligationMonitor = null;
    
    private static int processedAIDnum = 0; // needed to build unique id for each action
    
	public KAoSInterceptor()
    {   
        // Obtain the name of this client from the KAoS configuration file
        Registration registration = null;
        CSIFactory.setCurrentTransport("tcp");

        try 
        {
            registration = CSIFactory.getRegistration();
        } 
        catch (Exception e1) 
        {
            e1.printStackTrace();
        }
        
        myAgentConfig = registration.getAgentConfiguration(); // Obtain client bootstrap information e.g. _clientName
        _clientName = myAgentConfig.getAgentId();
	}
 
    public void checkPolicyForJBIAction(String jbiActionName) throws KAoSSecurityException
    {
        // -------------- Obligation Monitor Notification and Authorization Policy Checking --------------
    
        // Build Action Instance Description for the JBI action
        ActionInstanceDescription publishAID = null;

        try 
        {
            publishAID = new ActionInstanceDescriptionImpl(jbiActionName + "AID" + processedAIDnum,  jbiActionName, _clientName);
   
            processedAIDnum++; // increase the counter of constructed AIDs

        } 
        catch (URISyntaxException e1) 
        {
            e1.printStackTrace();
        }
        catch (UnimplementedException e) 
        {
            e.printStackTrace();
        } 
        
        // Update Obligation Monitor 
        if (myObligationMonitor != null)
            myObligationMonitor.obligationFulfilled(null, publishAID);
    
        // Check authorization
        PolicyChecking policyChecking = null;
        try 
        {
            policyChecking = CSIFactory.getPolicyChecking();
        } 
        catch (Exception e) 
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        if (policyChecking != null)
            try 
            {
                policyChecking.checkPermission(publishAID, null);
            } 
            catch (NullPointerException e) 
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (ServiceFailure e) 
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
    }
    
	public void checkPolicyForInfoObject(InfoObject IO, String jbiActionName) throws KAoSSecurityException
    {
        String originalMetainformation = null; 
        try 
        {
            if (IO != null)
            {
                // Add information about the information object beign processed - current assumptionis that the metadata is in OWL
                originalMetainformation = IO.getMetadata();
            }
        } 
        catch (UnimplementedException e) 
        {
            e.printStackTrace();
        } 
        catch (ObjectUnavailableException e) 
        {
            e.printStackTrace();
        } 
        catch (TimeoutException e) 
        {
            e.printStackTrace();
        }
        
        checkPolicyForInfoObject(originalMetainformation, jbiActionName);
    }
    
    public void checkPolicyForInfoObject(String IOmetainformation, String jbiActionName) throws KAoSSecurityException
    {
        // -------------- Obligation Monitor Notification and Authorization Policy Checking --------------
        
        // Build Action Instance Description for the JBI action
        ActionInstanceDescription publishAID = null;
        OntInstanceDescription currentInfoObject = null;
        try 
        {
            publishAID = new ActionInstanceDescriptionImpl(jbiActionName + "AID" + processedAIDnum,  jbiActionName, _clientName);
       
            processedAIDnum++; // increase the counter of constructed AIDs

            if (IOmetainformation != null)
            {
                // Add information about the information object beign processed - current assumptionis that the metadata is in OWL            
                currentInfoObject = new OntInstanceDescriptionImpl(IOmetainformation);
            
                OntPropertyDescription accessedEntity = new OntPropertyDescriptionImpl(ActionConcepts.accessedEntity());
                accessedEntity.addValue(currentInfoObject.getName());
                accessedEntity.setValueDescForValue(currentInfoObject.getName(), currentInfoObject);
           
                // -- Finishing the AID
                publishAID.addProperty(accessedEntity);
            }
        } 
        catch (URISyntaxException e1) 
        {
            e1.printStackTrace();
        }
        catch (UnimplementedException e) 
        {
            e.printStackTrace();
        } 
        
        // Update Obligation Monitor 
        if (getObligationMonitor() != null)
            getObligationMonitor().obligationFulfilled(null, publishAID);

        // Check authorization
        PolicyChecking policyChecking = null;
        try 
        {
            policyChecking = CSIFactory.getPolicyChecking();
        } 
        catch (Exception e) 
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        
        if (policyChecking != null)
            try 
            {
                policyChecking.checkPermission(publishAID, null);
            } 
            catch (NullPointerException e) 
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (ServiceFailure e) 
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
    }
    
    public InfoObject translateInfoObject(InfoObject IO)  
        throws PausedSequenceException, PlatformFailureException, PermissionDeniedException, SequenceStateException
    {
        // Retrive the metainformaiton from information object
        OntInstanceDescription currentInfoObject = null;
        String originalMetainformation = null; 
        try 
        {
            // Get information about the information object beign translated - current assumptionis that the metadata is in OWL
            originalMetainformation = IO.getMetadata();
            
            currentInfoObject = new OntInstanceDescriptionImpl(originalMetainformation);
        } 
        catch (URISyntaxException e1) 
        {
            e1.printStackTrace();
        }
        catch (UnimplementedException e) 
        {
            e.printStackTrace();
        } 
        catch (ObjectUnavailableException e) 
        {
            e.printStackTrace();
        } 
        catch (TimeoutException e) 
        {
            e.printStackTrace();
        }
        
        // Check if semantic translation to COI version of product is needed before publishing
        String _currentInfoObjectTranslate = null;
        String sourceTypeURL = (String)currentInfoObject.getTypes().toArray()[0];
        
        if (getSemanticTranslationService() == null)
            return IO;
        
        // TODO: currently simplyfy; assuems that metainfnormation has a single type and only single translation can exist
        String destinationTypeURL = getSemanticTranslationService().checkIfTranslationNecessary(sourceTypeURL);
        if (destinationTypeURL != null)
            _currentInfoObjectTranslate = getSemanticTranslationService().translateMetaInformation(originalMetainformation, 
                sourceTypeURL, destinationTypeURL, currentInfoObject.getName(), currentInfoObject.getName() + "Translated");

        InfoObject transaltedIO = null;
        if ( _currentInfoObjectTranslate != null)
        {
            try 
            {
                // Build new Information Object
                transaltedIO = JBIBaseClient.getConnection().createInfoObject(IO.getTypeName(), IO.getVersion(), IO.getPayload(), _currentInfoObjectTranslate);
            } 
            catch (InvalidPayloadException e) 
            {
                e.printStackTrace();
            } catch (InvalidMetadataException e) 
            {
                e.printStackTrace();
            } 
            catch (UnsupportedVersionException e) 
            {
                e.printStackTrace();
            } 
            catch (ObjectSizeException e) 
            {
                e.printStackTrace();
            } catch (VersionNumberException e) 
            {
                e.printStackTrace();
            } 
            catch (ObjectUnavailableException e) 
            {
                e.printStackTrace();
            } catch (TimeoutException e) 
            {
                e.printStackTrace();
            }
        }
        
        if (transaltedIO != null)
            IO = transaltedIO;
        
		return IO;
	}
    
    public String getUserId(String clientRep)
    {
        int indx = clientRep.indexOf("<username>");

        if (indx > 0)
            clientRep = clientRep.substring(indx + 10);

        indx = clientRep.indexOf("</username>");

        String id = clientRep.substring(0, indx);
        System.out.println("id: " + id);

        return id;
    }

    public void registerWithKAoS()
    {
        Registration registration = null;

        CSIFactory.setCurrentTransport("tcp");

        try
        {
            registration = CSIFactory.getRegistration();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            System.out.println("\n\n\t*** UNABLE TO CONNECT TO KAoS!  Will continue without KAoS services!");

            return;
        }

        // Try to register domains defined in agentConfig.cfg file.
        List domainNames = myAgentConfig.getDomains();

        for (Iterator domainsIter = domainNames.iterator(); domainsIter.hasNext(); )
        {
            String dName  = (String) domainsIter.next();
            System.out.println("\n Will register domain: " + dName + "\n");

            try
            {
                registration.registerDomain(dName);
            }
            catch (DirectoryFailure de)
            {
                System.out.println("\n\n\t*** Due to Directory Failure unable to register domain: " + dName + "\n\n\t*** Will continue without KAoS services!");
                de.printStackTrace();

                return;
            }
            catch(AlreadyRegisteredException ae)
            {
                System.out.println("\n\t*** The following domain has already been registered: " + dName);
            }
        }

        myAgentDesc = registration.createKAoSAgentDescription(myAgentConfig.getAgentId(), myAgentConfig.getAgentId(), (Vector)domainNames, "tcp", null);

        for (Iterator agentTypeIT = myAgentConfig.getOntologicalTypes().iterator(); agentTypeIT.hasNext(); )
            myAgentDesc.setEntityOntologicalType((String)agentTypeIT.next());

        // System.out.println("\n\tKAoSWrappedConnection.registerWithKAoS: will register in KAoS using KAoSAgentDescription:\n " + myAgentDesc);

        try
        {
            registration.registerEntity(myAgentDesc, true, false);    // guarded and replace registration
        }
        catch(DirectoryFailure df)
        {
            df.printStackTrace();
            System.out.println("\n\n\tDue to Directory Failure unable to register client: " + myAgentConfig.getAgentId() + "\n\n\t*** Will continue without KAoS services!");

        }
        catch(AlreadyRegisteredException are)
        {
            are.printStackTrace();
            System.out.println("\n\n\tClient: " + myAgentConfig.getAgentId() + " has been already registered with KAoS!");

        }
        catch(GuardInstantiationException gie)
        {
            gie.printStackTrace();
            System.out.println("\n\n\tDue to Guard instantiation failure unable to register client: " + myAgentConfig.getAgentId() + "\n\n\t*** Will continue without KAoS services!");
        }
    }
    
    public void deregisterFromKAoS()
    {
        try
        {
            Registration registration = CSIFactory.getRegistration();
            
            if (myAgentDesc != null)
                registration.deregisterEntity(myAgentDesc);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            System.out.println("\n\n\t*** UNABLE TO CONNECT TO KAoS!  Will continue without KAoS services!");

            return;
        }
    }
    
    public String getClientId()
    {
        return _clientName;
    }
    
    protected SemanticTranslationService getSemanticTranslationService()
    {
        if (mySemanticTranslationService == null)
        {
            //  Initilize Semantic Translator
            KAoSGuard guard = (KAoSGuard)GuardRetriever.getGuard();
            
            if (guard == null)
                return null;
            
            mySemanticTranslationService = (SemanticTranslationService)guard.getKAoSExtensionComponents(SemanticTranslatorManager.SEMANTIC_TRANSLATOR_MANAGER_TYPE, 
                    SemanticTranslationService.SEMANTIC_TRANSLATION_SERVICE_URL);        
        }
        
        return mySemanticTranslationService;
    }
    
    protected ObligationMonitor getObligationMonitor()
    {
        if (myObligationMonitor == null)
        {
            //  Initilize Obligation Monitor
            KAoSGuard guard = (KAoSGuard)GuardRetriever.getGuard();
            
            if (guard == null)
                return null;
            
            
            // Initilize ObligationMonitor
            myObligationMonitor = (ObligationMonitor)guard.getKAoSExtensionComponents(ObligationMonitorManager.OBLIGATION_MONITOR_MANAGER_TYPE, 
                    ObligationMonitor.OBLIGATION_MONITOR_URL);      
        }
        
        return myObligationMonitor;
    }
}
