package coi.jbi.interceptors;

import java.net.URISyntaxException;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.SubscriberSequence;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.UnimplementedException;

import coi.relationMonitor.RelationMonitorService;

import coi.jbi.client.InfoObjectHelper;
import coi.jbi.ontology.JBIActionConcepts;

import kaos.core.csi.OntInstanceDescription;
import kaos.core.csi.OntInstanceDescriptionImpl;
import kaos.policy.guard.GuardRetriever;
import kaos.policy.guard.KAoSGuard;

import org.infospherics.jbi.client.ObjectAvailableCallback;

/**
 * This class is used for subscription to a JBI service. It receives a callback from JBI each time
 * new data is available to a subscribed client. The base class reports each JBI callback to
 * the KAoS RelationMonitorService and then forwards ths callback to the class originally registered 
 * by the client for this subsription chanel
 */
public class KAoSObjectAvailableCallback extends KAoSInterceptor implements ObjectAvailableCallback
{
    protected RelationMonitorService _relationMonitorService = null;
    protected SubscriberSequence _subscriberSeq = null;
    protected String channelId = null;
    protected ObjectAvailableCallback _originalClientObjectAvailableCallback = null;

    public KAoSObjectAvailableCallback(ObjectAvailableCallback originalClientObjectAvailableCallback, SubscriberSequence mySubscriberSequence)
    {
        super();
        
        // Record the original client callback class
        _originalClientObjectAvailableCallback = originalClientObjectAvailableCallback;
        
        // Initilize the KAoS Relation Monitor Service
        KAoSGuard guard = (KAoSGuard)GuardRetriever.getGuard();
        _subscriberSeq = mySubscriberSequence;
        _relationMonitorService = (RelationMonitorService)guard.
        getKAoSExtensionComponents("RelationMonitor", RelationMonitorService.RELATION_MONITOR_SERVICE_URL);

        try
        {
            channelId = ((coi.jbi.interceptors.KAoSSubscriberSequence)mySubscriberSequence).getSequenceUID();
        }
        catch (UnimplementedException e)
        {
            e.printStackTrace();
        }
        catch (PlatformFailureException e)
        {
            e.printStackTrace();
        }
        catch (PermissionDeniedException e)
        {
            e.printStackTrace();
        }
    }

    // This method is called when the object is avialble on the subsription channel
    public void objectsAvailable(InfoObject[] objects)
    {
        // Parse the information object and rettrive intomtion form it needed for the report to the Realtion Monitor
        if (objects != null && objects.length != 0)
        {
            for (int i = 0; i < objects.length; i++)
            {
                InfoObject IO = objects[i];
 
                try
                {
                    InfoObjectHelper infoObjHelper = new InfoObjectHelper(IO);
                    String infoObjType = infoObjHelper.getTypeName();
                    
                    if (infoObjType.equals(_subscriberSeq.getInfoObjectTypeName()))
                    {
                        String pubId = infoObjHelper.getPublisherID();

                        // Report the relation
                        if ((_relationMonitorService != null) && (channelId != null) && !channelId.equals(pubId))
                        {
                            // Obtain ontologogy type from metainformation
                            OntInstanceDescription currentInfoObject = null;
                            String originalMetainformation = null; 
                            
                            try 
                            {
                                originalMetainformation = infoObjHelper.getMetadata();
                                currentInfoObject = new OntInstanceDescriptionImpl(originalMetainformation);
                                
                                // Check if the given object can be delivered
                                checkPolicyForInfoObject(originalMetainformation, JBIActionConcepts.DeliverInfoObject());
                            } 
                            catch (URISyntaxException e1) 
                            {
                                e1.printStackTrace();
                            }
                            catch (UnimplementedException e) 
                            {
                                e.printStackTrace();
                            } 
                            
                            String ontTypeOfMetainformation = "Unknown";
                            
                            if (currentInfoObject != null)
                                ontTypeOfMetainformation = (String)currentInfoObject.getTypes().toArray()[0];
                            
                            _relationMonitorService.reportRelation(pubId, channelId, ontTypeOfMetainformation, true);
                            System.out.println("Reported Relation " + pubId + "   " + channelId + "   " + ontTypeOfMetainformation);
                        }
                        else
                           System.out.println("\n\n\tNULL RelationMonitor Service");
                    }
                    else
                        // Received data I wasn't subscribed to - need xpath filter
                        System.out.println("received data I wasn't subscribed to - need xpath filter to exclude " + infoObjType);
                }
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
            
            // ------------ Forward object(s) to the original client callback -----------
            
            if (_originalClientObjectAvailableCallback != null)
                _originalClientObjectAvailableCallback.objectsAvailable(objects);
        }
    }
}