package coi.jbi.interceptors;

import java.lang.reflect.Constructor;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.PayloadCallback;
import org.infospherics.jbi.client.exception.InvalidAttributeXMLException;
import org.infospherics.jbi.client.exception.PausedSequenceException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.SequenceStateException;
import org.infospherics.jbi.client.exception.StaleSequenceUIDException;
import org.infospherics.jbi.client.exception.UnimplementedException;
import org.infospherics.jbi.client.exception.UnknownAttributeException;

import mil.af.rl.im.capi.client.core.interfaces.PublisherSequence;

import coi.jbi.ontology.JBIActionConcepts;

public class KAoSPublisherSequence extends KAoSInterceptor implements PublisherSequence 
{
	private PublisherSequence 	_publisherSequence;
    
	public KAoSPublisherSequence(String connUID, String seqUID, String informationObjectType, String version)
    {
        super();
        
		_publisherSequence = getOriginalPublisherSequence(connUID, seqUID, informationObjectType, version);
	}

    // Method creating original implementaion of the PublisherSequence based on the configuraiton in the original client.properties
    private PublisherSequence getOriginalPublisherSequence(String connUID, String seqUID, String informationObjectType, String version)
    {
        Class[] argTypes = {connUID.getClass(), seqUID.getClass(), informationObjectType.getClass(), version.getClass()};
        Object[] args = {connUID, seqUID, informationObjectType, version};
        
        // Instantiate implementation of capi Publish class
        Class capiClass = null;
        try
        {
            capiClass = this.getClass().getClassLoader().loadClass(JBIOriginalConfigParams.getClientProperties().getNameCapiPublisherImpl());
        }
        catch (ClassNotFoundException cnfe)
        {
            throw new PlatformFailureException("Specified CAPI implementation class was not found: ", cnfe);
        }

        try
        {
            Constructor connectionConstruct = capiClass.getConstructor(argTypes);
            PublisherSequence myPublisherSequence = (mil.af.rl.im.capi.client.core.interfaces.PublisherSequence)connectionConstruct.newInstance(args);
            
            return myPublisherSequence;
        }
        catch (Exception e)
        {
            String exceptionMessage = "Problem instantiating CAPI implementation:";

            if (e.getMessage() != null)
                exceptionMessage = exceptionMessage + "\n Details Provided: " + e.getMessage();

            throw new PlatformFailureException(exceptionMessage, e);
        }
    }
    
	public String publishInfoObject(InfoObject IO) throws PausedSequenceException, PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        // -------------- Obligation Monitor Notification and Authorization Policy Checking --------------
        checkPolicyForInfoObject(IO, JBIActionConcepts.Publish());

        InfoObject translatedIO = translateInfoObject(IO);
         
		return _publisherSequence.publishInfoObject(translatedIO);
	}

	public void setPayloadCallback(PayloadCallback callback) throws PlatformFailureException, UnimplementedException, SequenceStateException 
	{
		_publisherSequence.setPayloadCallback(callback);
	}

	public int setSequenceAttribute(String attributeTypeName, String version, String attributeXML) 
		throws PermissionDeniedException, InvalidAttributeXMLException, UnknownAttributeException, PlatformFailureException, SequenceStateException 
    {
		return _publisherSequence.setSequenceAttribute(attributeTypeName, version, attributeXML); 
	}

	public void removeSequenceAttribute(String attributeTypeName, String version)
	     throws PermissionDeniedException, UnknownAttributeException, PlatformFailureException, StaleSequenceUIDException, SequenceStateException 
    {		
		_publisherSequence.removeSequenceAttribute(attributeTypeName,version);
	}

	public String getSequenceAttribute(String attributeTypeName, String version)
		throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
		return _publisherSequence.getSequenceAttribute(attributeTypeName, version);
	}

	public void activateSequence() throws PlatformFailureException, PermissionDeniedException, SequenceStateException
	{
		_publisherSequence.activateSequence();
	}

	public void pauseSequence() throws PlatformFailureException, PermissionDeniedException 
    {
		_publisherSequence.pauseSequence();
	}

	public int getSequenceState() throws PlatformFailureException, PermissionDeniedException 
	{
		return _publisherSequence.getSequenceState();
	}
	
	public void resumeSequence() throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
	{
		_publisherSequence.resumeSequence();
	}

	public String getInfoObjectTypeName() 
	{
		return _publisherSequence.getInfoObjectTypeName();
	}

	public String getInfoObjectVersion()
	{
		return _publisherSequence.getInfoObjectVersion();
	}

    public void destroySequence() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        _publisherSequence.destroySequence();
    }

    public String getSequenceUID() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        return _publisherSequence.getSequenceUID();
    }
}
