package coi.jbi.interceptors;

import java.lang.reflect.Constructor;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.exception.InvalidAttributeXMLException;
import org.infospherics.jbi.client.exception.PausedSequenceException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.SequenceStateException;
import org.infospherics.jbi.client.exception.StaleSequenceUIDException;
import org.infospherics.jbi.client.exception.UnimplementedException;
import org.infospherics.jbi.client.exception.UnknownAttributeException;

import coi.jbi.ontology.JBIActionConcepts;

import mil.af.rl.im.capi.client.core.interfaces.QuerySequence;

public class KAoSQuerySequence extends KAoSInterceptor implements QuerySequence 
{
	private QuerySequence 	_querySequence;
	
	public KAoSQuerySequence(String connUID, String seqUID, String informationObjectType, String version)
    {
        super();
        
        _querySequence = getOriginalQuerySequence(connUID, seqUID, informationObjectType, version);
	}

     // Method creating original implementaion of the QuerySequence based on the configuraiton in the original client.properties
    private QuerySequence getOriginalQuerySequence(String connUID, String seqUID, String informationObjectType, String version)
    {
        Class[] argTypes = {connUID.getClass(), seqUID.getClass(), informationObjectType.getClass(), version.getClass()};
        Object[] args = {connUID, seqUID, informationObjectType, version};

        // Instantiate implementation of capi Connection class
        Class capiClass = null;
        try
        {
            capiClass = this.getClass().getClassLoader().loadClass(JBIOriginalConfigParams.getClientProperties().getNameCapiQueryImpl());
        }
        catch (ClassNotFoundException cnfe)
        {
            throw new PlatformFailureException("Specified CAPI implementation class was not found: ", cnfe);
        }

        try
        {
            Constructor connectionConstruct = capiClass.getConstructor(argTypes);

            QuerySequence impl = (mil.af.rl.im.capi.client.core.interfaces.QuerySequence)connectionConstruct.newInstance(args);
          
            return impl;
        }
        catch (Exception e)
        {
            String exceptionMessage = "Problem instantiating CAPI implementation:";

            if (e.getMessage() != null)
                exceptionMessage = exceptionMessage + "\n Details Provided: " + e.getMessage();

            throw new PlatformFailureException(exceptionMessage, e);
        }
    }
    
    public void destroySequence() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        _querySequence.destroySequence();
    }

    public String getSequenceUID() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        return _querySequence.getSequenceUID();
    }

    public int executeQuery(String predicate, int maxResultSetSize) throws PlatformFailureException, StaleSequenceUIDException, SequenceStateException 
    {
        // -------------- Obligation Monitor Notification and Authorization Policy Checking --------------
        checkPolicyForInfoObject((String)null, JBIActionConcepts.Query());
         
        return _querySequence.executeQuery(predicate, maxResultSetSize);
    }

    public InfoObject[] getNextResultSet() throws PlatformFailureException, StaleSequenceUIDException, SequenceStateException 
    {
        return _querySequence.getNextResultSet();
    }

    public void activateSequence() throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        _querySequence.activateSequence();
    }

    public String getInfoObjectTypeName() 
    {
        return _querySequence.getInfoObjectTypeName();
    }

    public String getInfoObjectVersion() 
    {
        return _querySequence.getInfoObjectVersion();
    }

    public String getSequenceAttribute(String attributeTypeName, String version) throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        return _querySequence.getSequenceAttribute(attributeTypeName, version);
    }

    public int getSequenceState() throws PlatformFailureException, PermissionDeniedException 
    {
        return _querySequence.getSequenceState();
    }

    public void pauseSequence() throws PlatformFailureException, PermissionDeniedException 
    {
        _querySequence.pauseSequence();
    }

    public void removeSequenceAttribute(String attributeTypeName, String version) 
        throws PermissionDeniedException, UnknownAttributeException, PlatformFailureException, StaleSequenceUIDException, SequenceStateException 
    {
        _querySequence.removeSequenceAttribute(attributeTypeName, version);
    }

    public void resumeSequence() throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        _querySequence.resumeSequence();
    }

    public int setSequenceAttribute(String attributeTypeName, String version, String attributeXML)
        throws PermissionDeniedException, InvalidAttributeXMLException, UnknownAttributeException, PlatformFailureException, SequenceStateException 
    {
        return _querySequence.setSequenceAttribute(attributeTypeName, version, attributeXML);
    }
}
