package coi.jbi.interceptors;

import java.lang.reflect.Constructor;

import mil.af.rl.im.capi.client.core.interfaces.Configurator;
import mil.af.rl.jbi.platform.shared.capi.core.plugin.interfaces.Receptor;

import org.infospherics.jbi.client.InfoObject;
import org.infospherics.jbi.client.ObjectAvailableCallback;
import org.infospherics.jbi.client.exception.InvalidAttributeXMLException;
import org.infospherics.jbi.client.exception.InvalidPredicateException;
import org.infospherics.jbi.client.exception.ObjectUnavailableException;
import org.infospherics.jbi.client.exception.PermissionDeniedException;
import org.infospherics.jbi.client.exception.PlatformFailureException;
import org.infospherics.jbi.client.exception.PredicateLanguageException;
import org.infospherics.jbi.client.exception.SequenceStateException;
import org.infospherics.jbi.client.exception.StaleSequenceUIDException;
import org.infospherics.jbi.client.exception.UnimplementedException;
import org.infospherics.jbi.client.exception.UnknownAttributeException;
import org.infospherics.jbi.client.exception.UnrecognizedObjectTypeException;

import mil.af.rl.im.capi.client.core.interfaces.SubscriberSequence;

public class KAoSSubscriberSequence extends KAoSInterceptor implements SubscriberSequence
{
    SubscriberSequence _subscriberSequence;
    
    public KAoSSubscriberSequence(String connUID, String seqUID, String informationObjectType, String version)
    {
        super();
        
        _subscriberSequence = getOriginalSubscriberSequence(connUID, seqUID, informationObjectType, version);
    }
    
    // Method creating original implementaion of the SubscriberSequence based on the configuraiton in the original client.properties
    private SubscriberSequence getOriginalSubscriberSequence(String connUID, String seqUID, String informationObjectType, String version)
    {
        // Load and instantiate the pluggable SubscriberSequence
        Class[] argTypes = {connUID.getClass(), seqUID.getClass(), informationObjectType.getClass(), version.getClass()};
        Object[] args = {connUID, seqUID, informationObjectType, version};

        Class capiClass = null;

        try
        {
            capiClass = this.getClass().getClassLoader().loadClass(JBIOriginalConfigParams.getClientProperties().getNameCapiSubscriberImpl());
        }
        catch (ClassNotFoundException cnfe)
        {
            throw new PlatformFailureException("Specified CAPI implementation class was not found: ", cnfe);
        }

        try
        {
            Constructor connectionConstruct = capiClass.getConstructor(argTypes);

            SubscriberSequence impl = (mil.af.rl.im.capi.client.core.interfaces.SubscriberSequence) connectionConstruct.newInstance(args);
            
            return impl;
        }
        catch (Exception e)
        {
            String exceptionMessage = "Problem instantiating CAPI implementation:";

            if (e.getMessage() != null)
                exceptionMessage = exceptionMessage + "\n Details Provided: " + e.getMessage();

            e.printStackTrace();

            throw new PlatformFailureException(exceptionMessage, e);
        }
    }
    
    public void destroySequence() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        _subscriberSequence.destroySequence();
    }

    public Configurator getConfigurator() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        return _subscriberSequence.getConfigurator();
    }

    public ObjectAvailableCallback getSequenceCallback() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        return _subscriberSequence.getSequenceCallback();
    }

    public String getSequenceUID() throws PlatformFailureException, PermissionDeniedException, UnimplementedException 
    {
        return _subscriberSequence.getSequenceUID();
    }

    public void setConfigurator(Configurator conf) 
    {
        _subscriberSequence.setConfigurator(conf);
    }

    public void setProliferatorType(String prolif) 
    {
        _subscriberSequence.setProliferatorType(prolif);
    }

    public void setReceptor(Receptor receptor)
    {
        _subscriberSequence.setReceptor(receptor);
    }

    public InfoObject[] getInfoObject() throws ObjectUnavailableException, PermissionDeniedException, PlatformFailureException, SequenceStateException 
    {
        return _subscriberSequence.getInfoObject();
    }

    public void setSequenceCallback(ObjectAvailableCallback callback) 
        throws UnrecognizedObjectTypeException, PermissionDeniedException, PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        // Create KAoS implemention of the ObjectAvailableCallback, initilized it with the original calback and set the KAoS callback in JBI
        ObjectAvailableCallback kaosObjectAvailableCallback = new KAoSObjectAvailableCallback(callback, this);
        
        _subscriberSequence.setSequenceCallback(kaosObjectAvailableCallback);
    }

    public void setSequencePredicate(String subscriptionPredicate) 
        throws InvalidPredicateException, PredicateLanguageException, PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        _subscriberSequence.setSequencePredicate(subscriptionPredicate);
    }

    public void activateSequence() throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        _subscriberSequence.activateSequence();
    }

    public String getInfoObjectTypeName() 
    {
        return _subscriberSequence.getInfoObjectTypeName();
    }

    public String getInfoObjectVersion() 
    {
        return _subscriberSequence.getInfoObjectVersion();
    }

    public String getSequenceAttribute(String attributeTypeName, String version) throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        return _subscriberSequence.getSequenceAttribute(attributeTypeName, version);
    }

    public int getSequenceState() throws PlatformFailureException, PermissionDeniedException 
    {
        return _subscriberSequence.getSequenceState();
    }

    public void pauseSequence() throws PlatformFailureException, PermissionDeniedException 
    {
        _subscriberSequence.pauseSequence();
    }

    public void removeSequenceAttribute(String attributeTypeName, String version) 
        throws PermissionDeniedException, UnknownAttributeException, PlatformFailureException, StaleSequenceUIDException, SequenceStateException 
    {
        _subscriberSequence.removeSequenceAttribute(attributeTypeName, version);
    }

    public void resumeSequence() throws PlatformFailureException, PermissionDeniedException, SequenceStateException 
    {
        _subscriberSequence.resumeSequence();
    }

    public int setSequenceAttribute(String attributeTypeName, String version, String attributeXML) 
        throws PermissionDeniedException, InvalidAttributeXMLException, UnknownAttributeException, PlatformFailureException, SequenceStateException 
    {
        return _subscriberSequence.setSequenceAttribute(attributeTypeName, version, attributeXML);
    }
};