/**
 * Last Modified by: $Author: jlott $
 * On: $Date: 2006/10/06 20:22:01 $
 */
 
package coi.ontologyAnnotation;

import java.util.List;

/**
 * @author rjeffers
 *
 */
public interface OntologyAnnotationService {
	
	/** 
     * Get a List of Strings containing the urls of owl files describing the actions this service supports.
	 *
	 * @return       List of urls pointing to OWL files. 
	 */	
	public List getActions();
	
	/** 
     * Get a List of Strings containing the urls of owl files describing the actions a service
     * with the given id.
     * 
     * @param serviceID   The id of the service, whose actions are being requested.
	 *
	 * @return       List of urls pointing to OWL files. 
	 */	
	public List getActions(String serviceID);
	
	/** 
     * Get a List of Strings containing the urls of owl files describing the types 
     * the specified action this service supports.
     * 
     * @param actionURL   The url of the action, whose types are being requested.
	 *
	 * @return       List of urls pointing to OWL files describing types of the given action.
	 *               of this service. 
	 */	
	public List getInfoObjectTypesForAction(String actionURL);
	
	/** 
     * Get a List of Strings containing the urls of owl files describing the types 
     * the specified action the given service supports. 
     * 
     * @param actionURL   The url of the action, whose types are being requested.
     * 
     * @param serviceID   The id of the service, whose action types are being requested.
	 *
	 * @return       List of urls pointing to OWL files describing types of the given action
	 * 				 of the given service. 
	 */	
	public List getInfoObjectTypesForAction(String actionURL, String serviceID);
	

}
