package coi.relationMonitor;

import kaos.core.csi.extension.KAoSExtensionComponentImpl;
import kaos.core.csi.observe.KAoSObservable;
import kaos.core.csi.observe.KAoSObservableImpl;
import kaos.core.csi.observe.KAoSObserver;
import kaos.core.csi.transport.TransportSupport;
import kaos.core.csi.transport.TransportSupportImpl;
import kaos.core.util.UniqueIdentifier;

import javax.agent.service.transport.MessageTransportService;
import javax.agent.service.transport.TransportFailure;
import java.util.Date;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

public class RelationMonitorServiceImpl  extends KAoSExtensionComponentImpl implements RelationMonitorService, KAoSObservable 
{
    private KAoSObservableImpl _observableImpl;

    private Hashtable _relationTable = new Hashtable(); // maps fromId to a hashtable containig toId(s) associated with RelationInfo
    
	public RelationMonitorServiceImpl() 
    {
		super();
        
        TransportSupport ts = TransportSupportImpl.getInstance();
        MessageTransportService mts = ts.getServiceRoot((String) ts.getSupportedTransportsNames().get(0)).getPreferredMessageTransportService();
        
        try 
        {
            _observableImpl = new KAoSObservableImpl(mts, mts.newLocalLocator(), "RelationMonitorService_" + UniqueIdentifier.GenerateUID());
        } 
        catch (TransportFailure ex) 
        {
            ex.printStackTrace();
        }
	}

	public RelationMonitorServiceImpl(String arg0, Vector arg1) 
    {
		super(arg0, arg1);
	}

	public RelationMonitorServiceImpl(String arg0, String arg1) 
    {
		super(arg0, arg1);
	}

    public void reportRelation (String fromId, String toId, String infoObjectType, boolean isProducer)
    {
        Hashtable fromIdRelations = (Hashtable)_relationTable.get(fromId);
        
        if (fromIdRelations == null) 
            fromIdRelations = new Hashtable();

        if ( ! fromIdRelations.containsKey(toId))
        {
            RelationInfo info = new RelationInfo(infoObjectType, new Date(), isProducer);
            fromIdRelations.put(toId, info);
        }
        else
        {
            RelationInfo existing = (RelationInfo)fromIdRelations.get(toId);
            if (existing != null) 
            {
                existing.updateRelationInfo(infoObjectType, new Date());
                fromIdRelations.put(toId, existing);
            }
        }
        
        _relationTable.put(fromId, fromIdRelations);
        
        // Now add info about the opposite direction
        Hashtable toIDRelations = (Hashtable)_relationTable.get(toId);
        if (toIDRelations == null) 
            toIDRelations = new Hashtable();
        
        Hashtable notifyInfo = null;
        
        if ( ! toIDRelations.containsKey(fromId))
        {    
            RelationInfo info = new RelationInfo(infoObjectType, new Date(), !isProducer);
            toIDRelations.put(fromId, info);
        
            notifyInfo = new Hashtable();
            notifyInfo.put(fromId, toId);
        }
        else
        {
            RelationInfo existing = (RelationInfo)toIDRelations.get(fromId);
            if (existing != null) 
            {
                existing.updateRelationInfo(infoObjectType, new Date());
                toIDRelations.put(fromId, existing);
            }
        }
        
        _relationTable.put(toId, toIDRelations);
        
        if (notifyInfo != null)
            notify(NEW_RELATION_UPDATE, new Hashtable(), notifyInfo);
    }
    
    // Returns Map of entityId -> RelationInfo
    public Map queryRelations (String fromId)
    {
        return (Hashtable)_relationTable.get(fromId);
    }

    public Map getRelations()
    {
        return (Hashtable) _relationTable;
    }

    public void reportProducerChannel(String producerId, String channelId, String infoObjectType)
    {
        Hashtable update = new Hashtable();
        update.put(PRODUCER_ID, producerId);
        update.put(producerId, channelId);
        update.put(channelId, infoObjectType);
        
        notify(NEW_PRODUCER_CHANNEL_UPDATE, new Hashtable(), update);
    }
    
    public void reportConsumerChannel(String consumerId, String channelId, String infoObjectType) 
    {
        Hashtable update = new Hashtable();
        update.put(CONSUMER_ID, consumerId);
        update.put(consumerId, channelId);
        update.put(channelId, infoObjectType);
        
        notify(NEW_CONSUER_CHANNEL_UPDATE, new Hashtable(), update);
    }
    
    public void registerObserver(String updateType, Object updateCondition, KAoSObserver observer) 
    {
        _observableImpl.registerObserver(updateType,updateCondition,observer);
    }

    public void deregisterObserver(String updateType, Object updateCondition, KAoSObserver observer) 
    {
        _observableImpl.deregisterObserver(updateType,updateCondition,observer);
    }

    public void notify(String updateType, Object updateCondition, Object update) 
    {
        _observableImpl.notify(updateType,updateCondition,update);
    }
}
