/**
 * Last Modified by: $Author: auszok $
 * On: $Date: 2007/04/16 18:49:55 $
 */
 
package coi.semanticTranslator;

/**
 * The interface of the service translating metainformation
 */
public interface SemanticTranslationService 
{
    public static final String SEMANTIC_TRANSLATION_SERVICE_URL = "http://ontology.ihmc.us/SemanticTranslation.owl#SemanticTranslationService";
    
    /**
     * Method adding SemanticTranslationInfo to the servcie repository
     * 
     * @param newSemanticTranslationInfo - new translation info
     */
    public void addTranslationInfo(SemanticTranslationInfo newSemanticTranslationInfo);
    
    /**
     * Method checking if there is translation from the givne type
     * 
     * @param _sourceTypeURL          - the url of the schema class of the translated metainformation
     * 
     * @return the url of the schema class being the destination for the translated metainformation
     */
    public String checkIfTranslationNecessary(String _sourceTypeURL);
    
    /**
     * Method translating metainformation from our semantic format to another
     * 
     * @param originalMetainformation - the original metainformation
     * @param _sourceTypeURL          - the url of the schema class of the translated metainformation
     * @param _destinationTypeURL;    - the url of the schema class being the destination for the translated metainformation
     * @param _sourceDataId           - id of the source instance data
     * @param _destinationDataId      - id of the destination instance data
     * 
     * @return translated metainformation
     */
    public String translateMetaInformation(String originalMetainformation, 
            String _sourceTypeURL, String _destinationTypeURL, String _sourceDataId, String _destinationDataId);
}
