package coi.semanticTranslator;

import java.util.Hashtable;
import java.util.Vector;
import java.io.StringReader;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QuerySolutionMap;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.ontology.OntModelSpec;

import kaos.core.csi.extension.KAoSExtensionComponentImpl;

public class SemanticTranslationServiceImpl  extends KAoSExtensionComponentImpl implements SemanticTranslationService
{
    private Hashtable _myTranslationTable = new Hashtable();
    
    public SemanticTranslationServiceImpl() 
    {
		super(); 
	}

	public SemanticTranslationServiceImpl(String arg0, Vector arg1) 
    {
		super(arg0, arg1);
	}

	public SemanticTranslationServiceImpl(String arg0, String arg1) 
    {
		super(arg0, arg1);
	}
    
    public void addTranslationInfo(SemanticTranslationInfo newSemanticTranslationInfo) 
    {
        if (newSemanticTranslationInfo.sourceTypeURL == null)
            return;
        
        if (newSemanticTranslationInfo.destinationTypeURL == null)
            return;
        
        if ( ! _myTranslationTable.containsKey(newSemanticTranslationInfo.sourceTypeURL))
            // Add new hashtable
            _myTranslationTable.put(newSemanticTranslationInfo.sourceTypeURL, new Hashtable());

       Hashtable sourceTypeURLTranslations = (Hashtable)_myTranslationTable.get(newSemanticTranslationInfo.sourceTypeURL);
       sourceTypeURLTranslations.put(newSemanticTranslationInfo.destinationTypeURL, newSemanticTranslationInfo);
    }

    public String translateMetaInformation(String originalMetainformation, 
            String _sourceTypeURL, String _destinationTypeURL, String _sourceDataId, String _destinationDataId) 
    {
        // Get query string from the local translation database
        Hashtable sourceTypeURLTranslations = (Hashtable)_myTranslationTable.get(_sourceTypeURL);
        
        if (sourceTypeURLTranslations == null)
            return null; // TODO: Throw exception
        
        String queryString = (String)sourceTypeURLTranslations.get(_destinationTypeURL);
        
        if (queryString == null)
            return null; // TODO: Throw exception
        
        Query query = QueryFactory.create(queryString);
        
        // Construct OntModel for originalMetainformation
        OntModelSpec oms = new OntModelSpec(OntModelSpec.OWL_MEM); // TODO: use database to store model
        Model originalMetainformationModel = ModelFactory.createOntologyModel(oms, null);
        StringReader originalMetainformationReader = new StringReader(originalMetainformation);
        originalMetainformationModel.read(originalMetainformationReader, "");

        // Construct the mapping of the variable inside query
        QuerySolutionMap queryBinding = new QuerySolutionMap();
        RDFNode sourceDataIdNode = originalMetainformationModel.createResource(_sourceDataId);
        RDFNode destinationDataIdNode = originalMetainformationModel.createResource(_destinationDataId);
        queryBinding.add(SemanticTranslationInfo.SOURCE_DATA_ID, sourceDataIdNode);
        queryBinding.add(SemanticTranslationInfo.DESTINATION_DATA_ID, destinationDataIdNode);
        
        // Perform the translation through query
        QueryExecution qexec = QueryExecutionFactory.create(query, originalMetainformationModel, queryBinding);
        
        Model resultModel = qexec.execConstruct() ;
        qexec.close() ;

        // Return the new metainformation
        return resultModel.toString();
    }

    public String checkIfTranslationNecessary(String _sourceTypeURL) 
    {
        if ( ! _myTranslationTable.containsKey(_sourceTypeURL))
            return null;
        
        Hashtable sourceTypeURLTranslations = (Hashtable)_myTranslationTable.get(_sourceTypeURL);
        
        return (String)sourceTypeURLTranslations.keySet().toArray()[0];
    }
}